library(reticulate)
library(nichenetr)
library(Seurat) # please update to Seurat V4
library(tidyverse)
library(circlize)
library(harmony)
library(ggplot2)
library(scales)
library(clusterProfiler)
library(xlsx)
library(BiocManager)
library(remotes)
library(devtools)
library(patchwork)
library(Matrix)
library(stringr)
library(Seurat)
library(purrr)
library(dplyr)
library(tibble)
library(ggplot2)
library(tidyverse)
library(cowplot)
library(patchwork)
library(ComplexHeatmap)
library(VennDiagram) 
library(stringi)

library(hrbrthemes)
library(tm)
library(proustr)
# co-expression network analysis packages:
library(WGCNA)
library(hdWGCNA)

# gene enrichment packages
library(enrichR)
library(GeneOverlap)
theme_set(theme_cowplot())

# :: DotPlots ----
# Color setting
my_color <- c('yellow', 'red')

# dotplot scale by size
dot <- function(obj, gene, title){
  DotPlot(obj, features = gene, dot.scale = 10, cols = my_color, scale.by = "size") +
    RotatedAxis() +
    labs(title = title) +
    theme(plot.title = element_text(hjust = 0.5))  +
    theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=0.5))
}


# dotplot scale by radius
dot01 <- function(obj, gene, title){
  DotPlot(obj, features = gene, dot.scale = 10, cols = my_color, scale.by = "radius") +
    RotatedAxis() +
    labs(title = title) +
    theme(plot.title = element_text(hjust = 0.5))  +
    theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=0.5))
}

# This analysis is about PiChip
# The reading depth is 200K
# Analysis starting date is as follows 
Sys.time()  # [1] "2022-01-30 20:51:37 EST"

# 1. Setting ----
# work on file name with complementary base and reverse its orders

# list up the file names
file <- dir(path = "data")
View(table(file))

# select only seq data file names
file <- file[2:41]
file
view(table(file))

library(Hmisc)

# Matching barcodes to sample names (word file)
matching <- Cs(P1.H10_MT_CTCTCTAT,
               P1.G10_MT_TATCCTCT,
               P1.H10_T_GTAAGGAG,
               P1.G10_T_ACTGCATA,
               P1.H10_M_AAGGAGTA,
               P1.G10_M_CTAAGCCT,
               P1.H10_CGTCTAAT,
               P1.G10_TCTCTCCG,
               P4.H10_MT_CTCTCTAT,
               P4.G10_MT_TATCCTCT,
               P4.H10_T_GTAAGGAG,
               P4.G10_T_ACTGCATA,
               P4.H10_M_AAGGAGTA,
               P4.G10_M_CTAAGCCT,
               P4.H10_GAGCCTTA,
               P4.G10_TCTCTCCG,
               P5.H10_MT_CTCTCTAT,
               P5.G10_MT_TATCCTCT,
               P5.H10_T_GTAAGGAG,
               P5.G10_T_ACTGCATA,
               P5.H10_M_AAGGAGTA,
               P5.G10_M_CTAAGCCT,
               P5.H10_GAGCCTTA,
               P5.G10_TCTCTCCG,
               P2.G10_MT_TATCCTCT,
               P2.G10_M_CTAAGCCT,
               P2.H10_MT_CTCTCTAT,
               P2.H10_CGTCTAAT,
               P2.G10_TCTCTCCG,
               P2.H10_T_GTAAGGAG,
               P2.H10_M_AAGGAGTA,
               P2.G10_T_ACTGCATA,
               P3.H10_MT_CTCTCTAT,
               P3.G10_MT_TATCCTCT,
               P3.H10_T_GTAAGGAG,
               P3.G10_T_ACTGCATA,
               P3.H10_M_AAGGAGTA,
               P3.G10_M_CTAAGCCT,
               P3.H10_GAGCCTTA,
               P3.G10_TCTCTCCG)

names(matching) <- substring(matching, 1, nchar(matching)-9)
matching
# extract only the last 8 characters (barcode)
library(stringr)

barcode.list <- lapply(file, FUN = function(x){
  str_sub(x, start = -10L, end = -1L)
})


# reverse the barcodes
barcodes_rv <- lapply(barcode.list, stri_reverse)
unlist(barcode.list)
unlist(barcodes_rv)


# converting function
convert <- function(x){
  chartr("ATGC","TACG",x)
}

# switch each letter to the mapped complementary base
converted_barcodes <- lapply(barcodes_rv, convert)
unlist(converted_barcodes)
# [1] "TATCCTCT-1" "CTAAGCCT-1" "CTCTCTAT-1" "CGTCTAAT-1" "TCTCTCCG-1" "GTAAGGAG-1" "AAGGAGTA-1" "ACTGCATA-1" "CCTAGAGT-4" "TTCTAGCT-4" "AAGGCTAT-4"
# [12] "TCGACTAG-4" "CTATTAAG-4" "GAGCCTTA-4" "TTATGCGA-4" "GCGTAAGA-4" "CCTAGAGT-5" "TTCTAGCT-5" "AAGGCTAT-5" "TCGACTAG-5" "CTATTAAG-5" "GAGCCTTA-5"
# [23] "TTATGCGA-5" "GCGTAAGA-5" "TATCCTCT-2" "CTAAGCCT-2" "CTCTCTAT-2" "CGTCTAAT-2" "TCTCTCCG-2" "GTAAGGAG-2" "AAGGAGTA-2" "ACTGCATA-2" "CCTAGAGT-3"
# [34] "TTCTAGCT-3" "AAGGCTAT-3" "TCGACTAG-3" "CTATTAAG-3" "GAGCCTTA-3" "TTATGCGA-3" "GCGTAAGA-3"



barcode.df <- data.frame("bc_orig" = file,
                         "bc_extracted" = unlist(barcode.list),
                         "bc_matched" = unlist(converted_barcodes),
                         "sample" = c("P1.G10_MT_TATCCTCT", 
                                      "P1.G10_M_CTAAGCCT",
                                      "P1.H10_MT_CTCTCTAT", 
                                      "P1.H10_CGTCTAAT", 
                                      "P1.G10_TCTCTCCG",
                                      "P1.H10_T_GTAAGGAG",
                                      "P1.H10_M_AAGGAGTA",  
                                      "P1.G10_T_ACTGCATA",
                                      "P4.H10_T_CCTAGAGT",
                                      "P4.G10_MT_TTCTAGCT",
                                      "P4.G10_M_AAGGCTAT",
                                      "P4.H10_MT_TCGACTAG",
                                      "P4.H10_M_CTATTAAG",
                                      "P4.H10_GAGCCTTA",
                                      "P4.G10_TTATGCGA",
                                      "P4.G10_T_GCGTAAGA",
                                      "P5.H10_T_CCTAGAGT",
                                      "P5.G10_MT_TTCTAGCT",
                                      "P5.G10_M_AAGGCTAT",
                                      "P5.H10_MT_TCGACTAG",
                                      "P5.H10_M_CTATTAAG",
                                      "P5.H10_GAGCCTTA",
                                      "P5.G10_TTATGCGA",
                                      "P5.G10_T_GCGTAAGA",
                                      "P2.G10_MT_TATCCTCT",
                                      "P2.G10_M_CTAAGCCT",
                                      "P2.H10_MT_CTCTCTAT",
                                      "P2.H10_CGTCTAAT",
                                      "P2.G10_TCTCTCCG",
                                      "P2.H10_T_GTAAGGAG",
                                      "P2.H10_M_AAGGAGTA",
                                      "P2.G10_T_ACTGCATA",
                                      "P3.H10_T_CCTAGAGT",
                                      "P3.G10_MT_TTCTAGCT",
                                      "P3.G10_M_AAGGCTAT",
                                      "P3.H10_MT_TCGACTAG",
                                      "P3.H10_M_CTATTAAG",
                                      "P3.H10_GAGCCTTA",
                                      "P3.G10_TTATGCGA",
                                      "P3.G10_T_GCGTAAGA"))


# PiChip.01.02
# 1. h10.mt     CTCTCTAT
# 2. g10.mt     TATCCTCT
# 3. h10.t     GTAAGGAG
# 4. g10.t     ACTGCATA
# 5. h10.m     AAGGAGTA
# 6. g10.m     CTAAGCCT
# 7. h10     CGTCTAAT
# 8. g10     TCTCTCCG

# PiChip.03.04.05
# 1. H10-MT 	TCGACTAG
# 2. G10-MT 	TTCTAGCT
# 3. H10-T		CCTAGAGT
# 4. G10-T 		GCGTAAGA
# 5. H10-M 	CTATTAAG
# 6. G10-M 	AAGGCTAT
# 7. H10 		GAGCCTTA
# 8. G10 		TTATGCGA
View(barcode.df)
barcode.df

#               bc_orig bc_extracted bc_matched             sample
# 1  PiChip_S1-AGAGGATA   1-AGAGGATA TATCCTCT-1 P1.G10_MT_TATCCTCT
# 2  PiChip_S1-AGGCTTAG   1-AGGCTTAG CTAAGCCT-1  P1.G10_M_CTAAGCCT
# 3  PiChip_S1-ATAGAGAG   1-ATAGAGAG CTCTCTAT-1 P1.H10_MT_CTCTCTAT
# 4  PiChip_S1-ATTAGACG   1-ATTAGACG CGTCTAAT-1    P1.H10_CGTCTAAT
# 5  PiChip_S1-CGGAGAGA   1-CGGAGAGA TCTCTCCG-1    P1.G10_TCTCTCCG
# 6  PiChip_S1-CTCCTTAC   1-CTCCTTAC GTAAGGAG-1  P1.H10_T_GTAAGGAG
# 7  PiChip_S1-TACTCCTT   1-TACTCCTT AAGGAGTA-1  P1.H10_M_AAGGAGTA
# 8  PiChip_S1-TATGCAGT   1-TATGCAGT ACTGCATA-1  P1.G10_T_ACTGCATA
# 9   PiChip04-ACTCTAGG   4-ACTCTAGG CCTAGAGT-4  P4.H10_T_CCTAGAGT
# 10  PiChip04-AGCTAGAA   4-AGCTAGAA TTCTAGCT-4 P4.G10_MT_TTCTAGCT
# 11  PiChip04-ATAGCCTT   4-ATAGCCTT AAGGCTAT-4  P4.G10_M_AAGGCTAT
# 12  PiChip04-CTAGTCGA   4-CTAGTCGA TCGACTAG-4 P4.H10_MT_TCGACTAG
# 13  PiChip04-CTTAATAG   4-CTTAATAG CTATTAAG-4  P4.H10_M_CTATTAAG
# 14  PiChip04-TAAGGCTC   4-TAAGGCTC GAGCCTTA-4    P4.H10_GAGCCTTA
# 15  PiChip04-TCGCATAA   4-TCGCATAA TTATGCGA-4    P4.G10_TTATGCGA
# 16  PiChip04-TCTTACGC   4-TCTTACGC GCGTAAGA-4  P4.G10_T_GCGTAAGA
# 17  PiChip05-ACTCTAGG   5-ACTCTAGG CCTAGAGT-5  P5.H10_T_CCTAGAGT
# 18  PiChip05-AGCTAGAA   5-AGCTAGAA TTCTAGCT-5 P5.G10_MT_TTCTAGCT
# 19  PiChip05-ATAGCCTT   5-ATAGCCTT AAGGCTAT-5  P5.G10_M_AAGGCTAT
# 20  PiChip05-CTAGTCGA   5-CTAGTCGA TCGACTAG-5 P5.H10_MT_TCGACTAG
# 21  PiChip05-CTTAATAG   5-CTTAATAG CTATTAAG-5  P5.H10_M_CTATTAAG
# 22  PiChip05-TAAGGCTC   5-TAAGGCTC GAGCCTTA-5    P5.H10_GAGCCTTA
# 23  PiChip05-TCGCATAA   5-TCGCATAA TTATGCGA-5    P5.G10_TTATGCGA
# 24  PiChip05-TCTTACGC   5-TCTTACGC GCGTAAGA-5  P5.G10_T_GCGTAAGA
# 25   PiChip2-AGAGGATA   2-AGAGGATA TATCCTCT-2 P2.G10_MT_TATCCTCT
# 26   PiChip2-AGGCTTAG   2-AGGCTTAG CTAAGCCT-2  P2.G10_M_CTAAGCCT
# 27   PiChip2-ATAGAGAG   2-ATAGAGAG CTCTCTAT-2 P2.H10_MT_CTCTCTAT
# 28   PiChip2-ATTAGACG   2-ATTAGACG CGTCTAAT-2    P2.H10_CGTCTAAT
# 29   PiChip2-CGGAGAGA   2-CGGAGAGA TCTCTCCG-2    P2.G10_TCTCTCCG
# 30   PiChip2-CTCCTTAC   2-CTCCTTAC GTAAGGAG-2  P2.H10_T_GTAAGGAG
# 31   PiChip2-TACTCCTT   2-TACTCCTT AAGGAGTA-2  P2.H10_M_AAGGAGTA
# 32   PiChip2-TATGCAGT   2-TATGCAGT ACTGCATA-2  P2.G10_T_ACTGCATA
# 33   PiChip3-ACTCTAGG   3-ACTCTAGG CCTAGAGT-3  P3.H10_T_CCTAGAGT
# 34   PiChip3-AGCTAGAA   3-AGCTAGAA TTCTAGCT-3 P3.G10_MT_TTCTAGCT
# 35   PiChip3-ATAGCCTT   3-ATAGCCTT AAGGCTAT-3  P3.G10_M_AAGGCTAT
# 36   PiChip3-CTAGTCGA   3-CTAGTCGA TCGACTAG-3 P3.H10_MT_TCGACTAG
# 37   PiChip3-CTTAATAG   3-CTTAATAG CTATTAAG-3  P3.H10_M_CTATTAAG
# 38   PiChip3-TAAGGCTC   3-TAAGGCTC GAGCCTTA-3    P3.H10_GAGCCTTA
# 39   PiChip3-TCGCATAA   3-TCGCATAA TTATGCGA-3    P3.G10_TTATGCGA
# 40   PiChip3-TCTTACGC   3-TCTTACGC GCGTAAGA-3  P3.G10_T_GCGTAAGA




# 2. Data read-in ----

# Create each individual Seurat object for every sample
library(Seurat)
library(Matrix)
read.in <- function(x){read.table(x, as.is = T, header = F)[, 1]}

# make a list
data.list <- list()
cn.list <- list()
rn.list <- list()

for (i in file){
  # read count matrix
  data.list[[str_sub(i, -10, -1)]] <- readMM(paste0("data/", i, "/", i, ".mtx"))
  
  # read in colnames
  cn.list[[str_sub(i, -10, -1)]] <- read.in(paste0("data/", i, "/", i, ".mtx.colnames"))
  
  # read in rownames
  rn.list[[str_sub(i, -10, -1)]] <- read.in(paste0("data/", i, "/", i, ".mtx.rownames"))
  
}

# sanity check
identical(names(data.list), names(cn.list))
identical(names(data.list), names(rn.list))

# Rename list elements
names(data.list) <- paste0(barcode.df[, "sample"], "_", names(data.list))
names(cn.list) <- paste0(barcode.df[, "sample"], "_", names(cn.list))
names(rn.list) <- paste0(barcode.df[, "sample"], "_", names(rn.list))


# sanity check
identical(names(data.list), names(cn.list))
identical(names(data.list), names(rn.list))


# Save permenant
# saveRDS(data.list, file = "objects/read_in_data/data.list.rds")
# saveRDS(cn.list, file = "objects/read_in_data/cn.list.rds")
# saveRDS(rn.list, file = "objects/read_in_data/rn.list.rds")

# Memory
library(disk.frame)
setup_disk.frame()
options(future.globals.maxSize = Inf)

# :: Gene annotation ----
# getBM
library(biomaRt)
listEnsembl()           # current ver: 105
listEnsemblArchives()    

# Retrieve BioMart
# ensembl <- biomaRt::useMart(biomart = "ensembl", dataset = "hsapiens_gene_ensembl",host = "www.ensembl.org")
# saveRDS(ensembl, file = "objects/biomart/ensembl_v105.rds")
ensembl <- readRDS( file = "objects/biomart/ensembl_v104.rds")

# check if all the rownames are identical
for(i in 2:length(rn.list)){
  print(identical(rn.list[[1]], rn.list[[i]]))
}

# All TRUE

# hgnc symbols matching with rownames
# bm <- biomaRt::getBM(attributes=c("ensembl_gene_id", "hgnc_symbol"),
#                      filters = "ensembl_gene_id",
#                      values = rn.list[[1]],
#                      mart= ensembl)

bm <- readRDS( file = "objects/biomart/bm.rds")
# BioMart cleaning
# 01. empty symbol
ids <- which(nchar(bm$hgnc_symbol)==0)
length(ids)     # 18169
bm <- bm[-ids,]

# 02. duplicated
ids.dup <- which(duplicated(bm$hgnc_symbol))
length(ids.dup)     # 17
bm <- bm[-ids.dup,]

# 03. final indices of ensembl ids (good.ids will be used for all rownames)
good.ids <- match(bm$ensembl_gene_id, rn.list[[1]])

# check
identical(bm$ensembl_gene_id, rn.list[[1]][good.ids])        # TRUE


# save cleaned bm permanent (saved on 01/31/2022)
# saveRDS(object = bm, file = "objects/biomart/bm.rds")
# bm <- readRDS("objects/biomart/bm.rds")


# _ Assembly ----
## 01. Select only rows that are good.ids in each dataset
## 02. Assemble rownames: rownames are hgnc column of the cleaned bm
## 03. Assemble colnames


# Assembly function
cleaning <- function(x){
  x <- x[good.ids, ]
}

good.data.list <- list()
good.data.list <- lapply(data.list, cleaning)  # good.data.list is the list of data with clean row names (no NAs, no duplicates in hgnc symbol)


# rownames
for(i in 1:length(good.data.list)){
  rownames(good.data.list[[i]]) <- bm$hgnc_symbol
}

for(i in 1:length(good.data.list)){
  colnames(good.data.list[[i]]) <- cn.list[[i]]
}

# check
head(good.data.list[[1]])
head(good.data.list[[1]][, 1:10])
head(good.data.list[[2]][, 1:10])


View(names(good.data.list))

# Check: no change in column numbers after cleaning. cleaning was only for rows.
identical(unlist(lapply(good.data.list, ncol)), unlist(lapply(data.list, ncol)))


# save permanent
# saveRDS(good.data.list, file = "objects/good.data.list.rds")
# good.data.list <- readRDS(file = "objects/good.data.list.rds")


# 3. Seurat object ----
seurat.obj <- function(x, c, f){CreateSeuratObject(counts = x, 
                                                   project = "PiChip", 
                                                   min.cells = c, min.features = f)}

# seurat.list ----
seurat.list <- list()
seurat.list <- lapply(good.data.list, seurat.obj, c = 3, f = 200)



# naming seurat.list
names(seurat.list)
# [1] "P1.G10_MT_TATCCTCT_1-AGAGGATA" "P1.G10_M_CTAAGCCT_1-AGGCTTAG"  "P1.H10_MT_CTCTCTAT_1-ATAGAGAG" "P1.H10_CGTCTAAT_1-ATTAGACG"    "P1.G10_TCTCTCCG_1-CGGAGAGA"   
# [6] "P1.H10_T_GTAAGGAG_1-CTCCTTAC"  "P1.H10_M_AAGGAGTA_1-TACTCCTT"  "P1.G10_T_ACTGCATA_1-TATGCAGT"  "P4.H10_T_CCTAGAGT_4-ACTCTAGG"  "P4.G10_MT_TTCTAGCT_4-AGCTAGAA"
# [11] "P4.G10_M_AAGGCTAT_4-ATAGCCTT"  "P4.H10_MT_TCGACTAG_4-CTAGTCGA" "P4.H10_M_CTATTAAG_4-CTTAATAG"  "P4.H10_GAGCCTTA_4-TAAGGCTC"    "P4.G10_TTATGCGA_4-TCGCATAA"   
# [16] "P4.G10_T_GCGTAAGA_4-TCTTACGC"  "P5.H10_T_CCTAGAGT_5-ACTCTAGG"  "P5.G10_MT_TTCTAGCT_5-AGCTAGAA" "P5.G10_M_AAGGCTAT_5-ATAGCCTT"  "P5.H10_MT_TCGACTAG_5-CTAGTCGA"
# [21] "P5.H10_M_CTATTAAG_5-CTTAATAG"  "P5.H10_GAGCCTTA_5-TAAGGCTC"    "P5.G10_TTATGCGA_5-TCGCATAA"    "P5.G10_T_GCGTAAGA_5-TCTTACGC"  "P2.G10_MT_TATCCTCT_2-AGAGGATA"
# [26] "P2.G10_M_CTAAGCCT_2-AGGCTTAG"  "P2.H10_MT_CTCTCTAT_2-ATAGAGAG" "P2.H10_CGTCTAAT_2-ATTAGACG"    "P2.G10_TCTCTCCG_2-CGGAGAGA"    "P2.H10_T_GTAAGGAG_2-CTCCTTAC" 
# [31] "P2.H10_M_AAGGAGTA_2-TACTCCTT"  "P2.G10_T_ACTGCATA_2-TATGCAGT"  "P3.H10_T_CCTAGAGT_3-ACTCTAGG"  "P3.G10_MT_TTCTAGCT_3-AGCTAGAA" "P3.G10_M_AAGGCTAT_3-ATAGCCTT" 
# [36] "P3.H10_MT_TCGACTAG_3-CTAGTCGA" "P3.H10_M_CTATTAAG_3-CTTAATAG"  "P3.H10_GAGCCTTA_3-TAAGGCTC"    "P3.G10_TTATGCGA_3-TCGCATAA"    "P3.G10_T_GCGTAAGA_3-TCTTACGC" 

# Remove the last 18 characters
names(seurat.list) <- substr(names(seurat.list), 1, nchar(names(seurat.list))-20)

# Re-arrange seurat list
seurat.list <- seurat.list[c("P1.G10_MT", "P1.G10_M",  "P1.G10_T", "P1.G10", "P1.H10_MT",  "P1.H10_M", "P1.H10_T", "P1.H10",
                             "P2.G10_MT", "P2.G10_M",  "P2.G10_T", "P2.G10", "P2.H10_MT",  "P2.H10_M", "P2.H10_T", "P2.H10",
                             "P3.G10_MT", "P3.G10_M",  "P3.G10_T", "P2.G10", "P3.H10_MT",  "P3.H10_M", "P3.H10_T", "P3.H10",
                             "P4.G10_MT", "P4.G10_M",  "P4.G10_T", "P4.G10", "P4.H10_MT",  "P4.H10_M", "P4.H10_T", "P4.H10",
                             "P5.G10_MT", "P5.G10_M",  "P5.G10_T", "P5.G10", "P5.H10_MT",  "P5.H10_M", "P5.H10_T", "P5.H10")]
              

# cell counts 
View(unlist(lapply(seurat.list, ncol)))
View(table(names(seurat.list)))

# metadata
# mitocondrial content metadata
for(i in 1:length(seurat.list)){
  seurat.list[[i]]$percent.mt <- PercentageFeatureSet(seurat.list[[i]], pattern = "^MT-")
}


# sample name
for(i in 1:length(seurat.list)){
  seurat.list[[i]]$each <- names(seurat.list[i])
}


# reading depth
for(i in 1:length(seurat.list)){
  seurat.list[[i]]$read_depth <- "200K"
}
# Sample 
for(i in c(1)){seurat.list[[i]]$sample <- "sample1"}
for(i in c(2)){seurat.list[[i]]$sample <- "sample1"}
for(i in c(3)){seurat.list[[i]]$sample <- "sample1"}
for(i in c(4)){seurat.list[[i]]$sample <- "sample1"}
for(i in c(5)){seurat.list[[i]]$sample <- "sample1"}
for(i in c(6)){seurat.list[[i]]$sample <- "sample1"}
for(i in c(7)){seurat.list[[i]]$sample <- "sample1"}
for(i in c(8)){seurat.list[[i]]$sample <- "sample1"}

for(i in c(9)){seurat.list[[i]]$sample <- "sample2"}
for(i in c(10)){seurat.list[[i]]$sample <- "sample2"}
for(i in c(11)){seurat.list[[i]]$sample <- "sample2"}
for(i in c(12)){seurat.list[[i]]$sample <- "sample2"}
for(i in c(13)){seurat.list[[i]]$sample <- "sample2"}
for(i in c(14)){seurat.list[[i]]$sample <- "sample2"}
for(i in c(15)){seurat.list[[i]]$sample <- "sample2"}
for(i in c(16)){seurat.list[[i]]$sample <- "sample2"}

for(i in c(17)){seurat.list[[i]]$sample <- "sample3"}
for(i in c(18)){seurat.list[[i]]$sample <- "sample3"}
for(i in c(19)){seurat.list[[i]]$sample <- "sample3"}
for(i in c(20)){seurat.list[[i]]$sample <- "sample3"}
for(i in c(21)){seurat.list[[i]]$sample <- "sample3"}
for(i in c(22)){seurat.list[[i]]$sample <- "sample3"}
for(i in c(23)){seurat.list[[i]]$sample <- "sample3"}
for(i in c(24)){seurat.list[[i]]$sample <- "sample3"}

for(i in c(25)){seurat.list[[i]]$sample <- "sample4"}
for(i in c(26)){seurat.list[[i]]$sample <- "sample4"}
for(i in c(27)){seurat.list[[i]]$sample <- "sample4"}
for(i in c(28)){seurat.list[[i]]$sample <- "sample4"}
for(i in c(29)){seurat.list[[i]]$sample <- "sample4"}
for(i in c(30)){seurat.list[[i]]$sample <- "sample4"}
for(i in c(31)){seurat.list[[i]]$sample <- "sample4"}
for(i in c(32)){seurat.list[[i]]$sample <- "sample4"}

for(i in c(33)){seurat.list[[i]]$sample <- "sample5"}
for(i in c(34)){seurat.list[[i]]$sample <- "sample5"}
for(i in c(35)){seurat.list[[i]]$sample <- "sample5"}
for(i in c(36)){seurat.list[[i]]$sample <- "sample5"}
for(i in c(37)){seurat.list[[i]]$sample <- "sample5"}
for(i in c(38)){seurat.list[[i]]$sample <- "sample5"}
for(i in c(39)){seurat.list[[i]]$sample <- "sample5"}
for(i in c(40)){seurat.list[[i]]$sample <- "sample5"}




# stim
for(i in c(1,2,3,4,
           9,10,11,12,
           17,18,19,20,
           25,26,27,28,
           33,34,35,36)){
  seurat.list[[i]]$stim <- "CTRL"
}
for(i in c(5,6,7,8,
           13,14,15,16,
           21,22,23,24,
           29,30,31,32,
           37,38,39,40)){
  seurat.list[[i]]$stim <- "AD"
}

# cell_addition
for(i in c(1,5,9,13,
           17,21, 25,29,
           33,37)){
  seurat.list[[i]]$mt <- "3D Microglia & T cells"
}
for(i in c(2,6, 10,14,
           18, 22, 26,30,
           34,38)){
  seurat.list[[i]]$mt <- "3D Microglia"
}
for(i in c(3, 7, 11, 15,
           19, 23, 27, 31, 
           35,39)){
  seurat.list[[i]]$mt <- "3D T cells"
}
for(i in c(4,8,12,16, 
           20,24,28,32,
           36,40)){
  seurat.list[[i]]$mt <- "3D"
}


# cell_addition_ad
for(i in c(1,9, 17,25, 33 )){
  seurat.list[[i]]$mt.ad <- "3D CTRL Microglia & T cells"
}
for(i in c(5,13, 21,29, 37 )){
  seurat.list[[i]]$mt.ad <- "3D AD Microglia & T cells"
}

for(i in c(2,10, 18,26, 34)){
  seurat.list[[i]]$mt.ad <- "3D CTRL Microglia"
}
for(i in c(6,14, 22,30, 38)){
  seurat.list[[i]]$mt.ad <- "3D AD Microglia"
}

for(i in c(3, 11, 19, 27, 35)){
  seurat.list[[i]]$mt.ad <- "3D CTRL T cells"
}
for(i in c(7,15, 23,31, 39)){
  seurat.list[[i]]$mt.ad <- "3D AD T cells"
}

for(i in c(4,12, 20,28, 36)){
  seurat.list[[i]]$mt.ad <- "3D CTRL"
}
for(i in c(8,16, 24,32, 40)){
  seurat.list[[i]]$mt.ad <- "3D AD"
}


# check metadata
lapply(seurat.list, FUN = function(x){
  head(x@meta.data)
})

# all looks good

# VlnPlot Function setting
violin <- function(seurat){
  VlnPlot(seurat,
          features = c("nFeature_RNA", "nCount_RNA", "percent.mt"), 
          ncol = 3, 
          pt.size = 0) 
}


# export
library(svglite)
for(i in 1:length(seurat.list)){
  svglite(filename = paste0("figures/QC/vln_", names(seurat.list[i]), ".svg"))
  
  print(violin(seurat.list[[i]]))
  
  dev.off()
}


# Quick check for gene count per sample
# Create a data frame (Median of gene count)
gene_per_sample <- data.frame("sample" = "",
                              "median gene counts" = "",
                              "mean gene counts" = "",
                              "min gene counts" = "",
                              "max gene counts" = "",
                              "cell counts" = "")

for(i in 1:length(seurat.list)){
  # 1st column is the sample name
  gene_per_sample[[i, 1]] <- names(seurat.list[i])
  
  # 2nd column is the median gene count
  gene_per_sample[[i, 2]] <- median(seurat.list[[i]]$nFeature_RNA)
  
  # 3rd column is the median gene count
  gene_per_sample[[i, 3]] <- round(mean(seurat.list[[i]]$nFeature_RNA), digits = 1)
  
  # 4th column is the median gene count
  gene_per_sample[[i, 4]] <- min(seurat.list[[i]]$nFeature_RNA)
  
  # 5th column is the median gene count
  gene_per_sample[[i, 5]] <- max(seurat.list[[i]]$nFeature_RNA)
  
  # 6th column is the cell count
  gene_per_sample[[i, 6]] <- ncol(seurat.list[[i]])
}

View(gene_per_sample)
#       sample median.gene.counts mean.gene.counts min.gene.counts max.gene.counts cell.counts
# 1  P1.G10_MT              325.5              641             197            6755         744
# 2   P1.G10_M                452            887.4             199            9192        3192
# 3   P1.G10_T                346            510.7             195            7940        1308
# 4     P1.G10              345.5            537.9             197            9006        2342
# 5  P1.H10_MT              657.5            891.7             198            5921         442
# 6   P1.H10_M                342              843             197            9678        2570
# 7   P1.H10_T              685.5            965.5             187            4786         390
# 8     P1.H10                287            525.6             197            9008        1457
# 9  P2.G10_MT                282            550.9             197            7833        1311
# 10  P2.G10_M                513              925             199           10051        3839
# 11  P2.G10_T              468.5            625.7             198            9541        1918
# 12    P2.G10                438            637.8             196           10457        2914
# 13 P2.H10_MT                595              927             199            6778         537
# 14  P2.H10_M                354              828             199            9893        2839
# 15  P2.H10_T              335.5            741.2             196            6711         942
# 16    P2.H10                333            543.1             198           10208        2186
# 17 P3.G10_MT                389            709.7             196            6368         559
# 18  P3.G10_M                447            885.5             199            9136        3869
# 19  P3.G10_T                406            665.5             190            4002         267
# 20    P2.G10                438            637.8             196           10457        2914
# 21 P3.H10_MT                377              536             193            4102         331
# 22  P3.H10_M              581.5           1258.3             199            7316         896
# 23  P3.H10_T                549            646.6             192            4831         258
# 24    P3.H10                391            629.9             198            4884         253
# 25 P4.G10_MT                361            706.8             198            6632         626
# 26  P4.G10_M                424            925.1             199            8027        2778
# 27  P4.G10_T              369.5            651.9             191            5012         396
# 28    P4.G10                307            516.1             198            6679        1764
# 29 P4.H10_MT                512            730.1             199            6713         451
# 30  P4.H10_M                468           1207.4             198            7733        1036
# 31  P4.H10_T              651.5            807.3             185            6300         314
# 32    P4.H10                351            674.1             196            6524         453
# 33 P5.G10_MT                322            673.9             197            6994         757
# 34  P5.G10_M                422            926.1             200            7944        2737
# 35  P5.G10_T                378            662.5             190            4736         358
# 36    P5.G10                302            512.4             198            6553        1676
# 37 P5.H10_MT              507.5            729.8             198            6678         446
# 38  P5.H10_M              527.5           1237.4             197            7560         960
# 39  P5.H10_T                637            785.5             191            6137         306
# 40    P5.H10              345.5            666.7             195            6361         436

## Compare with Tsai Human data
#       sample      median.gene.counts mean.gene.counts min.gene.counts max.gene.counts
# 1 Tsai_CTRL               1276           1860.4             201            7524
# 2   Tsai_AD                961           1530.4             201            7525
# :: QC Visualization ----
# Add number of genes per UMI
merged_seurat$log10GenesPerUMI <- log10(merged_seurat$nFeature_RNA) / log10(merged_seurat$nCount_RNA)

# extract metadata
metadata <- merged_seurat@meta.data

# add new columns to the metadata
# Add cell IDs to metadata
metadata$cells <- rownames(metadata)
head(metadata)

dim(metadata)   #  12467    10
str(metadata)

# Cell Counts by Sample
library(ggplot2)
library(tibble)
p1 <- metadata %>% 
  ggplot(aes(x=sample, fill=sample)) + 
  geom_bar(width=0.5) +
  theme_classic() +
  theme(axis.text.x = element_text(angle = 0, vjust = 0.5, hjust=0.5)) +
  theme(plot.title = element_text(hjust=0.5, face="bold")) +
  ggtitle("NCells")

svglite(filename = "figures/QC/cell_counts.svg", width = 8, height = 6)
p1
dev.off()


# Number of UMIs/transcripts per cell
p2 <- metadata %>% 
  ggplot(aes(color=sample, x=nCount_RNA, fill= sample)) + 
  geom_density(alpha = 0.2) + 
  scale_x_log10() + 
  theme_classic() +
  ylab("Cell density") +
  ggtitle("UMI per cell") +
  theme(plot.title = element_text(hjust=0.5, face="bold")) +
  geom_vline(xintercept = 300)


svglite(filename = "figures/QC/umi.svg", 
        width = 8, height = 6)
p2
dev.off()


# Detected genes per cell: distribution
p3 <- metadata %>% 
  ggplot(aes(color=sample, x=nFeature_RNA, fill= sample)) + 
  geom_density(alpha = 0.2) + 
  theme_classic() +
  scale_x_log10() + 
  geom_vline(xintercept = c(200, 500, 1000)) +
  ggtitle("Unique Genes per Cell") +
  theme(plot.title = element_text(hjust=0.5, face="bold"))

svglite(filename = "figures/QC/gene_count_distribution.svg", 
        width = 8, height = 6)
p3
dev.off()


# Detected genes per cell: distribution via boxplot
p4 <- metadata %>% 
  ggplot(aes(x=sample, y=log10(nFeature_RNA), fill=sample)) + 
  geom_boxplot() + 
  theme_classic() +
  theme(axis.text.x = element_text(angle = 0, vjust = 0.5, hjust=0.5)) +
  theme(plot.title = element_text(hjust=0.5, face="bold")) +
  ggtitle("nFeature_RNAs per Cell")

svglite(filename = "figures/QC/gene_count_logbox.svg", 
        width = 8, height = 6)
p4
dev.off()

p5 <- metadata %>% 
  ggplot(aes(x=sample, y=nFeature_RNA, fill=sample)) + 
  geom_boxplot() + 
  theme_classic() +
  theme(axis.text.x = element_text(angle = 0, vjust = 0.5, hjust=0.5)) +
  theme(plot.title = element_text(hjust=0.5, face="bold")) +
  ggtitle("nFeature_RNAs per Cell")


svglite(filename = "figures/QC/gene_count_box.svg", 
        width = 8, height = 6)
p5
dev.off()

# Number of genes per UMI
# Visualize the correlation between genes detected and number of UMIs and determine whether strong presence of cells with low numbers of genes/UMIs
p6 <- metadata %>% 
  ggplot(aes(x=nCount_RNA, y=nFeature_RNA, color=percent.mt)) + 
  geom_point() + 
  scale_colour_gradient(low = "gray90", high = "black") +
  stat_smooth(method=lm) +
  scale_x_log10() + 
  scale_y_log10() + 
  theme_classic() +
  geom_vline(xintercept = 300) +
  geom_hline(yintercept = 200) +
  facet_wrap(~sample)

svglite(filename = "figures/QC/umi_gene_mt.svg", 
        width = 8, height = 6)
p6
dev.off()


# Complexity of the gene expression: Detected genes per UMI
p7 <- metadata %>%
  ggplot(aes(x=log10GenesPerUMI, color = sample, fill= sample)) +
  geom_density(alpha = 0.2) +
  theme_classic() +
  geom_vline(xintercept = 0.8)

svglite(filename = "figures/QC/complexity.svg", 
        width = 8, height = 6)
p7
dev.off()


# ** Feb.02.2023 PiChip.Fig.4/5 ----
library(reticulate)
library(nichenetr)
library(Seurat) # please update to Seurat V4
library(tidyverse)
library(circlize)
library(harmony)
library(ggplot2)
library(scales)
library(clusterProfiler)
library(xlsx)
library(BiocManager)
library(remotes)
library(devtools)
library(patchwork)
library(Matrix)
library(stringr)
library(Seurat)
library(purrr)
library(dplyr)
library(tibble)
library(ggplot2)
library(tidyverse)
library(cowplot)
library(patchwork)
library(ComplexHeatmap)
library(VennDiagram) 
library(stringi)

library(hrbrthemes)
library(tm)
library(proustr)
# co-expression network analysis packages:
library(WGCNA)
library(hdWGCNA)

# gene enrichment packages
library(enrichR)
library(GeneOverlap)
theme_set(theme_cowplot())

# :: DotPlots ----
# Color setting
my_color <- c('yellow', 'red')

# dotplot scale by size
dot <- function(obj, gene, title){
  DotPlot(obj, features = gene, dot.scale = 10, cols = my_color, scale.by = "size") +
    RotatedAxis() +
    labs(title = title) +
    theme(plot.title = element_text(hjust = 0.5))  +
    theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=0.5))
}


# dotplot scale by radius
dot01 <- function(obj, gene, title){
  DotPlot(obj, features = gene, dot.scale = 10, cols = my_color, scale.by = "radius") +
    RotatedAxis() +
    labs(title = title) +
    theme(plot.title = element_text(hjust = 0.5))  +
    theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=0.5))
}
# saveRDS(seurat.list, file = "objects/seurat.list.ordered.Feb.02.2023.rds")
seurat.list <- readRDS(file = "objects/seurat.list.ordered.Feb.02.2023.rds")
# seurat.list <- seurat.list[c((-2:-4), (-6:-8), (-10:-12), (-14:-16), (-18:-20),(-22:-24), (-26:-28), (-30:-32), (-34:-36), (-38:-40))]
# seurat.list <- seurat.list[c(-2, -4, -6,-8, -10,-12, -14,-16, -18,-20,-22,-24, -26,-28, -30,-32, -34,-36, -38,-40)]

# 4-1. Merge seurat ----
seurat.list <- lapply(X = seurat.list, 
                      FUN = SCTransform, 
                      method = "glmGamPoi", 
                      vars.to.regress = "percent.mt")

var.features <- SelectIntegrationFeatures(object.list = seurat.list, nfeatures = 2000)

seurat.sct <- merge(x = seurat.list[[1]], 
                    y = seurat.list[2:length(seurat.list)], merge.data=TRUE, 
                    add.cell.ids = names(seurat.list))

seurat.sct <- subset(x = seurat.sct,
                     subset= (nFeature_RNA >= 200 & nFeature_RNA <= 9000) &
                       (percent.mt <= 30))

VariableFeatures(seurat.sct) <- var.features
filtered_seurat <- RunPCA(seurat.sct, npcs = 30, verbose = F)
filtered_seurat <- RunUMAP(filtered_seurat, reduction = "pca", dims = 1:30, verbose = F)

library(harmony)
filtered_seurat.harmony <- filtered_seurat %>% RunHarmony("each", plot_convergence = T)

DimPlot(object = filtered_seurat.harmony, reduction = "harmony", pt.size = 1, split.by = "mt.ad") + NoLegend()

filtered_seurat.harmony <- filtered_seurat.harmony %>%
  RunUMAP(reduction = "harmony", dims = 1:30) %>%
  FindNeighbors(reduction = "harmony", dims = 1:30) %>%
  FindClusters(resolution = c(0.4, 0.8, 1.2, 1.6))

names(filtered_seurat.harmony@meta.data)

Tri.res.list <- list(names(filtered_seurat.harmony@meta.data[13]),
                     names(filtered_seurat.harmony@meta.data[14]),
                     names(filtered_seurat.harmony@meta.data[15]),
                     names(filtered_seurat.harmony@meta.data[16]))

names(Tri.res.list) <- str_sub(names(filtered_seurat.harmony@meta.data[13:16]), start = 1L, end = -1L)

# :: View clusters
filtered_seurat.harmony$SCT_snn_res.0.4 <- factor(filtered_seurat.harmony$SCT_snn_res.0.4,
                                                  levels = 0:(length(unique(filtered_seurat.harmony$SCT_snn_res.0.4))-1))

filtered_seurat.harmony$SCT_snn_res.0.8 <- factor(filtered_seurat.harmony$SCT_snn_res.0.8,
                                                  levels = 0:(length(unique(filtered_seurat.harmony$SCT_snn_res.0.8))-1))

filtered_seurat.harmony$SCT_snn_res.1.2 <- factor(filtered_seurat.harmony$SCT_snn_res.1.2,
                                                  levels = 0:(length(unique(filtered_seurat.harmony$SCT_snn_res.1.2))-1))


filtered_seurat.harmony$SCT_snn_res.1.6 <- factor(filtered_seurat.harmony$SCT_snn_res.1.6,
                                                  levels = 0:(length(unique(filtered_seurat.harmony$SCT_snn_res.1.6))-1))

Idents(filtered_seurat.harmony) <- "SCT_snn_res.0.4"

DimPlot(filtered_seurat.harmony,  label = TRUE, pt.size = 0.8, label.size = 7) + NoLegend()
DimPlot(filtered_seurat.harmony,  label = TRUE, pt.size = 0.8, label.size = 7, split.by = "mt") + NoLegend()
DimPlot(filtered_seurat.harmony,  label = TRUE, pt.size = 0.8, label.size = 7, split.by = "mt.ad") + NoLegend()

table(filtered_seurat.harmony@active.ident, filtered_seurat.harmony@meta.data$mt.ad)
mt.ad.composition <- table(filtered_seurat.harmony@active.ident, filtered_seurat.harmony@meta.data$mt.ad)
write.xlsx(mt.ad.composition, 
           file = "results/mt.ad.composition.harmony.Jan.18.2023.xlsx")

saveRDS(filtered_seurat.harmony, file = "objects/Feb.18_filtered_seurat.harmony.alltogether.pc30.mt30.RNA200.rds")
filtered_seurat.harmony <- readRDS(file = "objects/Feb.18_filtered_seurat.harmony.alltogether.pc30.mt30.RNA200.rds")

# saveRDS(filtered_seurat.harmony, file = "objects/Feb.06_filtered_seurat.harmony.contains.T.mt20.RNA200.rds")
# filtered_seurat.harmony <- readRDS(file = "objects/Feb.06_filtered_seurat.harmony.contains.T.mt20.RNA200.rds")


DefaultAssay(filtered_seurat.harmony) <- "SCT"
Trimarkers <- c( "CD3D",  "SELL", "GIMAP5",  "GNLY", "NKG7", "CD8A", 
                 "GFAP",  "AQP4", "ID4", "AGT", "CSF1R",  "SPI1", "HLA-DRB1",
                  "WSB1", "KCNQ1OT1", "KCNJ6", "SYTL2",
                 "PLP1", "MBP", "MOBP","TOP2A", "MKI67")
dot(filtered_seurat.harmony , gene = Trimarkers, "Percent expression      Average gene expression")

filtered_seurat.harmony.find.all.markers <- FindAllMarkers(filtered_seurat.harmony, 
                                                             only.pos = TRUE, 
                                                             min.pct = 0.1,
                                                             assay = "RNA",
                                                             slot = "data", 
                                                             logfc.threshold = 0.1)
write.table(filtered_seurat.harmony.find.all.markers, file='results/filtered_seurat.harmonyalltogether.mt30.RNA200.find.all.markers.Feb.18.tsv', quote=FALSE, sep='\t', col.names = TRUE)

filtered_seurat.harmony.find.all.markers.top20 <- filtered_seurat.harmony.find.all.markers %>% 
  group_by(cluster) %>% 
  slice_max(n = 20, order_by = avg_log2FC)
view(filtered_seurat.harmony.find.all.markers.top20)


# "0" = "T cells", "1" = "Astrocyte","2" = "Microglia","3" = "Proliferative","4" = "Neuron","5" = "Neuron","6" = "Microglia","7" = "Microglia","8" = "Neuron","9" = "Neuron",
# "10" = "Neuron","11" = "Neuron","12" = "Astrocyte","13" = "Astrocyte","14" = "Astrocyte","15" = "Astrocyte","16" = "Astrocyte","17" = "Astrocyte","18" = "Astrocyte","19" = "Astrocyte","20" = "Astrocyte","21" = "Astrocyte","22" = "Astrocyte","23" = "Astrocyte","24" = "Microglia","25" = "T cells","26" = "na","27" = "Microglia")

filtered_seurat.harmony.name <- RenameIdents(filtered_seurat.harmony, 
                                                   "0" = "Neuron", 
                                                   "1" = "Astrocyte",
                                                   "2" = "Microglia",
                                                   "3" = "T cells",
                                                   "4" = "Microglia",
                                                   "5" = "Proliferative",
                                                   "6" = "Astrocyte",
                                                   "7" = "Astrocyte",
                                                   "8" = "Astrocyte",
                                                   "9" = "Neuron",
                                                   "10" = "Neuron",
                                                   "11" = "na",
                                                   "12" = "Astrocyte",
                                                   "13" = "Astrocyte",
                                                   "14" = "Astrocyte",
                                                   "15" = "Astrocyte",
                                                   "16" = "Microglia",
                                                   "17" = "Microglia")


pichip <- subset(filtered_seurat.harmony.name, idents = c("Astrocyte","Microglia","Neuron", "T cells", "Proliferative"))

table(pichip @active.ident, pichip @meta.data$mt)

DimPlot(pichip,  label = TRUE, pt.size = 0.8, label.size = 7) + NoLegend()
DimPlot(pichip,  label = TRUE, pt.size = 0.8, label.size = 7, split.by = "mt") + NoLegend()
DimPlot(pichip,  label = TRUE, pt.size = 0.8, label.size = 7, split.by = "mt.ad") + NoLegend()

DefaultAssay(pichip) <- "SCT"
Trimarkers <- c( "CD3D",  "SELL", "GIMAP5",  "GNLY", "NKG7", "CD8A", 
                 "GFAP",  "AQP4", "ID4", 
                 "CSF1R",  "SPI1", "HLA-DRB1",
                 "TOP2A", "MKI67",
                 "KCNQ1OT1", "KCNJ6", "SYTL2"
                 )
dot(pichip , gene = Trimarkers, "Percent expression      Average gene expression")

de <- function(x, id1, id2, assay){
  x <- FindMarkers(x,
                   ident.1 = id1,
                   ident.2 = id2,
                   test.use = "wilcox",
                   assay = assay,
                   slot = "data",
                   min.pct = 0.01,
                   logfc.threshold = 0.01)
  return(x)
}

# counting ----
de.rna.pichip <- list()
# 4-4. pichip.mt de AD.vs.CTRL ----
pichip.neu <- subset(pichip, idents = c("Neuron"))
table(pichip.neu@meta.data[["stim"]])
Idents(pichip.neu) <- "stim"

de.rna.pichip[["neu.AD.vs.CTRL"]] <- de(pichip.neu, 
                                        id1 = "AD", 
                                        id2 = "CTRL", 
                                        assay = "RNA")


pichip.ast <- subset(pichip, idents = c("Astrocyte"))
table(pichip.ast@meta.data[["stim"]])
Idents(pichip.ast) <- "stim"

de.rna.pichip[["ast.AD.vs.CTRL"]] <- de(pichip.ast, 
                                        id1 = "AD", 
                                        id2 = "CTRL", 
                                        assay = "RNA")

pichip.mic <- subset(pichip, idents = c("Microglia"))
table(pichip.mic@meta.data[["stim"]])
Idents(pichip.mic) <- "stim"

de.rna.pichip[["mic.AD.vs.CTRL"]] <- de(pichip.mic, 
                                        id1 = "AD", 
                                        id2 = "CTRL", 
                                        assay = "RNA")

pichip.t <- subset(pichip, idents = c("T cells"))
table(pichip.t@meta.data[["stim"]])
Idents(pichip.t) <- "stim"

de.rna.pichip[["t.AD.vs.CTRL"]] <- de(pichip.t, 
                                      id1 = "AD", 
                                      id2 = "CTRL", 
                                      assay = "RNA")


for(i in 1:length(de.rna.pichip)){
  write.csv(de.rna.pichip[[i]], 
            file = paste0("results/de_pichip_", 
                          names(de.rna.pichip[i]),
                          ".csv"),
            row.names = TRUE)
}
counting <- function(x){
  library(dplyr)
  p.filter <- x %>% filter(p_val_adj <= 0.2)
  positive.FC <- nrow(p.filter %>% filter(avg_log2FC > 0))
  negative.FC <- nrow(p.filter %>% filter(avg_log2FC < 0))
  print(paste0("pos = ", positive.FC, " ", "neg = ", negative.FC))
}

lapply(de.rna.pichip, counting)


# ** ReClustering ----
# ::T ReCluster (Jan.26) split.recluster ----
pichip.t <- subset(pichip, idents = c("T cells"))
table(pichip.t@meta.data[["mt.ad"]])

Idents(pichip.t) <- "mt.ad"
pichip.t.clustering <- subset(pichip.t, idents = c("3D AD T cells", "3D AD Microglia & T cells", "3D CTRL T cells", "3D CTRL Microglia & T cells"))
DimPlot(pichip.t.clustering)
table(pichip.t.clustering@meta.data[["mt.ad"]])

DefaultAssay(pichip.t.clustering) <- "RNA"
Idents(pichip.t.clustering) <- "mt.ad"

pichip.t.list <- SplitObject(pichip.t, split.by = "each")

pichip.t.list <- lapply(X = pichip.t.list, 
                          FUN = SCTransform, 
                          method = "glmGamPoi", 
                          vars.to.regress = "percent.mt")

var.features <- SelectIntegrationFeatures(object.list = pichip.t.list, nfeatures = 2000)

pichip.t.list.sct <- merge(x = pichip.t.list[[1]], 
                             y = pichip.t.list[2:length(pichip.t.list)], merge.data=TRUE, 
                             add.cell.ids = names(pichip.t.list))

pichip.t.list.sct <- subset(x = pichip.t.list.sct,
                              subset= (nFeature_RNA >= 200 & nFeature_RNA <= 9000) &
                                (percent.mt <= 10))

VariableFeatures(pichip.t.list.sct) <- var.features
pichip.t.list.sct <- RunPCA(pichip.t.list.sct, npcs = 30, verbose = F)
pichip.t.list.sct <- RunUMAP(pichip.t.list.sct, reduction = "pca", dims = 1:30, verbose = F)

pichip.t.list.sct_harmony <- pichip.t.list.sct %>% RunHarmony("mt.ad", assay.use = "SCT", plot_convergence = T)
DefaultAssay(pichip.t.list.sct_harmony) <- "SCT"
pichip.t.list.sct_harmony <- pichip.t.list.sct_harmony %>%
  RunUMAP(reduction = "harmony", dims = 1:30) %>%
  FindNeighbors(reduction = "harmony", dims = 1:30) %>%
  FindClusters(resolution = c(0.4, 0.8, 1.2, 1.6))


names(pichip.t.list.sct_harmony@meta.data)

Tri.res.list <- list(names(pichip.t.list.sct_harmony@meta.data[13]),
                     names(pichip.t.list.sct_harmony@meta.data[14]),
                     names(pichip.t.list.sct_harmony@meta.data[15]),
                     names(pichip.t.list.sct_harmony@meta.data[16]))

Tri.res.list
library(stringr)
names(Tri.res.list) <- str_sub(names(pichip.t.list.sct_harmony@meta.data[13:16]), start = 1L, end = -1L)



# :: View clusters
pichip.t.list.sct_harmony$SCT_snn_res.0.4 <- factor(pichip.t.list.sct_harmony$SCT_snn_res.0.4,
                                                  levels = 0:(length(unique(pichip.t.list.sct_harmony$SCT_snn_res.0.4))-1))

pichip.t.list.sct_harmony$SCT_snn_res.0.8 <- factor(pichip.t.list.sct_harmony$SCT_snn_res.0.8,
                                                  levels = 0:(length(unique(pichip.t.list.sct_harmony$SCT_snn_res.0.8))-1))

pichip.t.list.sct_harmony$SCT_snn_res.1.2 <- factor(pichip.t.list.sct_harmony$SCT_snn_res.1.2,
                                                  levels = 0:(length(unique(pichip.t.list.sct_harmony$SCT_snn_res.1.2))-1))

pichip.t.list.sct_harmony$SCT_snn_res.1.6 <- factor(pichip.t.list.sct_harmony$SCT_snn_res.1.6,
                                                  levels = 0:(length(unique(pichip.t.list.sct_harmony$SCT_snn_res.1.6))-1))

Idents(pichip.t.list.sct_harmony) <- "SCT_snn_res.0.8"

DimPlot(pichip.t.list.sct_harmony,  label = TRUE, pt.size = 1.8, label.size = 7)
# https://www.proteinatlas.org/ENSG00000116161-CACYBP/immune+cell
# CACYBP/IL7R CD4 Memory

DefaultAssay(pichip.t.list.sct_harmony) <- "SCT"
# t.markers <- c("CD3D",  "SELL", "GIMAP5", "CREM", "HSPH1", "ATAD3C", "FOXP3", "CD8A", "GNLY", "NKG7", "CCL5",  "CACYBP", "IL7R" )
t.markers <- c("CD3D",  "SELL", "GIMAP5", "CREM", "HSPH1",  "CD8A", "GNLY", "NKG7", "CCL5",  "CACYBP", "IL7R" )
dot(pichip.t.list.sct_harmony , gene = t.markers, "Percent expression      Average gene expression")


table(pichip.t.list.sct_harmony@active.ident, pichip.t.list.sct_harmony@meta.data$mt.ad)
composition <- table(pichip.t.list.sct_harmony@active.ident, pichip.t.list.sct_harmony@meta.data$mt.ad)
write.table(composition, file='results/pichip.t.list.sct_harmony.cells.composition.by.mt.ad.Feb.06.tsv', quote=FALSE, sep='\t', col.names = TRUE)

table(pichip.t.list.sct_harmony@active.ident, pichip.t.list.sct_harmony@meta.data$each)
composition <- table(pichip.t.list.sct_harmony@active.ident, pichip.t.list.sct_harmony@meta.data$each)
write.table(composition, file='results/pichip.t.list.sct_harmony.cells.composition.by.each.Feb.06.tsv', quote=FALSE, sep='\t', col.names = TRUE)


pichip.t.list.sct_harmony.find.all.markers <- FindAllMarkers(pichip.t.list.sct_harmony, 
                                                             only.pos = TRUE, 
                                                             min.pct = 0.1,
                                                             assay = "RNA",
                                                             slot = "data", 
                                                             logfc.threshold = 0.1)
pichip.t.list.sct_harmony.find.all.markers.top20 <- pichip.t.list.sct_harmony.find.all.markers %>% 
  group_by(cluster) %>% 
  slice_max(n = 20, order_by = avg_log2FC)
view(pichip.t.list.sct_harmony.find.all.markers.top20)
write.table(pichip.t.list.sct_harmony.find.all.markers, file='results/pichip.t.list.sct_harmony.find.all.markers.Feb.06.tsv', quote=FALSE, sep='\t', col.names = TRUE)

saveRDS(pichip.t.list.sct_harmony, file = "objects/Feb.06_pichip.t.list.sct_harmony.rds")
pichip.t.list.sct_harmony <- readRDS(file = "objects/Feb.06_pichip.t.list.sct_harmony.rds")

pichip.t.list.sct_harmony.name <- RenameIdents(pichip.t.list.sct_harmony, 
                                               "0" = "CD4+ Naive", 
                                               "1" = "CD8+ T cells",
                                               "2" = "CD4+ Naive",
                                               "3" = "CD4+ Memory",
                                               "4" = "CD4+ Memory",
                                               "5" = "CD4+ Memory",
                                               "6" = "CD8+ T cells")
DimPlot(pichip.t.list.sct_harmony.name,  label = TRUE, pt.size = 2.8, label.size = 7)
DimPlot(pichip.t.list.sct_harmony.name,   pt.size = 2.8, label.size = 7, group.by = "mt.ad")

pichip.t.list.sct_harmony.name$mt.ad <- factor(pichip.t.list.sct_harmony.name$mt.ad, levels =c( "3D CTRL T cells", "3D AD T cells",  "3D CTRL Microglia & T cells", "3D AD Microglia & T cells"))

DimPlot(pichip.t.list.sct_harmony.name,  label = TRUE, pt.size = 2.8, label.size = 7, split.by = "mt.ad")

# saveRDS(pichip.t.list.sct_harmony.name, file = "objects/Feb.06_pichip.t.list.sct_harmony.name.rds")
pichip.t.list.sct_harmony.name <- readRDS(file = "objects/Feb.06_pichip.t.list.sct_harmony.name.rds")

FeaturePlot(pichip.t.list.sct_harmony.name, features = c( "GNLY", "GZMA", "GZMB", "GZMH", "GZMM", "GZMK", "NKG7"))
FeaturePlot(pichip.t.list.sct_harmony.name, features = c( "GNLY", "GZMA", "GZMB", "GZMH", "GZMM", "GZMK", "NKG7"))
FeaturePlot(pichip.t.list.sct_harmony.name, features = c( "GZMA"))

# [Cytotoxic Gene lookup] ----
DefaultAssay(pichip.t.list.sct_harmony.name)
DefaultAssay(pichip.t.list.sct_harmony.name) <- "RNA"

test.GNLY <- WhichCells(pichip.t.list.sct_harmony.name, expression = GNLY > 0)
length(test.GNLY)   # 101

grep
??featureplot
fplot <- function(obj, markers, ncol, min = NA, max = NA){FeaturePlot(obj,
                                                                      reduction = "umap",
                                                                      features = markers,
                                                                      label = TRUE,label.size = 6,
                                                                      col = c("grey", "red"),
                                                                      ncol = ncol,
                                                                      min.cutoff = min,
                                                                      max.cutoff = 1,
                                                                      pt.size = 0.3)}
fplot(pichip.t.list.sct_harmony.name, "GZMA", 1)  

pichip.t.list.sct_harmony.name.cyto <- subset(pichip.t.list.sct_harmony.name, idents = c( "CD8+ Cytotoxic"))
table(pichip.t.list.sct_harmony.name.cyto@active.ident, pichip.t.list.sct_harmony.name.cyto@meta.data$each)
test.GNLY <- WhichCells(pichip.t.list.sct_harmony.name.cyto, expression = GZMA > 0)
length(test.GNLY)   # 79

xxx <- list()
for(i in 1:length(each.names)){
  xxx[[i]] <- grep(each.names[i], test.GNLY)
}
names(xxx) <- each.names

View(xxx)

xxx.unlist <- unlist(xxx)
xxx.unlist

xxx.df <- data.frame("each" = names(xxx),
                     "count" = sapply(xxx, length))



xxx.df %>% View
each.names <- unique(pichip.t.list.sct_harmony.name.cyto$each)
grep(each.names[1], x = test.GNLY)

test.GNLY
Idents(pichip.t.list.sct_harmony.name.cyto) <- "stim"
table(pichip.t.list.sct_harmony.name.cyto@meta.data$stim)
DefaultAssay(pichip.t.list.sct_harmony.name.cyto) <- "RNA"
de.line.list_rna.pichip.t.list.sct_harmony.name.cyto <- list()
de.line.list_rna.pichip.t.list.sct_harmony.name.cyto[["3D AD CD8T.vs.3D CTRL CD8T"]] <- de(pichip.t.list.sct_harmony.name.cyto, 
                                                                                id1 = "AD", 
                                                                                id2 = "CTRL", 
                                                                                assay = "RNA")
write.table(de.line.list_rna.pichip.t.list.sct_harmony.name.cyto[["3D AD CD8T.vs.3D CTRL CD8T"]], 
            file='results/Feb.06.23.3D AD CD8T.vs.3D CTRL CD8T.tsv', quote=FALSE, sep='\t', col.names = TRUE)

de <- function(x, id1, id2, assay){
  x <- FindMarkers(x,
                   ident.1 = id1,
                   ident.2 = id2,
                   test.use = "wilcox",
                   assay = assay,
                   slot = "data",
                   min.pct = 0.1,
                   logfc.threshold = 0.01)
  return(x)
}


library(UCell)
Idents(pichip.t.list.sct_harmony.name.cyto) <- "mt.ad"
table(pichip.t.list.sct_harmony.name.cyto@meta.data[["mt.ad"]])
# pichip.t.list.sct_harmony.name.cyto <- subset(pichip.t.list.sct_harmony.name.cyto, idents = c( "3D CTRL Microglia", "3D CTRL Microglia & T cells", "3D AD Microglia",  "3D AD Microglia & T cells"))
# pichip.t.list.sct_harmony.name.cyto$mt.ad <- factor(pichip.t.list.sct_harmony.name.cyto$mt.ad, levels = c("3D CTRL Microglia", "3D AD Microglia", "3D CTRL Microglia & T cells", "3D AD Microglia & T cells"))
 pichip.t.list.sct_harmony.name.cyto <- subset(pichip.t.list.sct_harmony.name.cyto, idents  = c( "3D AD T cells",  "3D AD Microglia & T cells"))
 pichip.t.list.sct_harmony.name.cyto$mt.ad <- factor(pichip.t.list.sct_harmony.name.cyto$mt.ad, levels = c("3D AD T cells",  "3D AD Microglia & T cells"))
DefaultAssay(pichip.t.list.sct_harmony.name.cyto) <- "RNA"
DimPlot(pichip.t.list.sct_harmony.name.cyto, label = TRUE, pt.size = 0.8, label.size = 7) + NoLegend()
rm(markers)
markers <- list()
# markers$Cytokine <- c("ARF1","CXCL9","CXCL10","CXCL16","CXCL8", "CXCL14", "CXCL5",
#                       "IL24","IL32", "IL17RA", "IL1B", "IL10", "IL21R", "IL18", "IL6R", "IL7R")
markers$Chemokine <- c("CXCL9","CXCL10","CXCL16","CXCL8", "CXCL14", "CXCL5")
markers$Cytotoxic <- c("GNLY", "GZMA", "GZMB",  "NKG7")
markers$Cytotoxic <- c("GNLY", "NKG7")
markers$Cytotoxic <- c( "GZMA", "GZMB", "GZMH", "GZMM", "GZMK")
pichip.t.list.sct_harmony.name.cyto <- AddModuleScore_UCell(pichip.t.list.sct_harmony.name.cyto, features = markers)
signature.names <- paste0(names(markers), "_UCell")
VlnPlot(pichip.t.list.sct_harmony.name.cyto, features = signature.names, group.by = "stim")
VlnPlot(pichip.t.list.sct_harmony.name.cyto, features = signature.names, group.by = "mt", pt.size = 0.4)
VlnPlot(pichip.t.list.sct_harmony.name.cyto, features = signature.names, group.by = "mt.ad")


# Cytotoxic Gene End ----

table(pichip.t.list.sct_harmony.name@active.ident, pichip.t.list.sct_harmony.name@meta.data$mt.ad)
composition <- table(pichip.t.list.sct_harmony.name@active.ident, pichip.t.list.sct_harmony.name@meta.data$mt.ad)
write.table(composition, file='results/pichip.t.list.sct_harmony.name.cells.composition.by.mt.ad.Feb.06.tsv', quote=FALSE, sep='\t', col.names = TRUE)

table(pichip.t.list.sct_harmony.name@active.ident, pichip.t.list.sct_harmony.name@meta.data$each)
composition <- table(pichip.t.list.sct_harmony.name@active.ident, pichip.t.list.sct_harmony.name@meta.data$each)
write.table(composition, file='results/pichip.t.list.sct_harmony.name.cells.composition.by.each.Feb.06.tsv', quote=FALSE, sep='\t', col.names = TRUE)



# ::T AD vs T CTRL DE ----
Idents(pichip.t.list.sct_harmony.name) <- "stim"
table(pichip.t.list.sct_harmony.name@meta.data$stim)
DefaultAssay(pichip.t.list.sct_harmony.name) <- "RNA"
de.line.list_rna.pichip.t.list.sct_harmony.name <- list()
de.line.list_rna.pichip.t.list.sct_harmony.name[["3D AD T.vs.3D CTRL T"]] <- de(pichip.t.list.sct_harmony.name, 
                                                                                id1 = "AD", 
                                                                                id2 = "CTRL", 
                                                                                assay = "RNA")
write.table(de.line.list_rna.pichip.t.list.sct_harmony.name[["3D AD T.vs.3D CTRL T"]], 
            file='results/Feb.06.23.3D AD T.vs.3D CTRL T.tsv', quote=FALSE, sep='\t', col.names = TRUE)


# ::T.Mic AD vs T.Mic CTRL DE ----
Idents(pichip.t.list.sct_harmony.name) <- "mt.ad"
table(pichip.t.list.sct_harmony.name@meta.data$mt.ad)
DefaultAssay(pichip.t.list.sct_harmony.name) <- "RNA"
de.line.list_rna.pichip.t.list.sct_harmony.name <- list()
de.line.list_rna.pichip.t.list.sct_harmony.name[["3D AD Microglia & T cells .vs.3D CTRL Microglia & T cells"]] <- de(pichip.t.list.sct_harmony.name, 
                                                                                                       id1 = "3D AD Microglia & T cells", 
                                                                                                       id2 = "3D CTRL Microglia & T cells", 
                                                                                                       assay = "RNA")
write.table(de.line.list_rna.pichip.t.list.sct_harmony.name[["3D AD Microglia & T cells .vs.3D CTRL Microglia & T cells"]], 
            file='results/Feb.06.23.3D.AD.T.Mic.vs.3D.CTRL.T.Mic.tsv', quote=FALSE, sep='\t', col.names = TRUE)

# ::T.Mic AD vs T.Mic CTRL export.vln ----
pichip.t.list.sct_harmony.name.ctrl <- subset(pichip.t.list.sct_harmony.name, idents = c( "3D CTRL Microglia & T cells"))
write.table(pichip.t.list.sct_harmony.name.ctrl@assays[["RNA"]]@data, file='results/pichip.t.list.sct_harmony.name.ctrl.tsv', quote=FALSE, sep='\t', col.names = TRUE)

pichip.t.list.sct_harmony.name.AD <- subset(pichip.t.list.sct_harmony.name, idents = c( "3D AD Microglia & T cells"))
write.table(pichip.t.list.sct_harmony.name.AD@assays[["RNA"]]@data, file='results/pichip.t.list.sct_harmony.name.AD.tsv', quote=FALSE, sep='\t', col.names = TRUE)

# ::T.Mic.AD vs T.AD DE ----
Idents(pichip.t.list.sct_harmony.name) <- "mt.ad"
table(pichip.t.list.sct_harmony.name@meta.data$mt.ad)
de.line.list_rna.pichip.t.list.sct_harmony.name <- list()
de.line.list_rna.pichip.t.list.sct_harmony.name[["3D AD Microglia & T cells .vs.3D AD T cells"]] <- de(pichip.t.list.sct_harmony.name, 
                                                                                           id1 = "3D AD Microglia & T cells", 
                                                                                           id2 = "3D AD T cells", 
                                                                                           assay = "RNA")
write.table(de.line.list_rna.pichip.t.list.sct_harmony.name[["3D AD Microglia & T cells .vs.3D AD T cells"]], file='results/3D AD Microglia & T cells .vs.3D AD T cells.tsv', quote=FALSE, sep='\t', col.names = TRUE)

# ::T.Mic.AD vs T.AD export.vln ----
pichip.t.list.sct_harmony.name.mt.t.ctrl <- subset(pichip.t.list.sct_harmony.name.t.tri, idents = c( "3D CTRL Microglia & T cells"))
write.table(pichip.t.list.sct_harmony.name.mt.t.ctrl@assays[["RNA"]]@data, file='pichip.t.list.sct_harmony.name.mt.t.ctrl.tsv', quote=FALSE, sep='\t', col.names = TRUE)

pichip.t.list.sct_harmony.name.mt.t.AD <- subset(pichip.t.list.sct_harmony.name.t.tri, idents = c( "3D AD Microglia & T cells"))
write.table(pichip.t.list.sct_harmony.name.mt.t.AD@assays[["RNA"]]@data, file='pichip.t.list.sct_harmony.name.mt.t.AD.tsv', quote=FALSE, sep='\t', col.names = TRUE)

library(UCell)
Idents(pichip.t.list.sct_harmony.name) <- "mt.ad"
table(pichip.t.list.sct_harmony.name@meta.data[["mt.ad"]])
# pichip.t.list.sct_harmony.name <- subset(pichip.t.list.sct_harmony.name, idents = c( "3D CTRL Microglia", "3D CTRL Microglia & T cells", "3D AD Microglia",  "3D AD Microglia & T cells"))
# pichip.t.list.sct_harmony.name$mt.ad <- factor(pichip.t.list.sct_harmony.name$mt.ad, levels = c("3D CTRL Microglia", "3D AD Microglia", "3D CTRL Microglia & T cells", "3D AD Microglia & T cells"))
pichip.t.list.sct_harmony.name <- subset(pichip.t.list.sct_harmony.name, idents  = c( "3D AD T cells",  "3D AD Microglia & T cells"))
pichip.t.list.sct_harmony.name$mt.ad <- factor(pichip.t.list.sct_harmony.name$mt.ad, levels = c("3D AD T cells",  "3D AD Microglia & T cells"))
DefaultAssay(pichip.t.list.sct_harmony.name) <- "RNA"
DimPlot(pichip.t.list.sct_harmony.name, label = TRUE, pt.size = 0.8, label.size = 7) + NoLegend()
rm(markers)
markers <- list()
# markers$Cytokine <- c("ARF1","CXCL9","CXCL10","CXCL16","CXCL8", "CXCL14", "CXCL5",
#                       "IL24","IL32", "IL17RA", "IL1B", "IL10", "IL21R", "IL18", "IL6R", "IL7R")
markers$Chemokine <- c("CXCL9","CXCL10","CXCL16","CXCL8", "CXCL14", "CXCL5")
markers$Cytotoxic <- c("GNLY", "GZMA", "GZMB", "GZMH", "GZMM", "GZMK", "NKG7")
markers$Interferon.Gamma <- c("NR1H3","CDC37","OTOP1","HCK","HPX","IRGM","IFNG","IFNGR1","IFNGR2","IRF1", "JAK1","JAK2","ARG1","PARP14","PPARG","MED1",
                              "PTPN2",
                              "SP100",
                              "STAT1",
                              "TP53",
                              "TXK",
                              "NR1H2",
                              "PARP9",
                              "NLRC5",
                              "SOCS1",
                              "NMI")
# markers$Interferon.Gamma <- c("STAT1", "IRF1", "HLA-B","GBP2","B2M","GBP1","HLA-E")
# markers$Endocytosis <- c("COLEC12","CD81", "ITGB2", "STAB1", "AP2S1", "APOE", "CALR", "CD36")
# markers$MHCI <- c("HLA-B","B2M","HLA-E")
# markers$T.activation <- c("DPP4","CD2","ZAP70","CD8B","CD8A","CD7","CCR7","CD3E")

pichip.t.list.sct_harmony.name <- AddModuleScore_UCell(pichip.t.list.sct_harmony.name, features = markers)
signature.names <- paste0(names(markers), "_UCell")

VlnPlot(pichip.t.list.sct_harmony.name, features = signature.names, group.by = "mt.ad", pt.size = 0.4)
VlnPlot(pichip.t.list.sct_harmony.name, features = signature.names, group.by = "mt.ad")

# end ----

# T ReCluster (Jan.26) CD8 T cells  ----
pichip.t.list.sct_harmony.name.cd8 <- subset(pichip.t.list.sct_harmony.name, idents = c("CD8+ T cells"))
table(pichip.t.list.sct_harmony.name.cd8@meta.data[["stim"]])
Idents(pichip.t.list.sct_harmony.name.cd8) <- "stim"

de.line.list_rna.pichip.t.list.sct_harmony.name <- list()
de.line.list_rna.pichip.t.list.sct_harmony.name[["CD8 AD.vs.CD8 CTRL"]] <- de(pichip.t.list.sct_harmony.name.cd8, 
                                                                             id1 = "AD", 
                                                                             id2 = "CTRL", 
                                                                             assay = "RNA")
write.table(de.line.list_rna.pichip.t.list.sct_harmony.name[["CD8 AD.vs.CD8 CTRL"]], 
            file='results/Feb.06.23.CD8 AD.vs.CD8 CTRL.tsv', quote=FALSE, sep='\t', col.names = TRUE)




# differential composition
# adding cell.type metadata w/ Jung 10.13.22 ----
rm(test)
test <- filtered_seurat.harmony.name@active.ident
test <- Idents(filtered_seurat.harmony.name)

View(as.data.frame(test))
meta.cluster.name <- as.data.frame(Idents(filtered_seurat.harmony.name))
rm(test)

# orignal seurat rowname check
rownames(filtered_seurat.harmony.name@meta.data)

# orignal seurat rowname = cell.type rowname check
identical(rownames(filtered_seurat.harmony.name@meta.data), rownames(meta.cluster.name))

filtered_seurat.harmony.name <- AddMetaData(filtered_seurat.harmony.name, meta.cluster.name, col.name = 'cell_type')
View(filtered_seurat.harmony.name)

# DCATS ----
library(DCATS)

# sampleID = str_c(filtered_seurat.harmony.name$stim, sample(c("p1", "p2", "p3", "p4"), length(Idents(filtered_seurat.harmony.name)), replace = TRUE))
# filtered_seurat.harmony.name = AddMetaData(filtered_seurat.harmony.name, sampleID, 's')
# Visualization
DimPlot(filtered_seurat.harmony.name, reduction = "umap", group.by = "stim")


knn_mat = knn_simMat(filtered_seurat.harmony.name@graphs$SCT_snn, filtered_seurat.harmony.name$cell_type)
print(knn_mat)



count_mat = table(filtered_seurat.harmony.name$each, filtered_seurat.harmony.name$cell_type)
count_mat



design_mat = data.frame(condition = c("CTRL", "CTRL", "STIM", "STIM"))
dcats_GLM(count_mat, design_mat, similarity_mat = knn_mat)



#     :::: T. DE export.vln----
# CD8
t.cells.name.cd8 <- subset(t.cells.name, idents = c("CD8 T cells"))
table(t.cells.name.cd8@meta.data[["stim"]])
Idents(t.cells.name.cd8) <- "stim"

de.line.list_rna.pichip[["cd8.T.AD.vs.CTRL"]] <- de(t.cells.name.cd8, 
                                                    id1 = "AD", 
                                                    id2 = "CTRL", 
                                                    assay = "RNA")
write.table(de.line.list_rna.pichip[["cd8.T.AD.vs.CTRL"]], file='d8.T.AD.vs.CTRL.tsv', quote=FALSE, sep='\t', col.names = TRUE)
VlnPlot(t.cells.name.cd8, features = c("CXCR3"), split.by = "stim", pt.size = 0, combine = FALSE)

pichip.mt.t.ctrl <- subset(pichip.t.tri, idents = c( "3D CTRL Microglia & T cells"))
write.table(pichip.mt.t.ctrl@assays[["RNA"]]@data, file='pichip.mt.t.ctrl.tsv', quote=FALSE, sep='\t', col.names = TRUE)

pichip.mt.t.AD <- subset(pichip.t.tri, idents = c( "3D AD Microglia & T cells"))
write.table(pichip.mt.t.AD@assays[["RNA"]]@data, file='pichip.mt.t.AD.tsv', quote=FALSE, sep='\t', col.names = TRUE)

# CD4 
t.cells.name.cd4.mem <- subset(t.cells.name, idents = c("CD4 Memory"))
table(t.cells.name.cd4.mem@meta.data[["stim"]])
Idents(t.cells.name.cd4.mem) <- "stim"

de.line.list_rna.pichip[["cd4.memory.T.AD.vs.CTRL"]] <- de(t.cells.name.cd4.mem, 
                                                    id1 = "AD", 
                                                    id2 = "CTRL", 
                                                    assay = "RNA")
write.table(de.line.list_rna.pichip[["cd4.memory.T.AD.vs.CTRL"]], file='cd4.memory.T.AD.vs.CTRL.tsv', quote=FALSE, sep='\t', col.names = TRUE)
VlnPlot(t.cells.name.cd4.mem, features = c("CXCR3"), split.by = "stim", pt.size = 0, combine = FALSE)


table(t.cells.name@active.ident, t.cells.name@meta.data$mt.ad)

# :: Mic ReCluster ----

table(pichip.mic@meta.data[["mt.ad"]])

Idents(pichip.mic) <- "mt.ad"
pichip.mic.clustering <- subset(pichip.mic, idents = c("3D AD Microglia", "3D AD Microglia & T cells", "3D CTRL Microglia", "3D CTRL Microglia & T cells"))
DimPlot(pichip.mic.clustering)
table(pichip.mic.clustering@meta.data[["mt.ad"]])

DefaultAssay(pichip.mic.clustering) <- "RNA"
Idents(pichip.mic.clustering) <- "mt.ad"


pichip.mic.clustering
pichip.mic.clustering_harmony <- pichip.mic.clustering %>% RunHarmony("mt.ad", assay.use = "SCT", plot_convergence = T)


harmony_embeddings <- Embeddings(pichip.mic.clustering_harmony, 'harmony')
harmony_embeddings[1:5, 1:5]

pichip.mic.clustering_harmony <- pichip.mic.clustering_harmony %>%
  RunUMAP(reduction = "harmony", dims = 1:30) %>%
  FindNeighbors(reduction = "harmony", dims = 1:30) %>%
  FindClusters(resolution = c(0.4, 0.8, 1.2, 1.6))

names(pichip.mic.clustering_harmony@meta.data)

Tri.res.list <- list(names(pichip.mic.clustering_harmony@meta.data[13]),
                     names(pichip.mic.clustering_harmony@meta.data[14]),
                     names(pichip.mic.clustering_harmony@meta.data[15]),
                     names(pichip.mic.clustering_harmony@meta.data[16]))

Tri.res.list

library(stringr)
names(Tri.res.list) <- str_sub(names(pichip.mic.clustering_harmony@meta.data[13:16]), start = 1L, end = -1L)
View(Tri.res.list)


# :: View clusters
pichip.mic.clustering_harmony$RNA_snn_res.0.4 <- factor(pichip.mic.clustering_harmony$RNA_snn_res.0.4,
                                                    levels = 0:(length(unique(pichip.mic.clustering_harmony$RNA_snn_res.0.4))-1))

pichip.mic.clustering_harmony$RNA_snn_res.0.8 <- factor(pichip.mic.clustering_harmony$RNA_snn_res.0.8,
                                                    levels = 0:(length(unique(pichip.mic.clustering_harmony$RNA_snn_res.0.8))-1))

pichip.mic.clustering_harmony$RNA_snn_res.1.2 <- factor(pichip.mic.clustering_harmony$RNA_snn_res.1.2,
                                                    levels = 0:(length(unique(pichip.mic.clustering_harmony$RNA_snn_res.1.2))-1))

pichip.mic.clustering_harmony$RNA_snn_res.1.6 <- factor(pichip.mic.clustering_harmony$RNA_snn_res.1.6,
                                                    levels = 0:(length(unique(pichip.mic.clustering_harmony$RNA_snn_res.1.6))-1))

Idents(pichip.mic.clustering_harmony) <- "RNA_snn_res.0.8"

DimPlot(pichip.mic.clustering_harmony,  label = TRUE, pt.size = 1.8, label.size = 7) 



DefaultAssay(pichip.mic.clustering_harmony) <- "SCT"
Micmarkers <- c("HLA-DRB1", "HLA-DPB1","HLA-DQA1", "CD74",
                "P2RY12", "P2RY13", "CX3CR1", "TMEM119",
                 "NOS", "INOS", "NOS2A", "HEP-NOS",
                "P35", "CLMF", "NFSK", "NKSF1", "IL12B",
                "IRF1", "IFI6", "IRF7", "ISG15", "STAT1", "IFIT3", 
                "SPP1","CD9", "TREM2", "LPL",
                "CCL2", "CCL3", "CXCL10", "CCL8", "CXCL11",
                "TOP2A", "MKI67" )
dot(pichip.mic.clustering_harmony, gene = Micmarkers, "Percent expression      Average gene expression")

DotPlot(pichip.mic.clustering_harmony, features = Micmarkers,  cols = c("blue", "red", 'black', "salmon"), split.by = "mt.ad", dot.scale = 15) 


composition.mic.clustering <- table(pichip.mic.clustering_harmony@active.ident, pichip.mic.clustering_harmony@meta.data$mt.ad)
write.table(composition.mic.clustering, file='results/composition.mic.clustering.res.0.8.tsv', quote=FALSE, sep='\t', col.names = TRUE)
# https://www.proteinatlas.org/ENSG00000198851-CD3E/immune+cell
pichip.mic.clustering_harmony.find.all.markers <- FindAllMarkers(pichip.mic.clustering_harmony, 
                                                             only.pos = TRUE, 
                                                             min.pct = 0.1,
                                                             assay = "RNA",
                                                             slot = "data", 
                                                             logfc.threshold = 0.1)
write.table(pichip.mic.clustering_harmony.find.all.markers, file='results/composition.mic.clustering.res.0.8.find.markers.tsv', quote=FALSE, sep='\t', col.names = TRUE)

pichip.mic.clustering_harmony.find.all.markers.top20 <- pichip.mic.clustering_harmony.find.all.markers %>% 
  group_by(cluster) %>% 
  slice_max(n = 10, order_by = avg_log2FC)
view(pichip.mic.clustering_harmony.find.all.markers.top20)

# Mic SCT recluster ----

mic.only <- subset(pichip.mic.clustering_harmony, idents = c("2", "3", 
                                                             "7","8","9", "10", "11", "12",
                                                             "13","14","15",  "17", "18", "19"))

# split the dataset into a list of two seurat objects (stim and CTRL)
mic.only.list <- SplitObject(mic.only, split.by = "mt.ad")


mic.only.list <- lapply(X = mic.only.list, 
                      FUN = SCTransform, 
                      method = "glmGamPoi", 
                      return.only.var.genes = FALSE)

mic.only.features <- SelectIntegrationFeatures(object.list = mic.only.list, nfeatures = 3000)
mic.only.list <- PrepSCTIntegration(object.list = mic.only.list, anchor.features = mic.only.features)


mic.only.anchors <- FindIntegrationAnchors(object.list = mic.only.list, normalization.method = "SCT",
                                         anchor.features = mic.only.features)
mic.only.combined.sct <- IntegrateData(anchorset = mic.only.anchors, normalization.method = "SCT")

mic.only.combined.sct <- RunPCA(mic.only.combined.sct, verbose = FALSE)
mic.only.combined.sct <- RunUMAP(mic.only.combined.sct, reduction = "pca", dims = 1:30, verbose = FALSE)
mic.only.combined.sct <- FindNeighbors(mic.only.combined.sct, reduction = "pca", dims = 1:30)
mic.only.combined.sct <- FindClusters(mic.only.combined.sct, resolution = c(0.4, 0.8, 1.2, 1.6))

names(mic.only.combined.sct@meta.data)

Tri.res.list <- list(names(mic.only.combined.sct@meta.data[17]),
                     names(mic.only.combined.sct@meta.data[18]),
                     names(mic.only.combined.sct@meta.data[19]),
                     names(mic.only.combined.sct@meta.data[20]))

Tri.res.list

library(stringr)
names(Tri.res.list) <- str_sub(names(mic.only.combined.sct@meta.data[17:20]), start = 1L, end = -1L)
View(Tri.res.list)


# :: View clusters
mic.only.combined.sct$integrated_snn_res.0.4 <- factor(mic.only.combined.sct$integrated_snn_res.0.4,
                                                        levels = 0:(length(unique(mic.only.combined.sct$integrated_snn_res.0.4))-1))

mic.only.combined.sct$integrated_snn_res.0.8 <- factor(mic.only.combined.sct$integrated_snn_res.0.8,
                                                        levels = 0:(length(unique(mic.only.combined.sct$integrated_snn_res.0.8))-1))

mic.only.combined.sct$integrated_snn_res.1.2 <- factor(mic.only.combined.sct$integrated_snn_res.1.2,
                                                        levels = 0:(length(unique(mic.only.combined.sct$integrated_snn_res.1.2))-1))

mic.only.combined.sct$integrated_snn_res.1.6 <- factor(mic.only.combined.sct$integrated_snn_res.1.6,
                                                        levels = 0:(length(unique(mic.only.combined.sct$integrated_snn_res.1.6))-1))

Idents(mic.only.combined.sct) <- "integrated_snn_res.0.8"

DimPlot(mic.only.combined.sct,  label = TRUE, pt.size = 1.8, label.size = 7) 


# mic.only <- subset(mic.only_harmony, idents = c("0","2","3", "4", "5", "6"))
DimPlot(mic.only.combined.sct,  label = TRUE, pt.size = 1.8, label.size = 7) 

Micmarkers <- c("HLA-DRB1", "HLA-DPB1","HLA-DQA1", "CD74",
                "P2RY12", "P2RY13", "CX3CR1", "TMEM119",
                "IRF1", "IFI6", "IRF7", "ISG15", "STAT1", "IFIT3", 
                "SPP1","CD9", "TREM2", "LPL",
                "CCL2", "CCL3", "CXCL10", "CCL8", "CXCL11",
                "TOP2A", "MKI67" )
dot(mic.only.combined.sct, gene = Micmarkers, "Percent expression      Average gene expression")

composition.mic.only.combined.sct <- table(mic.only.combined.sct@active.ident, mic.only.combined.sct@meta.data$mt.ad)
write.table(composition.mic.only.combined.sct, file='results/composition.mic.only.combined.sct.res.0.8.tsv', quote=FALSE, sep='\t', col.names = TRUE)

composition.mic.only.combined.sct <- table(mic.only.combined.sct@active.ident, mic.only.combined.sct@meta.data$mt.ad)
write.table(composition.mic.only.combined.sct, file='results/composition.mic.only.combined.sct.res.0.8.tsv', quote=FALSE, sep='\t', col.names = TRUE)


DefaultAssay(mic.only.combined.sct) <- "RNA"
DotPlot(mic.only.combined.sct, features = Micmarkers,  cols = c("blue", "red"), split.by = "stim", dot.scale = 15) 


mic.only.combined.sct.find.all.markers <- FindAllMarkers(mic.only.combined.sct, 
                                                              only.pos = TRUE, 
                                                              min.pct = 0.10,
                                                              assay = "RNA",
                                                              slot = "data", 
                                                              logfc.threshold = 0.05)
mic.only.combined.sct.find.all.markers.top20 <- mic.only.combined.sct.find.all.markers %>% 
  group_by(cluster) %>% 
  slice_max(n = 20, order_by = avg_log2FC)
view(mic.only.combined.sct.find.all.markers.top20)
# end----

# :: Ast ReCluster ----

table(pichip.ast@meta.data[["mt.ad"]])

Idents(pichip.ast) <- "mt.ad"
pichip.ast.clustering <- subset(pichip.ast, idents = c( "3D CTRL",  "3D AD"))
DimPlot(pichip.ast.clustering)
table(pichip.ast.clustering@meta.data[["mt.ad"]])

DefaultAssay(pichip.ast.clustering) <- "RNA"
Idents(pichip.ast.clustering) <- "mt.ad"


# :: Ast. 3D vs 3D.Mic DE ----

VlnPlot(pichip.ast.clustering, features = c( "CXCL10", "CCL3", "CCL2", "CCL4"), split.by = "mt.ad", pt.size = 1, combine = TRUE, slot = "data")
VlnPlot(pichip.ast.clustering, features = c( "CXCL10"), split.by = "stim", pt.size = 1, combine = TRUE, slot = "data")
de <- function(x, id1, id2, assay){
  x <- FindMarkers(x,
                   ident.1 = id1,
                   ident.2 = id2,
                   test.use = "wilcox",
                   assay = assay,
                   slot = "data",
                   min.pct = 0.01,
                   logfc.threshold = 0.01)
  return(x)
}
# counting -
DefaultAssay(pichip.ast.clustering) <- "RNA"
table(pichip.ast.clustering@meta.data[["mt.ad"]])
Idents(pichip.ast.clustering) <- "mt.ad"

de.line.list_rna.pichip.ast <- list()
#:: pichip.astrocyte AD.vs.CTRL ----
de.line.list_rna.pichip.ast[["ast.3D AD.vs.3D CTRL"]] <- de(pichip.ast.clustering, 
                                                                id1 = "3D AD", 
                                                                id2 = "3D CTRL", 
                                                                assay = "RNA")

write.table(de.line.list_rna.pichip.ast[["ast.3D AD.vs.3D CTRL"]], file='results/Ast.ast.3D AD.vs.3D CTRL.tsv', quote=FALSE, sep='\t', col.names = TRUE)

pichip.ast.clustering$mt.ad <- factor(pichip.ast.clustering$mt.ad, levels = c("3D CTRL", "3D AD"))




counting <- function(x){
  library(dplyr)
  p.filter <- x %>% filter(p_val_adj <= 0.2)
  positive.FC <- nrow(p.filter %>% filter(avg_log2FC > 0))
  negative.FC <- nrow(p.filter %>% filter(avg_log2FC < 0))
  print(paste0("pos = ", positive.FC, " ", "neg = ", negative.FC))
}

lapply(de.line.list_rna.pichip, counting)


#:::: DE.pichip.mic.vs.3D.mic in AD ----

table(pichip.mic@meta.data[["mt.ad"]])

Idents(pichip.mic) <- "mt.ad"
pichip.mic.01 <- subset(pichip.mic, idents = c("3D AD Microglia",  "3D AD Microglia & T cells"))
DimPlot(pichip.mic.01)
table(pichip.mic.01@meta.data[["mt.ad"]])

DefaultAssay(pichip.mic.01) <- "RNA"
Idents(pichip.mic.01) <- "mt.ad"

de.line.list_pichip.mic.01 <- list()


library(UCell)



Idents(pichip.mic) <- "mt.ad"
table(pichip.mic@meta.data[["mt.ad"]])
pichip.mic.02 <- subset(pichip.mic, idents = c( "3D CTRL Microglia", "3D CTRL Microglia & T cells", "3D AD Microglia",  "3D AD Microglia & T cells"))
pichip.mic.02$mt.ad <- factor(pichip.mic.02$mt.ad, levels = c("3D CTRL Microglia", "3D CTRL Microglia & T cells", "3D AD Microglia",  "3D AD Microglia & T cells"))
DefaultAssay(pichip.mic.02) <- "RNA"
DimPlot(pichip.mic.02, label = TRUE, pt.size = 0.8, label.size = 7) + NoLegend()
rm(markers)
markers <- list()
markers$Endocytosis <- c("COLEC12","CD81", "ITGB2", "STAB1", "AP2S1", "APOE", "CALR", "CD36")
markers$Interferon.Gamma <- c("STAT1","HLA-B","GBP2","B2M","GBP1","HLA-E")
markers$MHCI <- c("HLA-B","B2M","HLA-E")
markers$T.activation <- c("DPP4","CD2","ZAP70","CD8B","CD8A","CD7","CCR7","CD3E")

pichip.mic.02 <- AddModuleScore_UCell(pichip.mic.02, features = markers)
signature.names <- paste0(names(markers), "_UCell")

VlnPlot(pichip.mic.02, features = signature.names, group.by = "mt.ad", pt.size = 0.4)
RidgePlot(pichip.mic.02, features = signature.names, group.by = "mt.ad")
??VlnPlot


# :: Ast .ast.end ----


mic.only.name <- RenameIdents(mic.only, 
                                               "0" = "CD4 Naive", 
                                               "1" = "CD4 Memory",
                                               "2" = "CD8 T cells",
                                               "3" = "CD4 Memory",
                                               "4" = "CD4 Naive",
                                               "5" = "CD8 T cells",
                                               "6" = "CD4 Memory",
                                               "7" = "Treg")
DimPlot(pichip.mic.clustering_harmony.name,  label = TRUE, pt.size = 0.8, label.size = 7) + NoLegend()
t.markers <- c("CD3D",  "SELL", "GIMAP5", "CREM", "HSPH1","CD8A",  "CCL5", "FOXP3",  "CACYBP" )
dot(pichip.mic.clustering_harmony.name , gene = t.markers, "Percent expression      Average gene expression")

table(pichip.mic.clustering_harmony.name@active.ident, pichip.mic.clustering_harmony.name@meta.data$sample)
composition <- table(pichip.mic.clustering_harmony.name@active.ident, pichip.mic.clustering_harmony.name@meta.data$sample)
write.table(composition, file='t.cells.composition.tsv', quote=FALSE, sep='\t', col.names = TRUE)

# :: Mic.T ReCluster/Interaction ----
pichip <- subset(filtered_seurat.harmony.name, idents = c("Astrocyte","T cells","Microglia","Neuron"))
pichip.mt <- subset(pichip, idents = c("T cells", "Microglia"))
Idents(pichip.mt) <- "mt.ad"
pichip.mt <- subset(pichip.mt, idents = c("3D AD Microglia & T cells",  "3D CTRL Microglia & T cells"))

DefaultAssay(pichip.mt) <- "SCT"
pichip.mt
pichip.mt_combined.harmony <- pichip.mt %>% RunHarmony("sample", assay.use = "SCT", plot_convergence = T)


harmony_embeddings <- Embeddings(pichip.mt_combined.harmony, 'harmony')
harmony_embeddings[1:5, 1:5]

pichip.mt_combined.harmony <- pichip.mt_combined.harmony %>%
  RunUMAP(reduction = "harmony", dims = 1:30) %>%
  FindNeighbors(reduction = "harmony", dims = 1:30) %>%
  FindClusters(resolution = c(0.4, 0.8, 1.2, 1.6))

names(pichip.mt_combined.harmony@meta.data)

Tri.res.list <- list(names(pichip.mt_combined.harmony@meta.data[13]),
                     names(pichip.mt_combined.harmony@meta.data[14]),
                     names(pichip.mt_combined.harmony@meta.data[15]),
                     names(pichip.mt_combined.harmony@meta.data[16]))

Tri.res.list

library(stringr)
names(Tri.res.list) <- str_sub(names(pichip.mt_combined.harmony@meta.data[13:16]), start = 1L, end = -1L)
View(Tri.res.list)


# :: View clusters
pichip.mt_combined.harmony$RNA_snn_res.0.4 <- factor(pichip.mt_combined.harmony$RNA_snn_res.0.4,
                                                    levels = 0:(length(unique(pichip.mt_combined.harmony$RNA_snn_res.0.4))-1))

pichip.mt_combined.harmony$RNA_snn_res.0.8 <- factor(pichip.mt_combined.harmony$RNA_snn_res.0.8,
                                                    levels = 0:(length(unique(pichip.mt_combined.harmony$RNA_snn_res.0.8))-1))

pichip.mt_combined.harmony$RNA_snn_res.1.2 <- factor(pichip.mt_combined.harmony$RNA_snn_res.1.2,
                                                    levels = 0:(length(unique(pichip.mt_combined.harmony$RNA_snn_res.1.2))-1))

pichip.mt_combined.harmony$RNA_snn_res.1.6 <- factor(pichip.mt_combined.harmony$RNA_snn_res.1.6,
                                                    levels = 0:(length(unique(pichip.mt_combined.harmony$RNA_snn_res.1.6))-1))

Idents(pichip.mt_combined.harmony) <- "RNA_snn_res.0.4"

DimPlot(pichip.mt_combined.harmony,  label = TRUE, pt.size = 0.8, label.size = 7) 



DefaultAssay(pichip.mt_combined.harmony) <- "RNA"
mt.markers <- c("CSF1R", "SPP1", "CD3D", "CD8A", "GNLY", "NKG7", "CCL5",  "CACYBP", 
                "TCF7", "CD4", "CCR7", "FHIT", "LEF1", "NOSIP", "PIK3IP1",
                "IL7R", "TMSB10",  "ITGB1", "LTB", "TRAC", "AQP3", "LDHB", "IL32", "MAL", # CD4+ Central Memory T
                "FYB1", "GZMK", "GZMA", "KLRB1") # CD4+ Effector Memory T
dot(pichip.mic.t , gene = mt.markers, "Percent expression      Average gene expression")
# https://www.proteinatlas.org/ENSG00000198851-CD3E/immune+cell
pichip.mt_combined.harmony.find.all.markers <- FindAllMarkers(pichip.mt_combined.harmony, 
                                                             only.pos = TRUE, 
                                                             min.pct = 0.10,
                                                             assay = "RNA",
                                                             slot = "data", 
                                                             logfc.threshold = 0.05)
pichip.mt_combined.harmony.find.all.markers.top20 <- pichip.mt_combined.harmony.find.all.markers %>% 
  group_by(cluster) %>% 
  slice_max(n = 20, order_by = avg_log2FC)
view(pichip.mt_combined.harmony.find.all.markers.top20)



pichip.mt_combined.harmony.name <- RenameIdents(pichip.mt_combined.harmony, 
                                                
                                               "1" = "Microglia",
                                               "2" = "Microglia",
                                               "4" = "Microglia",
                                               "6" = "Microglia",
                                               "7" = "Microglia",
                                               "9" = "Microglia",
                                               
                                               "3" = "na",
                                               
                                               "5" = "CD8 Cytotoxic",
                                               
                                               "0" = "CD4 Naive",
                                               "8" = "CD4 Memory",
                                               
                                               "10" = "na",
                                               "11" = "na",
                                               "12" = "na",
                                               "13" = "na",
                                               "14" = "na"
                                               )

pichip.mic.t <- subset(pichip.mt_combined.harmony.name, idents = c("CD4 Naive",  "Microglia", "CD8 Cytotoxic", "CD4 Memory" ))

DimPlot(pichip.mic.t,  label = TRUE, pt.size = 1.8, label.size = 7) + NoLegend()
DimPlot(pichip.mic.t,  label = TRUE, pt.size = 1.8, label.size = 7, split.by = "stim") + NoLegend()
mt.markers <- c("CSF1R", "SPP1", "CD3D", "CD8A", "CCL5",
                "TCF7",  "CCR7",
                "IL7R",  "LTB", 
                "FYB1") # CD4+ Effector Memory T
dot(pichip.mic.t , gene = mt.markers, "Percent expression      Average gene expression")

pichip.mic.t.find.all.markers <- FindAllMarkers(pichip.mic.t , 
                                                              only.pos = TRUE, 
                                                              min.pct = 0.10,
                                                              assay = "RNA",
                                                              slot = "data", 
                                                              logfc.threshold = 0.05)
pichip.mic.t.find.all.markers.top20 <- pichip.mic.t.find.all.markers %>% 
  group_by(cluster) %>% 
  slice_max(n = 20, order_by = avg_log2FC)
view(pichip.mic.t.find.all.markers.top20)


table(pichip.mt_combined.harmony.name@active.ident, pichip.mt_combined.harmony.name@meta.data$mt.ad)
# :: Mic.T. DE export.vln----
pichip.mt <- subset(pichip, idents = c("T cells", "Microglia"))
de.line.list_rna.pichip[["t.3D AD Microglia & T cells.vs.3D CTRL Microglia & T cells"]] <- de(pichip.mic.t, 
                                                                                              id1 = "3D AD Microglia & T cells", 
                                                                                              id2 = "3D CTRL Microglia & T cells", 
                                                                                              assay = "RNA")
write.table(de.line.list_rna.pichip[["t.3D AD Microglia & T cells.vs.3D CTRL Microglia & T cells"]], file='pichip.mt.t.DE.tsv', quote=FALSE, sep='\t', col.names = TRUE)

pichip.mt.t.ctrl <- subset(pichip.t.tri, idents = c( "3D CTRL Microglia & T cells"))
write.table(pichip.mt.t.ctrl@assays[["RNA"]]@data, file='pichip.mt.t.ctrl.tsv', quote=FALSE, sep='\t', col.names = TRUE)

pichip.mt.t.AD <- subset(pichip.t.tri, idents = c( "3D AD Microglia & T cells"))
write.table(pichip.mt.t.AD@assays[["RNA"]]@data, file='pichip.mt.t.AD.tsv', quote=FALSE, sep='\t', col.names = TRUE)


# :: T cell only by condition ----
Idents(pichip.mt_combined.harmony.name) <- "mt.ad"
t.cells <- subset(pichip.mt_combined.harmony.name, idents = c("3D AD Microglia & T cells", "3D AD T cells",  "3D CTRL Microglia & T cells", "3D CTRL T cells"))
Idents(t.cells) <- "RNA_snn_res.0.4"
t.cells.name <- RenameIdents(t.cells, 
                             "0" = "CD4 Naive", 
                             "1" = "CD4 Memory",
                             "2" = "CD8 T cells",
                             "3" = "CD4 Memory",
                             "4" = "CD4 Naive",
                             "5" = "CD8 T cells",
                             "6" = "CD4 Memory",
                             "7" = "Treg")


table(t.cells.name@active.ident, t.cells.name@meta.data$mt.ad)

# Nichenet mic.T ----
library(nichenetr)
library(Seurat) # please update to Seurat V4
library(tidyverse)
library(circlize)
pichip <- subset(filtered_seurat.harmony.name, idents = c("Astrocyte","T cells","Microglia","Neuron"))
pichip.mic.t <- subset(pichip, idents = c("Microglia", "T cells"))
pichip.mic.t
pichip.mic.t_combined.harmony <- pichip.mic.t %>% RunHarmony("sample", assay.use = "SCT", plot_convergence = T)


harmony_embeddings <- Embeddings(pichip.mic.t_combined.harmony, 'harmony')
harmony_embeddings[1:5, 1:5]

pichip.mic.t_combined.harmony <- pichip.mic.t_combined.harmony %>%
  RunUMAP(reduction = "harmony", dims = 1:30) %>%
  FindNeighbors(reduction = "harmony", dims = 1:30) %>%
  FindClusters(resolution = c(0.4, 0.8, 1.2, 1.6))

names(pichip.mic.t_combined.harmony@meta.data)

Tri.res.list <- list(names(pichip.mic.t_combined.harmony@meta.data[13]),
                     names(pichip.mic.t_combined.harmony@meta.data[14]),
                     names(pichip.mic.t_combined.harmony@meta.data[15]),
                     names(pichip.mic.t_combined.harmony@meta.data[16]))

Tri.res.list

library(stringr)
names(Tri.res.list) <- str_sub(names(pichip.mic.t_combined.harmony@meta.data[13:16]), start = 1L, end = -1L)
View(Tri.res.list)


# :: View clusters
pichip.mic.t_combined.harmony$RNA_snn_res.0.4 <- factor(pichip.mic.t_combined.harmony$RNA_snn_res.0.4,
                                                    levels = 0:(length(unique(pichip.mic.t_combined.harmony$RNA_snn_res.0.4))-1))

pichip.mic.t_combined.harmony$RNA_snn_res.0.8 <- factor(pichip.mic.t_combined.harmony$RNA_snn_res.0.8,
                                                    levels = 0:(length(unique(pichip.mic.t_combined.harmony$RNA_snn_res.0.8))-1))

pichip.mic.t_combined.harmony$RNA_snn_res.1.2 <- factor(pichip.mic.t_combined.harmony$RNA_snn_res.1.2,
                                                    levels = 0:(length(unique(pichip.mic.t_combined.harmony$RNA_snn_res.1.2))-1))

pichip.mic.t_combined.harmony$RNA_snn_res.1.6 <- factor(pichip.mic.t_combined.harmony$RNA_snn_res.1.6,
                                                    levels = 0:(length(unique(pichip.mic.t_combined.harmony$RNA_snn_res.1.6))-1))

Idents(pichip.mic.t_combined.harmony) <- "RNA_snn_res.0.8"

DimPlot(pichip.mic.t_combined.harmony,  label = TRUE, pt.size = 1.8, label.size = 7) 



DefaultAssay(pichip.mic.t_combined.harmony) <- "SCT"
t.markers <- c("CSF1R", "SPI1", 
               "CD3D", "CD8A", "GNLY", "NKG7", "CCL5", "CCL7",
               "TCF7",  "FHIT", "LEF1", "MAL", "NOSIP", "LDHB", "PIK3IP1",
               "CD4", "CCR7", "IL7R", "LTB", "TRAC", "IL32", "FYB1", "CREM", "HSPH1", "FOXP3" )
dot(pichip.mic.t_combined.harmony , gene = t.markers, "Percent expression      Average gene expression")
# https://www.proteinatlas.org/ENSG00000198851-CD3E/immune+cell
pichip.mic.t_combined.harmony.find.all.markers <- FindAllMarkers(pichip.mic.t_combined.harmony, 
                                                             only.pos = TRUE, 
                                                             min.pct = 0.05,
                                                             assay = "RNA",
                                                             slot = "data", 
                                                             logfc.threshold = 0.05)
pichip.mic.t_combined.harmony.find.all.markers.top20 <- pichip.mic.t_combined.harmony.find.all.markers %>% 
  group_by(cluster) %>% 
  slice_max(n = 5, order_by = avg_log2FC)
view(pichip.mic.t_combined.harmony.find.all.markers.top20)


pichip.mic.t_combined.harmony.name <- RenameIdents(pichip.mic.t_combined.harmony, 
                                               "0" = "CD4 Memory", 
                                               "1" = "Microglia",
                                               "2" = "Microglia",
                                               "3" = "na",
                                               "4" = "CD8 T cells",
                                               "5" = "Microglia",
                                               "6" = "Microglia",
                                               "7" = "Microglia",
                                               "8" = "Microglia",
                                               "9" = "na",
                                               "10" = "CD4 Naive",
                                               "11" = "Microglia",
                                               "12" = "Microglia",
                                               "13" = "CD4 Naive",
                                               "14" = "Microglia",
                                               "15" = "Microglia",
                                               "16" = "Microglia",
                                               "17" = "na",
                                               "18" = "na",
                                               "19" = "na",
                                               "20" = "na",
                                               "21" = "na",
                                               "22" = "na")
DimPlot(pichip.mic.t_combined.harmony.name,  label = TRUE, pt.size = 0.8, label.size = 7) + NoLegend()
t.markers <- c("CSF1R", "SPI1", 
               "CD3D", "CD8A", "GNLY", "NKG7", "CCL5", "CCL7",
               "TCF7",  "FHIT", "LEF1", "MAL", "NOSIP", "LDHB", "PIK3IP1",
               "CD4", "CCR7", "IL7R", "LTB", "TRAC", "IL32", "FYB1", "CREM", "HSPH1", "FOXP3" )
dot(pichip.mic.t_combined.harmony.name , gene = t.markers, "Percent expression      Average gene expression")

table(pichip.mic.t_combined.harmony.name@active.ident, pichip.mic.t_combined.harmony.name@meta.data$mt.ad)
composition <- table(pichip.mic.t_combined.harmony.name@active.ident, pichip.mic.t_combined.harmony.name@meta.data$mt.ad)
write.table(composition, file='t.cells.composition.tsv', quote=FALSE, sep='\t', col.names = TRUE)

table(pichip.mic.t_combined.harmony.name@meta.data[["mt.ad"]])
Idents(pichip.mic.t_combined.harmony.name) <- "mt.ad"
pichip.mic.t.01 <- subset(pichip.mic.t_combined.harmony.name, idents = c("3D AD Microglia & T cells", "3D CTRL Microglia & T cells"))

Idents(pichip.mic.t.01) <- "RNA_snn_res.0.8"

pichip.mic.t.01.name <- RenameIdents(pichip.mic.t.01, 
                                                   "0" = "CD4 Memory", 
                                                   "1" = "Microglia",
                                                   "2" = "Microglia",
                                                   "3" = "na",
                                                   "4" = "CD8 T cells",
                                                   "5" = "Microglia",
                                                   "6" = "Microglia",
                                                   "7" = "Microglia",
                                                   "8" = "Microglia",
                                                   "9" = "na",
                                                   "10" = "CD4 Naive",
                                                   "11" = "Microglia",
                                                   "12" = "Microglia",
                                                   "13" = "CD4 Naive",
                                                   "14" = "Microglia",
                                                   "15" = "Microglia",
                                                   "16" = "Microglia",
                                                   "17" = "na",
                                                   "18" = "na",
                                                   "19" = "na",
                                                   "20" = "na",
                                                   "21" = "na",
                                                   "22" = "na")

pichip.mic.t.02 <- subset(pichip.mic.t.01.name, idents = c("Microglia", "CD4 Naive", "CD4 Memory", "CD8 T cells" ))
pichip.mic.t.03 <- subset(pichip.mic.t.01.name, idents = c("CD4 Naive", "CD4 Memory", "CD8 T cells" ))
DimPlot(pichip.mic.t.02)
DimPlot(pichip.mic.t.02,  label = TRUE, pt.size = 1.8, label.size = 7) + NoLegend()

DoHeatmap(pichip.mic.t.02, 
          features = c("LTB", "TOMM7", "SH3D19", "TRBC2", "CD74", "APOE", "CD68", "SPP1", "CD8B", "CD8B2", "CCL5", "BCAS4",  "RPS27", "RPL30"),
          group.bar = TRUE,
          group.colors = NULL,
          disp.min = NULL,
          disp.max = NULL,
          slot = "data",
          assay = "RNA",
          draw.lines = TRUE, 
          lines.width = NULL,
          angle = 0,
          label = TRUE,
          size = 5,
          hjust = 0.5,
          group.bar.height = 0.01,
          combine = TRUE
)  + scale_fill_gradientn(colors = c("blue", "white", "red"), na.value = "white") + theme(axis.text.y = element_text(size = 10))

test <- pichip.mic.t.02@active.ident
test <- Idents(pichip.mic.t.02)

View(as.data.frame(test))
meta.cluster.name <- as.data.frame(Idents(pichip.mic.t.02))
rm(test)

# orignal seurat rowname check
rownames(pichip.mic.t.02@meta.data)

# orignal seurat rowname = cell.type rowname check
identical(rownames(pichip.mic.t.02@meta.data), rownames(meta.cluster.name))

pichip.mic.t.02 <- AddMetaData(pichip.mic.t.02, meta.cluster.name, col.name = 'cell_type')
View(pichip.mic.t.02)




seuratObj <- pichip.mic.t.02
seuratObj@meta.data %>% head()

seuratObj@meta.data$cell_type %>% table() 
## .
# CD4 Memory   Microglia CD8 T cells   CD4 Naive 
# 799        1065         245         192 
DimPlot(seuratObj, reduction = "umap")

ligand_target_matrix = readRDS(url("https://zenodo.org/record/3260758/files/ligand_target_matrix.rds"))
ligand_target_matrix[1:5,1:5] # target genes in rows, ligands in columns

lr_network = readRDS(url("https://zenodo.org/record/3260758/files/lr_network.rds"))
head(lr_network)

weighted_networks = readRDS(url("https://zenodo.org/record/3260758/files/weighted_networks.rds"))
weighted_networks_lr = weighted_networks$lr_sig %>% inner_join(lr_network %>% distinct(from,to), by = c("from","to"))

head(weighted_networks$lr_sig) # interactions and their weights in the ligand-receptor + signaling network

head(weighted_networks$gr) # interactions and their weights in the gene regulatory network

# : Niche define receiver/sender ----
## receiver
receiver = "Microglia"
expressed_genes_receiver@active.assay = "RNA"
expressed_genes_receiver = get_expressed_genes(receiver, seuratObj, pct = 0.01)

background_expressed_genes = expressed_genes_receiver %>% .[. %in% rownames(ligand_target_matrix)]

## sender
sender_celltypes = c("CD8 T cells",     "CD4 Naive",    "CD4 Memory")
seurat_obj_receiver@active.assay = "RNA"
list_expressed_genes_sender = sender_celltypes %>% unique() %>% lapply(get_expressed_genes, seuratObj, 0.01) # lapply to get the expressed genes of every sender cell type separately here
expressed_genes_sender = list_expressed_genes_sender %>% unlist() %>% unique()

seurat_obj_receiver= subset(seuratObj, idents = receiver)
seurat_obj_receiver = SetIdent(seurat_obj_receiver, value = seurat_obj_receiver[["stim"]])

condition_oi = "AD"
condition_reference = "CTRL" 

seurat_obj_receiver@active.assay = "RNA"
DE_table_receiver = FindMarkers(object = seurat_obj_receiver, ident.1 = condition_oi, ident.2 = condition_reference, min.pct = 0.01) %>% rownames_to_column("gene")

geneset_oi = DE_table_receiver %>% filter(p_val_adj <= 0.2 & abs(avg_log2FC) >= 0.01) %>% pull(gene)
geneset_oi = geneset_oi %>% .[. %in% rownames(ligand_target_matrix)]

ligands = lr_network %>% pull(from) %>% unique()
receptors = lr_network %>% pull(to) %>% unique()

expressed_ligands = intersect(ligands,expressed_genes_sender)
expressed_receptors = intersect(receptors,expressed_genes_receiver)

potential_ligands = lr_network %>% filter(from %in% expressed_ligands & to %in% expressed_receptors) %>% pull(from) %>% unique()

ligand_activities = predict_ligand_activities(geneset = geneset_oi, background_expressed_genes = background_expressed_genes, ligand_target_matrix = ligand_target_matrix, potential_ligands = potential_ligands)

ligand_activities = ligand_activities %>% arrange(-pearson) %>% mutate(rank = rank(desc(pearson)))
ligand_activities

best_upstream_ligands = ligand_activities %>% top_n(20, pearson) %>% arrange(-pearson) %>% pull(test_ligand) %>% unique()

DotPlot(seuratObj, features = best_upstream_ligands %>% rev(), cols = "RdYlBu") + RotatedAxis()


# https://github.com/saeyslab/nichenetr/blob/master/vignettes/seurat_steps.md#active-target-gene-inference
active_ligand_target_links_df = best_upstream_ligands %>% lapply(get_weighted_ligand_target_links,geneset = geneset_oi, ligand_target_matrix = ligand_target_matrix, n = 200) %>% bind_rows() %>% drop_na()

active_ligand_target_links = prepare_ligand_target_visualization(ligand_target_df = active_ligand_target_links_df, ligand_target_matrix = ligand_target_matrix, cutoff = 0.33)

order_ligands = intersect(best_upstream_ligands, colnames(active_ligand_target_links)) %>% rev() %>% make.names()
order_targets = active_ligand_target_links_df$target %>% unique() %>% intersect(rownames(active_ligand_target_links)) %>% make.names()
rownames(active_ligand_target_links) = rownames(active_ligand_target_links) %>% make.names() # make.names() for heatmap visualization of genes like H2-T23
colnames(active_ligand_target_links) = colnames(active_ligand_target_links) %>% make.names() # make.names() for heatmap visualization of genes like H2-T23

vis_ligand_target = active_ligand_target_links[order_targets,order_ligands] %>% t()

p_ligand_target_network = vis_ligand_target %>% make_heatmap_ggplot("Prioritized ligands","Predicted target genes", color = "purple",legend_position = "top", x_axis_position = "top",legend_title = "Regulatory potential")  + theme(axis.text.x = element_text(face = "italic")) + scale_fill_gradient2(low = "whitesmoke",  high = "purple", breaks = c(0,0.0045,0.0090))
p_ligand_target_network

# https://github.com/saeyslab/nichenetr/blob/master/vignettes/seurat_steps.md#receptors-of-top-ranked-ligands
lr_network_top = lr_network %>% filter(from %in% best_upstream_ligands & to %in% expressed_receptors) %>% distinct(from,to)
best_upstream_receptors = lr_network_top %>% pull(to) %>% unique()

lr_network_top_df_large = weighted_networks_lr %>% filter(from %in% best_upstream_ligands & to %in% best_upstream_receptors)

lr_network_top_df = lr_network_top_df_large %>% spread("from","weight",fill = 0)
lr_network_top_matrix = lr_network_top_df %>% select(-to) %>% as.matrix() %>% magrittr::set_rownames(lr_network_top_df$to)

dist_receptors = dist(lr_network_top_matrix, method = "binary")
hclust_receptors = hclust(dist_receptors, method = "ward.D2")
order_receptors = hclust_receptors$labels[hclust_receptors$order]

dist_ligands = dist(lr_network_top_matrix %>% t(), method = "binary")
hclust_ligands = hclust(dist_ligands, method = "ward.D2")
order_ligands_receptor = hclust_ligands$labels[hclust_ligands$order]

order_receptors = order_receptors %>% intersect(rownames(lr_network_top_matrix))
order_ligands_receptor = order_ligands_receptor %>% intersect(colnames(lr_network_top_matrix))

vis_ligand_receptor_network = lr_network_top_matrix[order_receptors, order_ligands_receptor]
rownames(vis_ligand_receptor_network) = order_receptors %>% make.names()
colnames(vis_ligand_receptor_network) = order_ligands_receptor %>% make.names()


p_ligand_receptor_network = vis_ligand_receptor_network %>% t() %>% make_heatmap_ggplot("Ligands","Receptors", color = "mediumvioletred", x_axis_position = "top",legend_title = "Prior interaction potential")
p_ligand_receptor_network

# https://github.com/saeyslab/nichenetr/blob/master/vignettes/seurat_steps.md#receptors-of-top-ranked-ligands-but-after-considering-only-bona-fide-ligand-receptor-interactions-documented-in-literature-and-publicly-available-databases
lr_network_strict = lr_network %>% filter(database != "ppi_prediction_go" & database != "ppi_prediction")
ligands_bona_fide = lr_network_strict %>% pull(from) %>% unique()
receptors_bona_fide = lr_network_strict %>% pull(to) %>% unique()

lr_network_top_df_large_strict = lr_network_top_df_large %>% distinct(from,to) %>% inner_join(lr_network_strict, by = c("from","to")) %>% distinct(from,to)
lr_network_top_df_large_strict = lr_network_top_df_large_strict %>% inner_join(lr_network_top_df_large, by = c("from","to"))

lr_network_top_df_strict = lr_network_top_df_large_strict %>% spread("from","weight",fill = 0)
lr_network_top_matrix_strict = lr_network_top_df_strict %>% select(-to) %>% as.matrix() %>% magrittr::set_rownames(lr_network_top_df_strict$to)

dist_receptors = dist(lr_network_top_matrix_strict, method = "binary")
hclust_receptors = hclust(dist_receptors, method = "ward.D2")
order_receptors = hclust_receptors$labels[hclust_receptors$order]

dist_ligands = dist(lr_network_top_matrix_strict %>% t(), method = "binary")
hclust_ligands = hclust(dist_ligands, method = "ward.D2")
order_ligands_receptor = hclust_ligands$labels[hclust_ligands$order]

order_receptors = order_receptors %>% intersect(rownames(lr_network_top_matrix_strict))
order_ligands_receptor = order_ligands_receptor %>% intersect(colnames(lr_network_top_matrix_strict))

vis_ligand_receptor_network_strict = lr_network_top_matrix_strict[order_receptors, order_ligands_receptor]
rownames(vis_ligand_receptor_network_strict) = order_receptors %>% make.names()
colnames(vis_ligand_receptor_network_strict) = order_ligands_receptor %>% make.names()

p_ligand_receptor_network_strict = vis_ligand_receptor_network_strict %>% t() %>% make_heatmap_ggplot("Ligands","Receptors", color = "mediumvioletred", x_axis_position = "top",legend_title = "Prior interaction potential\n(bona fide)")
p_ligand_receptor_network_strict

# https://github.com/saeyslab/nichenetr/blob/master/vignettes/seurat_steps.md#6-add-log-fold-change-information-of-ligands-from-sender-cells
# DE analysis for each sender cell type
# this uses a new nichenetr function - reinstall nichenetr if necessary!
DE_table_all = Idents(seuratObj) %>% levels() %>% intersect(sender_celltypes) %>% lapply(get_lfc_celltype, seurat_obj = seuratObj, condition_colname = "stim", condition_oi = condition_oi, condition_reference = condition_reference, expression_pct = 0.01, celltype_col = NULL) %>% reduce(full_join) 
# use this if cell type labels are the identities of your Seurat object -- if not: indicate the celltype_col properly
DE_table_all[is.na(DE_table_all)] = 0

# Combine ligand activities with DE information
ligand_activities_de = ligand_activities %>% select(test_ligand, pearson) %>% rename(ligand = test_ligand) %>% left_join(DE_table_all %>% rename(ligand = gene))
ligand_activities_de[is.na(ligand_activities_de)] = 0

# make LFC heatmap
lfc_matrix = ligand_activities_de  %>% select(-ligand, -pearson) %>% as.matrix() %>% magrittr::set_rownames(ligand_activities_de$ligand)
rownames(lfc_matrix) = rownames(lfc_matrix) %>% make.names()

order_ligands = order_ligands[order_ligands %in% rownames(lfc_matrix)]
vis_ligand_lfc = lfc_matrix[order_ligands,]

colnames(vis_ligand_lfc) = vis_ligand_lfc %>% colnames() %>% make.names()

p_ligand_lfc = vis_ligand_lfc %>% make_threecolor_heatmap_ggplot("Prioritized ligands","LFC in Sender", low_color = "midnightblue",mid_color = "white", mid = median(vis_ligand_lfc), high_color = "red",legend_position = "top", x_axis_position = "top", legend_title = "LFC") + theme(axis.text.y = element_text(face = "italic"))
p_ligand_lfc

# change colors a bit to make them more stand out
p_ligand_lfc = p_ligand_lfc + scale_fill_gradientn(colors = c("midnightblue","blue", "grey95", "grey99","firebrick1","red"),values = c(0,0.1,0.2,0.25, 0.40, 0.7,1), limits = c(vis_ligand_lfc %>% min() - 0.1, vis_ligand_lfc %>% max() + 0.1))
p_ligand_lfc

# https://github.com/saeyslab/nichenetr/blob/master/vignettes/seurat_steps.md#7-summary-visualizations-of-the-nichenet-analysis
# ligand activity heatmap
ligand_pearson_matrix = ligand_activities %>% select(pearson) %>% as.matrix() %>% magrittr::set_rownames(ligand_activities$test_ligand)

rownames(ligand_pearson_matrix) = rownames(ligand_pearson_matrix) %>% make.names()
colnames(ligand_pearson_matrix) = colnames(ligand_pearson_matrix) %>% make.names()

vis_ligand_pearson = ligand_pearson_matrix[order_ligands, ] %>% as.matrix(ncol = 1) %>% magrittr::set_colnames("Pearson")
p_ligand_pearson = vis_ligand_pearson %>% make_heatmap_ggplot("Prioritized ligands","Ligand activity", color = "darkorange",legend_position = "top", x_axis_position = "top", legend_title = "Pearson correlation coefficient\ntarget gene prediction ability)") + theme(legend.text = element_text(size = 9))

# ligand expression Seurat dotplot
order_ligands_adapted = order_ligands
order_ligands_adapted[order_ligands_adapted == "H2.M3"] = "H2-M3" # cf required use of make.names for heatmap visualization | this is not necessary if these ligands are not in the list of prioritized ligands!
order_ligands_adapted[order_ligands_adapted == "H2.T23"] = "H2-T23" # cf required use of make.names for heatmap visualization | this is not necessary if these ligands are not in the list of prioritized ligands!
rotated_dotplot = DotPlot(seuratObj %>% subset(celltype %in% sender_celltypes), features = order_ligands_adapted, cols = "RdYlBu") + coord_flip() + theme(legend.text = element_text(size = 10), legend.title = element_text(size = 12)) # flip of coordinates necessary because we want to show ligands in the rows when combining all plots

figures_without_legend = cowplot::plot_grid(
  p_ligand_pearson + theme(legend.position = "none", axis.ticks = element_blank()) + theme(axis.title.x = element_text()),
  rotated_dotplot + theme(legend.position = "none", axis.ticks = element_blank(), axis.title.x = element_text(size = 12), axis.text.y = element_text(face = "italic", size = 9), axis.text.x = element_text(size = 9,  angle = 90,hjust = 0)) + ylab("Expression in Sender") + xlab("") + scale_y_discrete(position = "right"),
  p_ligand_lfc + theme(legend.position = "none", axis.ticks = element_blank()) + theme(axis.title.x = element_text()) + ylab(""),
  p_ligand_target_network + theme(legend.position = "none", axis.ticks = element_blank()) + ylab(""),
  align = "hv",
  nrow = 1,
  rel_widths = c(ncol(vis_ligand_pearson)+6, ncol(vis_ligand_lfc) + 7, ncol(vis_ligand_lfc) + 8, ncol(vis_ligand_target)))

legends = cowplot::plot_grid(
  ggpubr::as_ggplot(ggpubr::get_legend(p_ligand_pearson)),
  ggpubr::as_ggplot(ggpubr::get_legend(rotated_dotplot)),
  ggpubr::as_ggplot(ggpubr::get_legend(p_ligand_lfc)),
  ggpubr::as_ggplot(ggpubr::get_legend(p_ligand_target_network)),
  nrow = 1,
  align = "h", rel_widths = c(1.5, 1, 1, 1))

combined_plot = cowplot::plot_grid(figures_without_legend, legends, rel_heights = c(10,5), nrow = 2, align = "hv")
combined_plot


# : Niche Circos ----
# indicated cell types should be cell class identities
# check via: 
# seuratObj %>% Idents() %>% table()
rm(nichenet_output)
sender_celltypes = c( "CD8 T cells",     "CD4 Naive",    "CD4 Memory" )
seuratObj@active.assay = "RNA"
nichenet_output = nichenet_seuratobj_aggregate(
  seurat_obj = seuratObj, 
  receiver = "Microglia", 
  condition_colname = "stim", condition_oi = "AD", condition_reference = "CTRL", 
  sender = sender_celltypes, 
  lfc_cutoff = 0.01,
  
  ligand_target_matrix = ligand_target_matrix, lr_network = lr_network, weighted_networks = weighted_networks, organism = "human")

nichenet_output$ligand_activities

DotPlot(seuratObj, features = nichenet_output$top_ligands %>% rev(), cols = "RdYlBu") + RotatedAxis()

DotPlot(seuratObj, features = nichenet_output$top_ligands %>% rev(), split.by = "stim") + RotatedAxis()
VlnPlot(seuratObj, features = c("Il15", "Cxcl10","Cxcl16"), split.by = "stim", pt.size = 0, combine = FALSE)
nichenet_output$ligand_target_heatmap
??VlnPlot


# Circos
# avg_expression_ligands = AverageExpression(seuratObj %>% subset(subset = aggregate == "LCMV"),features = nichenet_output$top_ligands) # if want to look specifically in LCMV-only cells
avg_expression_ligands = AverageExpression(seuratObj, features = nichenet_output$top_ligands)

sender_ligand_assignment = avg_expression_ligands$RNA %>% apply(1, function(ligand_expression){
  ligand_expression > (ligand_expression %>% mean() + ligand_expression %>% sd())
}) %>% t()
sender_ligand_assignment = sender_ligand_assignment %>% apply(2, function(x){x[x == TRUE]}) %>% purrr::keep(function(x){length(x) > 0})
names(sender_ligand_assignment)
# "Microglia"     "CD8 Cytotoxic" "CD4 Memory"

all_assigned_ligands = sender_ligand_assignment %>% lapply(function(x){names(x)}) %>% unlist()
unique_ligands = all_assigned_ligands %>% table() %>% .[. == 1] %>% names()
general_ligands = nichenet_output$top_ligands %>% setdiff(unique_ligands)

CD8_specific_ligands = sender_ligand_assignment[["CD8 T cells"]] %>% names() %>% setdiff(general_ligands)
Mic_specific_ligands = sender_ligand_assignment[["Microglia"]] %>% names() %>% setdiff(general_ligands)
CD4_specific_ligands = sender_ligand_assignment[["CD4 Naive"]] %>% names() %>% setdiff(general_ligands)


ligand_type_indication_df = tibble(
  ligand_type = c(rep("CD4-specific", times = CD4_specific_ligands %>% length()),
                  rep("Microglia-specific", times = Mic_specific_ligands %>% length()),
                  rep("CD8-specific", times = CD8_specific_ligands %>% length()),
                  rep("General", times = general_ligands %>% length())),
  ligand = c(CD8_specific_ligands, Mic_specific_ligands, CD4_specific_ligands, general_ligands))


active_ligand_target_links_df = nichenet_output$ligand_target_df %>% mutate(target_type = "AD-DE") %>% inner_join(ligand_type_indication_df) 
# if you want ot make circos plots for multiple gene sets, combine the different data frames and differentiate which target belongs to which gene set via the target type

cutoff_include_all_ligands = active_ligand_target_links_df$weight # %>% quantile(0.99)

active_ligand_target_links_df_circos = active_ligand_target_links_df # %>% filter(weight > cutoff_include_all_ligands)

ligands_to_remove = setdiff(active_ligand_target_links_df$ligand %>% unique(), active_ligand_target_links_df_circos$ligand %>% unique())
targets_to_remove = setdiff(active_ligand_target_links_df$target %>% unique(), active_ligand_target_links_df_circos$target %>% unique())

circos_links = active_ligand_target_links_df %>% filter(!target %in% targets_to_remove &!ligand %in% ligands_to_remove)

grid_col_ligand =c("General" = "lawngreen",
                   "CD4-specific" = "royalblue",
                   "Microglia-specific" = "darkgreen",
                   "CD8-specific" = "violet")
grid_col_target =c(
  "AD-DE" = "tomato")

grid_col_tbl_ligand = tibble(ligand_type = grid_col_ligand %>% names(), color_ligand_type = grid_col_ligand)
grid_col_tbl_target = tibble(target_type = grid_col_target %>% names(), color_target_type = grid_col_target)

circos_links = circos_links %>% mutate(ligand = paste(ligand," ")) # extra space: make a difference between a gene as ligand and a gene as target!
circos_links = circos_links %>% inner_join(grid_col_tbl_ligand) %>% inner_join(grid_col_tbl_target)
links_circle = circos_links %>% select(ligand,target, weight)

ligand_color = circos_links %>% distinct(ligand,color_ligand_type)
grid_ligand_color = ligand_color$color_ligand_type %>% set_names(ligand_color$ligand)
target_color = circos_links %>% distinct(target,color_target_type)
grid_target_color = target_color$color_target_type %>% set_names(target_color$target)

grid_col =c(grid_ligand_color,grid_target_color)

# give the option that links in the circos plot will be transparant ~ ligand-target potential score
transparency = circos_links %>% mutate(weight =(weight-min(weight))/(max(weight)-min(weight))) %>% mutate(transparency = 1-weight) %>% .$transparency 

target_order = circos_links$target %>% unique()
# ligand_order = c(CD8_specific_ligands, Mic_specific_ligands, CD4_specific_ligands ,  general_ligands) %>% c(paste(.," ")) %>% intersect(circos_links$ligand)
ligand_order = c(CD8_specific_ligands, Mic_specific_ligands, CD4_specific_ligands ,  general_ligands)
order = c(ligand_order,target_order)


width_same_cell_same_ligand_type = 0.5
width_different_cell = 5
width_ligand_target = 15
width_same_cell_same_target_type = 0.5



gaps = c(
  rep(width_same_cell_same_ligand_type, times = (circos_links %>% filter(ligand_type == "CD8-specific") %>% distinct(ligand) %>% nrow() -1)),
  width_different_cell,
  rep(width_same_cell_same_ligand_type, times = (circos_links %>% filter(ligand_type == "Microglia-specific") %>% distinct(ligand) %>% nrow() -1)),
  width_different_cell,
  rep(width_same_cell_same_ligand_type, times = (circos_links %>% filter(ligand_type == "General") %>% distinct(ligand) %>% nrow() -1)),
  width_ligand_target,
  rep(width_same_cell_same_target_type, times = (circos_links %>% filter(target_type == "AD-DE") %>% distinct(target) %>% nrow() -1)),
  width_ligand_target
)


# plot
circos.par(gap.degree = gaps)
chordDiagram(links_circle, directional = 1,order=order,link.sort = TRUE, link.decreasing = FALSE, grid.col = grid_col,transparency = 0, diffHeight = 0.005, direction.type = c("diffHeight", "arrows"),link.arr.type = "big.arrow", link.visible = links_circle$weight >= cutoff_include_all_ligands,annotationTrack = "grid", 
             preAllocateTracks = list(track.height = 0.075))
# we go back to the first track and customize sector labels
circos.track(track.index = 1, panel.fun = function(x, y) {
  circos.text(CELL_META$xcenter, CELL_META$ylim[1], CELL_META$sector.index,
              facing = "clockwise", niceFacing = TRUE, adj = c(0, 0.55), cex = 0.8)
}, bg.border = NA) #



circos.par(gap.degree = gaps)
chordDiagram(links_circle, directional = 1,order=order,link.sort = TRUE, link.decreasing = FALSE, grid.col = grid_col,transparency = transparency, diffHeight = 0.005, direction.type = c("diffHeight", "arrows"),link.arr.type = "big.arrow", link.visible = links_circle$weight >= cutoff_include_all_ligands,annotationTrack = "grid", 
             preAllocateTracks = list(track.height = 0.075))
# we go back to the first track and customize sector labels
circos.track(track.index = 1, panel.fun = function(x, y) {
  circos.text(CELL_META$xcenter, CELL_META$ylim[1], CELL_META$sector.index,
              facing = "clockwise", niceFacing = TRUE, adj = c(0, 0.55), cex = 0.8)
}, bg.border = NA) #

# : ligand-receptor ----
# https://github.com/saeyslab/nichenetr/blob/master/vignettes/circos.md#visualize-ligand-receptor-interactions-of-the-prioritized-ligands-in-a-circos-plot
# get the ligand-receptor network of the top-ranked ligands
lr_network_top = lr_network %>% filter(from %in% best_upstream_ligands & to %in% expressed_receptors) %>% distinct(from,to)
best_upstream_receptors = lr_network_top %>% pull(to) %>% unique()

# get the weights of the ligand-receptor interactions as used in the NicheNet model
weighted_networks = readRDS(url("https://zenodo.org/record/3260758/files/weighted_networks.rds"))
lr_network_top_df = weighted_networks$lr_sig %>% filter(from %in% best_upstream_ligands & to %in% best_upstream_receptors)  %>% rename(ligand = from, receptor = to)

lr_network_top_df = lr_network_top_df %>% mutate(receptor_type = "p_emt_receptor") %>% inner_join(ligand_type_indication_df)


grid_col_ligand =c("General" = "lawngreen",
                   "CD4-specific" = "royalblue",
                   "Microglia-specific" = "darkgreen",
                   "CD8-specific" = "violet")
grid_col_receptor =c(
  "p_emt_receptor" = "darkred")

grid_col_tbl_ligand = tibble(ligand_type = grid_col_ligand %>% names(), color_ligand_type = grid_col_ligand)
grid_col_tbl_receptor = tibble(receptor_type = grid_col_receptor %>% names(), color_receptor_type = grid_col_receptor)

circos_links = lr_network_top_df %>% mutate(ligand = paste(ligand," ")) # extra space: make a difference between a gene as ligand and a gene as receptor!
circos_links = circos_links %>% inner_join(grid_col_tbl_ligand) %>% inner_join(grid_col_tbl_receptor)
links_circle = circos_links %>% select(ligand,receptor, weight)

ligand_color = circos_links %>% distinct(ligand,color_ligand_type)
grid_ligand_color = ligand_color$color_ligand_type %>% set_names(ligand_color$ligand)
receptor_color = circos_links %>% distinct(receptor,color_receptor_type)
grid_receptor_color = receptor_color$color_receptor_type %>% set_names(receptor_color$receptor)

grid_col =c(grid_ligand_color,grid_receptor_color)

# give the option that links in the circos plot will be transparant ~ ligand-receptor potential score
transparency = circos_links %>% mutate(weight =(weight-min(weight))/(max(weight)-min(weight))) %>% mutate(transparency = 1-weight) %>% .$transparency 


receptor_order = circos_links$receptor %>% unique()
ligand_order = c(Neu_specific_ligands,Ast_specific_ligands, Mic_specific_ligands,general_ligands) %>% c(paste(.," ")) %>% intersect(circos_links$ligand)
order = c(ligand_order,receptor_order)



width_same_cell_same_ligand_type = 0.5
width_different_cell = 6
width_ligand_receptor = 15
width_same_cell_same_receptor_type = 0.5

gaps = c(
  # width_ligand_receptor,
  rep(width_same_cell_same_ligand_type, times = (circos_links %>% filter(ligand_type == "General") %>% distinct(ligand) %>% nrow() -1)),
  width_different_cell,
  rep(width_same_cell_same_ligand_type, times = (circos_links %>% filter(ligand_type == "Neuron-specific") %>% distinct(ligand) %>% nrow() -1)), 
  width_ligand_receptor,
  rep(width_same_cell_same_ligand_type, times = (circos_links %>% filter(ligand_type == "Microglia-specific") %>% distinct(ligand) %>% nrow() -1)), 
  width_ligand_receptor,
  rep(width_same_cell_same_ligand_type, times = (circos_links %>% filter(ligand_type == "Astrocyte-specific") %>% distinct(ligand) %>% nrow() -1)), 
  width_ligand_receptor,
  rep(width_same_cell_same_receptor_type, times = (circos_links %>% filter(receptor_type == "p_emt_receptor") %>% distinct(receptor) %>% nrow() -1)),
  width_ligand_receptor
)

circos.par(gap.degree = gaps)
chordDiagram(links_circle, directional = 1,order=order,link.sort = TRUE, link.decreasing = FALSE, grid.col = grid_col,transparency = 0, diffHeight = 0.005, direction.type = c("diffHeight", "arrows"),link.arr.type = "big.arrow", link.visible = links_circle$weight >= cutoff_include_all_ligands,annotationTrack = "grid", 
             preAllocateTracks = list(track.height = 0.075))
# we go back to the first track and customize sector labels
circos.track(track.index = 1, panel.fun = function(x, y) {
  circos.text(CELL_META$xcenter, CELL_META$ylim[1], CELL_META$sector.index,
              facing = "clockwise", niceFacing = TRUE, adj = c(0, 0.55), cex = 0.8)
}, bg.border = NA) #


circos.par(gap.degree = gaps)
chordDiagram(links_circle, directional = 1,order=order,link.sort = TRUE, link.decreasing = FALSE, grid.col = grid_col,transparency = transparency, diffHeight = 0.005, direction.type = c("diffHeight", "arrows"),link.arr.type = "big.arrow", link.visible = links_circle$weight >= cutoff_include_all_ligands,annotationTrack = "grid", 
             preAllocateTracks = list(track.height = 0.075))
# we go back to the first track and customize sector labels
circos.track(track.index = 1, panel.fun = function(x, y) {
  circos.text(CELL_META$xcenter, CELL_META$ylim[1], CELL_META$sector.index,
              facing = "clockwise", niceFacing = TRUE, adj = c(0, 0.55), cex = 0.8)
}, bg.border = NA) #


# Nichenet mic.neu.ast.T ----
library(nichenetr)
library(Seurat) # please update to Seurat V4
library(tidyverse)
library(circlize)
DimPlot(pichip.mt)
test <- pichip.mt@active.ident
test <- Idents(pichip.mt)

View(as.data.frame(test))
meta.cluster.name <- as.data.frame(Idents(pichip.mt))
rm(test)

# orignal seurat rowname check
rownames(pichip.mt@meta.data)

# orignal seurat rowname = cell.type rowname check
identical(rownames(pichip.mt@meta.data), rownames(meta.cluster.name))

pichip.mt <- AddMetaData(pichip.mt, meta.cluster.name, col.name = 'celltype')



seuratObj <- pichip.mt
seuratObj@meta.data %>% head()

seuratObj@meta.data$celltype %>% table() 
## .
##     Astrocyte   T cells Microglia    Neuron 
##          2386      1551      1201       102 
DimPlot(seuratObj, reduction = "umap")

ligand_target_matrix = readRDS(url("https://zenodo.org/record/3260758/files/ligand_target_matrix.rds"))
ligand_target_matrix[1:5,1:5] # target genes in rows, ligands in columns

lr_network = readRDS(url("https://zenodo.org/record/3260758/files/lr_network.rds"))
head(lr_network)

weighted_networks = readRDS(url("https://zenodo.org/record/3260758/files/weighted_networks.rds"))
weighted_networks_lr = weighted_networks$lr_sig %>% inner_join(lr_network %>% distinct(from,to), by = c("from","to"))

head(weighted_networks$lr_sig) # interactions and their weights in the ligand-receptor + signaling network

head(weighted_networks$gr) # interactions and their weights in the gene regulatory network

# : Niche define receiver/sender ----
## receiver
receiver = "T cells"
expressed_genes_receiver@active.assay = "RNA"
expressed_genes_receiver = get_expressed_genes(receiver, seuratObj, pct = 0.01)

background_expressed_genes = expressed_genes_receiver %>% .[. %in% rownames(ligand_target_matrix)]

## sender
sender_celltypes = c("Astrocyte","Microglia", "Neuron")
seurat_obj_receiver@active.assay = "RNA"
list_expressed_genes_sender = sender_celltypes %>% unique() %>% lapply(get_expressed_genes, seuratObj, 0.01) # lapply to get the expressed genes of every sender cell type separately here
expressed_genes_sender = list_expressed_genes_sender %>% unlist() %>% unique()

seurat_obj_receiver= subset(seuratObj, idents = receiver)
seurat_obj_receiver = SetIdent(seurat_obj_receiver, value = seurat_obj_receiver[["stim"]])

condition_oi = "AD"
condition_reference = "CTRL" 

seurat_obj_receiver@active.assay = "RNA"
DE_table_receiver = FindMarkers(object = seurat_obj_receiver, ident.1 = condition_oi, ident.2 = condition_reference, min.pct = 0.01) %>% rownames_to_column("gene")

geneset_oi = DE_table_receiver %>% filter(p_val_adj <= 0.2 & abs(avg_log2FC) >= 0.01) %>% pull(gene)
geneset_oi = geneset_oi %>% .[. %in% rownames(ligand_target_matrix)]

ligands = lr_network %>% pull(from) %>% unique()
receptors = lr_network %>% pull(to) %>% unique()

expressed_ligands = intersect(ligands,expressed_genes_sender)
expressed_receptors = intersect(receptors,expressed_genes_receiver)

potential_ligands = lr_network %>% filter(from %in% expressed_ligands & to %in% expressed_receptors) %>% pull(from) %>% unique()

ligand_activities = predict_ligand_activities(geneset = geneset_oi, background_expressed_genes = background_expressed_genes, ligand_target_matrix = ligand_target_matrix, potential_ligands = potential_ligands)

ligand_activities = ligand_activities %>% arrange(-pearson) %>% mutate(rank = rank(desc(pearson)))
ligand_activities

best_upstream_ligands = ligand_activities %>% top_n(170, pearson) %>% arrange(-pearson) %>% pull(test_ligand) %>% unique()

DotPlot(seuratObj, features = best_upstream_ligands %>% rev(), cols = "RdYlBu") + RotatedAxis()


# https://github.com/saeyslab/nichenetr/blob/master/vignettes/seurat_steps.md#active-target-gene-inference
active_ligand_target_links_df = best_upstream_ligands %>% lapply(get_weighted_ligand_target_links,geneset = geneset_oi, ligand_target_matrix = ligand_target_matrix, n = 200) %>% bind_rows() %>% drop_na()

active_ligand_target_links = prepare_ligand_target_visualization(ligand_target_df = active_ligand_target_links_df, ligand_target_matrix = ligand_target_matrix, cutoff = 0.33)

order_ligands = intersect(best_upstream_ligands, colnames(active_ligand_target_links)) %>% rev() %>% make.names()
order_targets = active_ligand_target_links_df$target %>% unique() %>% intersect(rownames(active_ligand_target_links)) %>% make.names()
rownames(active_ligand_target_links) = rownames(active_ligand_target_links) %>% make.names() # make.names() for heatmap visualization of genes like H2-T23
colnames(active_ligand_target_links) = colnames(active_ligand_target_links) %>% make.names() # make.names() for heatmap visualization of genes like H2-T23

vis_ligand_target = active_ligand_target_links[order_targets,order_ligands] %>% t()

p_ligand_target_network = vis_ligand_target %>% make_heatmap_ggplot("Prioritized ligands","Predicted target genes", color = "purple",legend_position = "top", x_axis_position = "top",legend_title = "Regulatory potential")  + theme(axis.text.x = element_text(face = "italic")) + scale_fill_gradient2(low = "whitesmoke",  high = "purple", breaks = c(0,0.0045,0.0090))
p_ligand_target_network

# https://github.com/saeyslab/nichenetr/blob/master/vignettes/seurat_steps.md#receptors-of-top-ranked-ligands
lr_network_top = lr_network %>% filter(from %in% best_upstream_ligands & to %in% expressed_receptors) %>% distinct(from,to)
best_upstream_receptors = lr_network_top %>% pull(to) %>% unique()

lr_network_top_df_large = weighted_networks_lr %>% filter(from %in% best_upstream_ligands & to %in% best_upstream_receptors)

lr_network_top_df = lr_network_top_df_large %>% spread("from","weight",fill = 0)
lr_network_top_matrix = lr_network_top_df %>% select(-to) %>% as.matrix() %>% magrittr::set_rownames(lr_network_top_df$to)

dist_receptors = dist(lr_network_top_matrix, method = "binary")
hclust_receptors = hclust(dist_receptors, method = "ward.D2")
order_receptors = hclust_receptors$labels[hclust_receptors$order]

dist_ligands = dist(lr_network_top_matrix %>% t(), method = "binary")
hclust_ligands = hclust(dist_ligands, method = "ward.D2")
order_ligands_receptor = hclust_ligands$labels[hclust_ligands$order]

order_receptors = order_receptors %>% intersect(rownames(lr_network_top_matrix))
order_ligands_receptor = order_ligands_receptor %>% intersect(colnames(lr_network_top_matrix))

vis_ligand_receptor_network = lr_network_top_matrix[order_receptors, order_ligands_receptor]
rownames(vis_ligand_receptor_network) = order_receptors %>% make.names()
colnames(vis_ligand_receptor_network) = order_ligands_receptor %>% make.names()


p_ligand_receptor_network = vis_ligand_receptor_network %>% t() %>% make_heatmap_ggplot("Ligands","Receptors", color = "mediumvioletred", x_axis_position = "top",legend_title = "Prior interaction potential")
p_ligand_receptor_network

# https://github.com/saeyslab/nichenetr/blob/master/vignettes/seurat_steps.md#receptors-of-top-ranked-ligands-but-after-considering-only-bona-fide-ligand-receptor-interactions-documented-in-literature-and-publicly-available-databases
lr_network_strict = lr_network %>% filter(database != "ppi_prediction_go" & database != "ppi_prediction")
ligands_bona_fide = lr_network_strict %>% pull(from) %>% unique()
receptors_bona_fide = lr_network_strict %>% pull(to) %>% unique()

lr_network_top_df_large_strict = lr_network_top_df_large %>% distinct(from,to) %>% inner_join(lr_network_strict, by = c("from","to")) %>% distinct(from,to)
lr_network_top_df_large_strict = lr_network_top_df_large_strict %>% inner_join(lr_network_top_df_large, by = c("from","to"))

lr_network_top_df_strict = lr_network_top_df_large_strict %>% spread("from","weight",fill = 0)
lr_network_top_matrix_strict = lr_network_top_df_strict %>% select(-to) %>% as.matrix() %>% magrittr::set_rownames(lr_network_top_df_strict$to)

dist_receptors = dist(lr_network_top_matrix_strict, method = "binary")
hclust_receptors = hclust(dist_receptors, method = "ward.D2")
order_receptors = hclust_receptors$labels[hclust_receptors$order]

dist_ligands = dist(lr_network_top_matrix_strict %>% t(), method = "binary")
hclust_ligands = hclust(dist_ligands, method = "ward.D2")
order_ligands_receptor = hclust_ligands$labels[hclust_ligands$order]

order_receptors = order_receptors %>% intersect(rownames(lr_network_top_matrix_strict))
order_ligands_receptor = order_ligands_receptor %>% intersect(colnames(lr_network_top_matrix_strict))

vis_ligand_receptor_network_strict = lr_network_top_matrix_strict[order_receptors, order_ligands_receptor]
rownames(vis_ligand_receptor_network_strict) = order_receptors %>% make.names()
colnames(vis_ligand_receptor_network_strict) = order_ligands_receptor %>% make.names()

p_ligand_receptor_network_strict = vis_ligand_receptor_network_strict %>% t() %>% make_heatmap_ggplot("Ligands","Receptors", color = "mediumvioletred", x_axis_position = "top",legend_title = "Prior interaction potential\n(bona fide)")
p_ligand_receptor_network_strict

# https://github.com/saeyslab/nichenetr/blob/master/vignettes/seurat_steps.md#6-add-log-fold-change-information-of-ligands-from-sender-cells
# DE analysis for each sender cell type
# this uses a new nichenetr function - reinstall nichenetr if necessary!
DE_table_all = Idents(seuratObj) %>% levels() %>% intersect(sender_celltypes) %>% lapply(get_lfc_celltype, seurat_obj = seuratObj, condition_colname = "stim", condition_oi = condition_oi, condition_reference = condition_reference, expression_pct = 0.01, celltype_col = NULL) %>% reduce(full_join) 
# use this if cell type labels are the identities of your Seurat object -- if not: indicate the celltype_col properly
DE_table_all[is.na(DE_table_all)] = 0

# Combine ligand activities with DE information
ligand_activities_de = ligand_activities %>% select(test_ligand, pearson) %>% rename(ligand = test_ligand) %>% left_join(DE_table_all %>% rename(ligand = gene))
ligand_activities_de[is.na(ligand_activities_de)] = 0

# make LFC heatmap
lfc_matrix = ligand_activities_de  %>% select(-ligand, -pearson) %>% as.matrix() %>% magrittr::set_rownames(ligand_activities_de$ligand)
rownames(lfc_matrix) = rownames(lfc_matrix) %>% make.names()

order_ligands = order_ligands[order_ligands %in% rownames(lfc_matrix)]
vis_ligand_lfc = lfc_matrix[order_ligands,]

colnames(vis_ligand_lfc) = vis_ligand_lfc %>% colnames() %>% make.names()

p_ligand_lfc = vis_ligand_lfc %>% make_threecolor_heatmap_ggplot("Prioritized ligands","LFC in Sender", low_color = "midnightblue",mid_color = "white", mid = median(vis_ligand_lfc), high_color = "red",legend_position = "top", x_axis_position = "top", legend_title = "LFC") + theme(axis.text.y = element_text(face = "italic"))
p_ligand_lfc

# change colors a bit to make them more stand out
p_ligand_lfc = p_ligand_lfc + scale_fill_gradientn(colors = c("midnightblue","blue", "grey95", "grey99","firebrick1","red"),values = c(0,0.1,0.2,0.25, 0.40, 0.7,1), limits = c(vis_ligand_lfc %>% min() - 0.1, vis_ligand_lfc %>% max() + 0.1))
p_ligand_lfc

# https://github.com/saeyslab/nichenetr/blob/master/vignettes/seurat_steps.md#7-summary-visualizations-of-the-nichenet-analysis
# ligand activity heatmap
ligand_pearson_matrix = ligand_activities %>% select(pearson) %>% as.matrix() %>% magrittr::set_rownames(ligand_activities$test_ligand)

rownames(ligand_pearson_matrix) = rownames(ligand_pearson_matrix) %>% make.names()
colnames(ligand_pearson_matrix) = colnames(ligand_pearson_matrix) %>% make.names()

vis_ligand_pearson = ligand_pearson_matrix[order_ligands, ] %>% as.matrix(ncol = 1) %>% magrittr::set_colnames("Pearson")
p_ligand_pearson = vis_ligand_pearson %>% make_heatmap_ggplot("Prioritized ligands","Ligand activity", color = "darkorange",legend_position = "top", x_axis_position = "top", legend_title = "Pearson correlation coefficient\ntarget gene prediction ability)") + theme(legend.text = element_text(size = 9))

# ligand expression Seurat dotplot
order_ligands_adapted = order_ligands
order_ligands_adapted[order_ligands_adapted == "H2.M3"] = "H2-M3" # cf required use of make.names for heatmap visualization | this is not necessary if these ligands are not in the list of prioritized ligands!
order_ligands_adapted[order_ligands_adapted == "H2.T23"] = "H2-T23" # cf required use of make.names for heatmap visualization | this is not necessary if these ligands are not in the list of prioritized ligands!
rotated_dotplot = DotPlot(seuratObj %>% subset(celltype %in% sender_celltypes), features = order_ligands_adapted, cols = "RdYlBu") + coord_flip() + theme(legend.text = element_text(size = 10), legend.title = element_text(size = 12)) # flip of coordinates necessary because we want to show ligands in the rows when combining all plots

figures_without_legend = cowplot::plot_grid(
  p_ligand_pearson + theme(legend.position = "none", axis.ticks = element_blank()) + theme(axis.title.x = element_text()),
  rotated_dotplot + theme(legend.position = "none", axis.ticks = element_blank(), axis.title.x = element_text(size = 12), axis.text.y = element_text(face = "italic", size = 9), axis.text.x = element_text(size = 9,  angle = 90,hjust = 0)) + ylab("Expression in Sender") + xlab("") + scale_y_discrete(position = "right"),
  p_ligand_lfc + theme(legend.position = "none", axis.ticks = element_blank()) + theme(axis.title.x = element_text()) + ylab(""),
  p_ligand_target_network + theme(legend.position = "none", axis.ticks = element_blank()) + ylab(""),
  align = "hv",
  nrow = 1,
  rel_widths = c(ncol(vis_ligand_pearson)+6, ncol(vis_ligand_lfc) + 7, ncol(vis_ligand_lfc) + 8, ncol(vis_ligand_target)))

legends = cowplot::plot_grid(
  ggpubr::as_ggplot(ggpubr::get_legend(p_ligand_pearson)),
  ggpubr::as_ggplot(ggpubr::get_legend(rotated_dotplot)),
  ggpubr::as_ggplot(ggpubr::get_legend(p_ligand_lfc)),
  ggpubr::as_ggplot(ggpubr::get_legend(p_ligand_target_network)),
  nrow = 1,
  align = "h", rel_widths = c(1.5, 1, 1, 1))

combined_plot = cowplot::plot_grid(figures_without_legend, legends, rel_heights = c(10,5), nrow = 2, align = "hv")
combined_plot


# : Niche Circos ----
# indicated cell types should be cell class identities
# check via: 
# seuratObj %>% Idents() %>% table()
rm(nichenet_output)
sender_celltypes = c("Astrocyte","Microglia", "Neuron")
seuratObj@active.assay = "RNA"
nichenet_output = nichenet_seuratobj_aggregate(
  seurat_obj = seuratObj, 
  receiver = "T cells", 
  condition_colname = "stim", condition_oi = "AD", condition_reference = "CTRL", 
  sender = sender_celltypes, 
  lfc_cutoff = 0.01,

  ligand_target_matrix = ligand_target_matrix, lr_network = lr_network, weighted_networks = weighted_networks, organism = "human")

nichenet_output$ligand_activities

DotPlot(seuratObj, features = nichenet_output$top_ligands %>% rev(), cols = "RdYlBu") + RotatedAxis()

DotPlot(seuratObj, features = nichenet_output$top_ligands %>% rev(), split.by = "stim") + RotatedAxis()
VlnPlot(seuratObj, features = c("Il15", "Cxcl10","Cxcl16"), split.by = "stim", pt.size = 0, combine = FALSE)
nichenet_output$ligand_target_heatmap
??VlnPlot


# Circos
# avg_expression_ligands = AverageExpression(seuratObj %>% subset(subset = aggregate == "LCMV"),features = nichenet_output$top_ligands) # if want to look specifically in LCMV-only cells
avg_expression_ligands = AverageExpression(seuratObj, features = nichenet_output$top_ligands)

sender_ligand_assignment = avg_expression_ligands$RNA %>% apply(1, function(ligand_expression){
  ligand_expression > (ligand_expression %>% mean() + ligand_expression %>% sd())
}) %>% t()
sender_ligand_assignment = sender_ligand_assignment %>% apply(2, function(x){x[x == TRUE]}) %>% purrr::keep(function(x){length(x) > 0})
names(sender_ligand_assignment)
# "Astrocyte" "T cells"   "Microglia" "Neuron"

all_assigned_ligands = sender_ligand_assignment %>% lapply(function(x){names(x)}) %>% unlist()
unique_ligands = all_assigned_ligands %>% table() %>% .[. == 1] %>% names()
general_ligands = nichenet_output$top_ligands %>% setdiff(unique_ligands)

Ast_specific_ligands = sender_ligand_assignment[["Astrocyte"]] %>% names() %>% setdiff(general_ligands)
Mic_specific_ligands = sender_ligand_assignment[["Microglia"]] %>% names() %>% setdiff(general_ligands)
Neu_specific_ligands = sender_ligand_assignment[["Neuron"]] %>% names() %>% setdiff(general_ligands)


ligand_type_indication_df = tibble(
  ligand_type = c(rep("Astrocyte-specific", times = Ast_specific_ligands %>% length()),
                  rep("Microglia-specific", times = Mic_specific_ligands %>% length()),
                  rep("Neuron-specific", times = Neu_specific_ligands %>% length()),
                  rep("General", times = general_ligands %>% length())),
  ligand = c(Ast_specific_ligands, Mic_specific_ligands, Neu_specific_ligands, general_ligands))


active_ligand_target_links_df = nichenet_output$ligand_target_df %>% mutate(target_type = "AD-DE") %>% inner_join(ligand_type_indication_df) 
# if you want ot make circos plots for multiple gene sets, combine the different data frames and differentiate which target belongs to which gene set via the target type

cutoff_include_all_ligands = active_ligand_target_links_df$weight # %>% quantile(0.99)

active_ligand_target_links_df_circos = active_ligand_target_links_df # %>% filter(weight > cutoff_include_all_ligands)

ligands_to_remove = setdiff(active_ligand_target_links_df$ligand %>% unique(), active_ligand_target_links_df_circos$ligand %>% unique())
targets_to_remove = setdiff(active_ligand_target_links_df$target %>% unique(), active_ligand_target_links_df_circos$target %>% unique())

circos_links = active_ligand_target_links_df %>% filter(!target %in% targets_to_remove &!ligand %in% ligands_to_remove)

grid_col_ligand =c("General" = "lawngreen",
                   "Astrocyte-specific" = "royalblue",
                   "Microglia-specific" = "darkgreen",
                   "Neuron-specific" = "violet")
grid_col_target =c(
  "AD-DE" = "tomato")

grid_col_tbl_ligand = tibble(ligand_type = grid_col_ligand %>% names(), color_ligand_type = grid_col_ligand)
grid_col_tbl_target = tibble(target_type = grid_col_target %>% names(), color_target_type = grid_col_target)

circos_links = circos_links %>% mutate(ligand = paste(ligand," ")) # extra space: make a difference between a gene as ligand and a gene as target!
circos_links = circos_links %>% inner_join(grid_col_tbl_ligand) %>% inner_join(grid_col_tbl_target)
links_circle = circos_links %>% select(ligand,target, weight)

ligand_color = circos_links %>% distinct(ligand,color_ligand_type)
grid_ligand_color = ligand_color$color_ligand_type %>% set_names(ligand_color$ligand)
target_color = circos_links %>% distinct(target,color_target_type)
grid_target_color = target_color$color_target_type %>% set_names(target_color$target)

grid_col =c(grid_ligand_color,grid_target_color)

# give the option that links in the circos plot will be transparant ~ ligand-target potential score
transparency = circos_links %>% mutate(weight =(weight-min(weight))/(max(weight)-min(weight))) %>% mutate(transparency = 1-weight) %>% .$transparency 

target_order = circos_links$target %>% unique()
ligand_order = c(Ast_specific_ligands, Mic_specific_ligands, Neu_specific_ligands,  general_ligands) %>% c(paste(.," ")) %>% intersect(circos_links$ligand)
order = c(ligand_order,target_order)


width_same_cell_same_ligand_type = 0.5
width_different_cell = 5
width_ligand_target = 15
width_same_cell_same_target_type = 0.5



gaps = c(
  rep(width_same_cell_same_ligand_type, times = (circos_links %>% filter(ligand_type == "Astrocyte-specific") %>% distinct(ligand) %>% nrow() -1)),
  width_different_cell,
  rep(width_same_cell_same_ligand_type, times = (circos_links %>% filter(ligand_type == "Microglia-specific") %>% distinct(ligand) %>% nrow() -1)),
  width_different_cell,
  rep(width_same_cell_same_ligand_type, times = (circos_links %>% filter(ligand_type == "Neuron-specific") %>% distinct(ligand) %>% nrow() -1)),
  width_different_cell,
  rep(width_same_cell_same_ligand_type, times = (circos_links %>% filter(ligand_type == "General") %>% distinct(ligand) %>% nrow() -1)),
  width_ligand_target,
  rep(width_same_cell_same_target_type, times = (circos_links %>% filter(target_type == "AD-DE") %>% distinct(target) %>% nrow() -1)),
  width_ligand_target
)


# plot
circos.par(gap.degree = gaps)
chordDiagram(links_circle, directional = 1,order=order,link.sort = TRUE, link.decreasing = FALSE, grid.col = grid_col,transparency = 0, diffHeight = 0.005, direction.type = c("diffHeight", "arrows"),link.arr.type = "big.arrow", link.visible = links_circle$weight >= cutoff_include_all_ligands,annotationTrack = "grid", 
             preAllocateTracks = list(track.height = 0.075))
# we go back to the first track and customize sector labels
circos.track(track.index = 1, panel.fun = function(x, y) {
  circos.text(CELL_META$xcenter, CELL_META$ylim[1], CELL_META$sector.index,
              facing = "clockwise", niceFacing = TRUE, adj = c(0, 0.55), cex = 0.8)
}, bg.border = NA) #



circos.par(gap.degree = gaps)
chordDiagram(links_circle, directional = 1,order=order,link.sort = TRUE, link.decreasing = FALSE, grid.col = grid_col,transparency = transparency, diffHeight = 0.005, direction.type = c("diffHeight", "arrows"),link.arr.type = "big.arrow", link.visible = links_circle$weight >= cutoff_include_all_ligands,annotationTrack = "grid", 
             preAllocateTracks = list(track.height = 0.075))
# we go back to the first track and customize sector labels
circos.track(track.index = 1, panel.fun = function(x, y) {
  circos.text(CELL_META$xcenter, CELL_META$ylim[1], CELL_META$sector.index,
              facing = "clockwise", niceFacing = TRUE, adj = c(0, 0.55), cex = 0.8)
}, bg.border = NA) #

# : ligand-receptor ----
# https://github.com/saeyslab/nichenetr/blob/master/vignettes/circos.md#visualize-ligand-receptor-interactions-of-the-prioritized-ligands-in-a-circos-plot
# get the ligand-receptor network of the top-ranked ligands
lr_network_top = lr_network %>% filter(from %in% best_upstream_ligands & to %in% expressed_receptors) %>% distinct(from,to)
best_upstream_receptors = lr_network_top %>% pull(to) %>% unique()

# get the weights of the ligand-receptor interactions as used in the NicheNet model
weighted_networks = readRDS(url("https://zenodo.org/record/3260758/files/weighted_networks.rds"))
lr_network_top_df = weighted_networks$lr_sig %>% filter(from %in% best_upstream_ligands & to %in% best_upstream_receptors)  %>% rename(ligand = from, receptor = to)

lr_network_top_df = lr_network_top_df %>% mutate(receptor_type = "p_emt_receptor") %>% inner_join(ligand_type_indication_df)


grid_col_ligand =c("General" = "lawngreen",
                   "Neuron-specific" = "royalblue",
                   "Astrocyte-specific" = "darkgreen",
                   "Microglia-specific" = "violet")
grid_col_receptor =c(
  "p_emt_receptor" = "darkred")

grid_col_tbl_ligand = tibble(ligand_type = grid_col_ligand %>% names(), color_ligand_type = grid_col_ligand)
grid_col_tbl_receptor = tibble(receptor_type = grid_col_receptor %>% names(), color_receptor_type = grid_col_receptor)

circos_links = lr_network_top_df %>% mutate(ligand = paste(ligand," ")) # extra space: make a difference between a gene as ligand and a gene as receptor!
circos_links = circos_links %>% inner_join(grid_col_tbl_ligand) %>% inner_join(grid_col_tbl_receptor)
links_circle = circos_links %>% select(ligand,receptor, weight)

ligand_color = circos_links %>% distinct(ligand,color_ligand_type)
grid_ligand_color = ligand_color$color_ligand_type %>% set_names(ligand_color$ligand)
receptor_color = circos_links %>% distinct(receptor,color_receptor_type)
grid_receptor_color = receptor_color$color_receptor_type %>% set_names(receptor_color$receptor)

grid_col =c(grid_ligand_color,grid_receptor_color)

# give the option that links in the circos plot will be transparant ~ ligand-receptor potential score
transparency = circos_links %>% mutate(weight =(weight-min(weight))/(max(weight)-min(weight))) %>% mutate(transparency = 1-weight) %>% .$transparency 


receptor_order = circos_links$receptor %>% unique()
ligand_order = c(Neu_specific_ligands,Ast_specific_ligands, Mic_specific_ligands,general_ligands) %>% c(paste(.," ")) %>% intersect(circos_links$ligand)
order = c(ligand_order,receptor_order)



width_same_cell_same_ligand_type = 0.5
width_different_cell = 6
width_ligand_receptor = 15
width_same_cell_same_receptor_type = 0.5

gaps = c(
  # width_ligand_receptor,
  rep(width_same_cell_same_ligand_type, times = (circos_links %>% filter(ligand_type == "General") %>% distinct(ligand) %>% nrow() -1)),
  width_different_cell,
  rep(width_same_cell_same_ligand_type, times = (circos_links %>% filter(ligand_type == "Neuron-specific") %>% distinct(ligand) %>% nrow() -1)), 
  width_ligand_receptor,
  rep(width_same_cell_same_ligand_type, times = (circos_links %>% filter(ligand_type == "Microglia-specific") %>% distinct(ligand) %>% nrow() -1)), 
  width_ligand_receptor,
  rep(width_same_cell_same_ligand_type, times = (circos_links %>% filter(ligand_type == "Astrocyte-specific") %>% distinct(ligand) %>% nrow() -1)), 
  width_ligand_receptor,
  rep(width_same_cell_same_receptor_type, times = (circos_links %>% filter(receptor_type == "p_emt_receptor") %>% distinct(receptor) %>% nrow() -1)),
  width_ligand_receptor
)

circos.par(gap.degree = gaps)
chordDiagram(links_circle, directional = 1,order=order,link.sort = TRUE, link.decreasing = FALSE, grid.col = grid_col,transparency = 0, diffHeight = 0.005, direction.type = c("diffHeight", "arrows"),link.arr.type = "big.arrow", link.visible = links_circle$weight >= cutoff_include_all_ligands,annotationTrack = "grid", 
             preAllocateTracks = list(track.height = 0.075))
# we go back to the first track and customize sector labels
circos.track(track.index = 1, panel.fun = function(x, y) {
  circos.text(CELL_META$xcenter, CELL_META$ylim[1], CELL_META$sector.index,
              facing = "clockwise", niceFacing = TRUE, adj = c(0, 0.55), cex = 0.8)
}, bg.border = NA) #


circos.par(gap.degree = gaps)
chordDiagram(links_circle, directional = 1,order=order,link.sort = TRUE, link.decreasing = FALSE, grid.col = grid_col,transparency = transparency, diffHeight = 0.005, direction.type = c("diffHeight", "arrows"),link.arr.type = "big.arrow", link.visible = links_circle$weight >= cutoff_include_all_ligands,annotationTrack = "grid", 
             preAllocateTracks = list(track.height = 0.075))
# we go back to the first track and customize sector labels
circos.track(track.index = 1, panel.fun = function(x, y) {
  circos.text(CELL_META$xcenter, CELL_META$ylim[1], CELL_META$sector.index,
              facing = "clockwise", niceFacing = TRUE, adj = c(0, 0.55), cex = 0.8)
}, bg.border = NA) #


# Niche DEG b/w conditions of interest ----
# https://github.com/saeyslab/nichenetr/blob/master/vignettes/differential_nichenet_pEMT.md#differential-nichenet-analysis-between-conditions-of-interest


# OLD  ------


seurat_integrated.mt.combined.name <- RenameIdents(seurat_integrated.mt.combined, 
                           `0` = "na", 
                           `1` = "na",
                           `2` = "Astrocyte",
                           `3` = "Astrocyte",
                           `4` = "T cells",
                           `5` = "Microglia",
                           `6` = "Microglia",
                           `7` = "Astrocyte",
                           `8` = "Neuron",
                           `9` = "na",
                           `10` = "na",
                           `11` = "na"
                           
)

seurat_integrated.done <- subset(seurat_integrated.mt.combined.name, idents = c( "Microglia", "Astrocyte", "T cells", "Neuron"))

markers.to.plot <- c(c("CD3D",  "SELL", "GIMAP5",  "GNLY", "NKG7", 
                       "CD8A", "CSF1R",   'SPI1', "AQP4",  "ID4", "VIM", "GFAP", "WSB1",  "SYTL2", "KCNJ6"))
markers.to.plot <- c(c("VIM", "GFAP", "CD3D",  "CD8A", "GNLY", "NKG7", 
                      "CSF1R",   'SPI1',   "WSB1",  "SYTL2", "KCNJ6",
                      "TOP2A", "MKI67"))

DefaultAssay(seurat_integrated.mt.combined.name) <- "RNA"
dot01(seurat_integrated.mt.combined.name, gene = markers.to.plot, "DotPlot (2.0)_by radius")+
  theme(axis.text.x = element_text(angle = 45, vjust = 0.5, hjust=0.5))

Idents(seurat_integrated.done) <- "integrated_snn_res.0.4"

seurat_integrated.mt.combined$stim <- factor(seurat_integrated.mt.combined$stim, levels = c("CTRL",  "AD"))
DimPlot(seurat_integrated.done,  label = TRUE,repel = TRUE, pt.size = 2.0, label.size = 7) 

table(seurat_integrated.mt.combined.name@active.ident, seurat_integrated.mt.combined.name@meta.data$addition)

de <- function(x, id1, id2, assay){
  x <- FindMarkers(x,
                   ident.1 = id1,
                   ident.2 = id2,
                   test.use = "wilcox",
                   assay = assay,
                   slot = "data",
                   min.pct = 0.1,
                   logfc.threshold = 0.1)
  return(x)
}

# check
table(seurat_mg$integrated_snn_res.0.4)
table(seurat_mg$line)
table(seurat_mg$sample)

seurat_mg <- subset(seurat_integrated.mt.combined.name, idents = c( "Microglia"))
# DE by line
Idents(seurat_mg) <- "sample"


# DE RNA assay
# refernce: https://github.com/satijalab/seurat/issues/2906

de.line.list_rna <- list()
de.line.list_rna[["H10_MT vs. H10_M"]] <- de(seurat_mg, id1 = "H10_MT", 
                                          id2 = "H10_M", 
                                          assay = "RNA")
de.line.list_rna[["H10_M vs. G10_M"]] <- de(seurat_mg, 
                                            id1 = "H10_M", 
                                            id2 = "G10_M", 
                                            assay = "RNA")
de.line.list_rna[["H10_MT vs. G10_MT"]] <- de(seurat_mg, 
                                       id1 = "H10_MT", 
                                       id2 = "G10_MT",
                                       assay = "RNA")

# export as xlsx
library(xlsx)
for(i in 1:length(de.line.list_rna)){
  write.xlsx(de.line.list_rna[[i]], 
             file = "results/microglia.T_de_line_rna_results.xlsx", 
             sheetName = names(de.line.list_rna[i]),
             append = T)
}


Idents(seurat_mg) <- "stim"

de.line.list_rna <- list()
de.line.list_rna[["AD vs. CTRL"]] <- de(seurat_mg, id1 = "AD", 
                                             id2 = "CTRL", 
                                             assay = "RNA")

# export as xlsx
library(xlsx)
for(i in 1:length(de.line.list_rna)){
  write.xlsx(de.line.list_rna[[i]], 
             file = "results/seurat_mg.AD.vs.CTRL_de_line_rna_results.xlsx", 
             sheetName = names(de.line.list_rna[i]),
             append = T)
}


seurat_ast <- subset(seurat_integrated.mt.combined.name, idents = c( "Astrocyte"))
# DE by line
Idents(seurat_ast) <- "sample"


# DE RNA assay
# refernce: https://github.com/satijalab/seurat/issues/2906

de.line.list_rna <- list()
de.line.list_rna[["H10_MT vs. H10_M"]] <- de(seurat_ast, id1 = "H10_MT", 
                                             id2 = "H10_M", 
                                             assay = "RNA")
de.line.list_rna[["H10_M vs. G10_M"]] <- de(seurat_ast, 
                                            id1 = "H10_M", 
                                            id2 = "G10_M", 
                                            assay = "RNA")
de.line.list_rna[["H10_MT vs. G10_MT"]] <- de(seurat_ast, 
                                              id1 = "H10_MT", 
                                              id2 = "G10_MT",
                                              assay = "RNA")

# export as xlsx
library(xlsx)
for(i in 1:length(de.line.list_rna)){
  write.xlsx(de.line.list_rna[[i]], 
             file = "results/astrocyte.T_de_line_rna_results.xlsx", 
             sheetName = names(de.line.list_rna[i]),
             append = T)
}



Idents(seurat_ast) <- "stim"

de.line.list_rna <- list()
de.line.list_rna[["AD vs. CTRL"]] <- de(seurat_ast, id1 = "AD", 
                                        id2 = "CTRL", 
                                        assay = "RNA")

# export as xlsx
library(xlsx)
for(i in 1:length(de.line.list_rna)){
  write.xlsx(de.line.list_rna[[i]], 
             file = "results/seurat_ast.AD.vs.CTRL_de_line_rna_results.xlsx", 
             sheetName = names(de.line.list_rna[i]),
             append = T)
}



seurat_T <- subset(seurat_integrated.mt.combined.name, idents = c( "T cells"))
# DE by line
Idents(seurat_T) <- "stim"


# DE RNA assay
# refernce: https://github.com/satijalab/seurat/issues/2906

de.line.list_rna <- list()
de.line.list_rna[["AD vs. CTRL"]] <- de(seurat_T, id1 = "AD", 
                                             id2 = "CTRL", 
                                             assay = "RNA")
de.line.list_rna[["H10_M vs. G10_M"]] <- de(seurat_T, 
                                            id1 = "H10_M", 
                                            id2 = "G10_M", 
                                            assay = "RNA")
de.line.list_rna[["H10_MT vs. G10_MT"]] <- de(seurat_T, 
                                              id1 = "H10_MT", 
                                              id2 = "G10_MT",
                                              assay = "RNA")

# export as xlsx
library(xlsx)
for(i in 1:length(de.line.list_rna)){
  write.xlsx(de.line.list_rna[[i]], 
             file = "results/seurat_T.AD.vs.CTRL_de_line_rna_results.xlsx", 
             sheetName = names(de.line.list_rna[i]),
             append = T)
}



# 2022.03.17 re.re.clustering ----

seurat_integrated.mt1 <- subset(seurat_integrated, idents = c( "2", "3", "4", "5","6","7","8"))
seurat_integrated.mt1.list <- SplitObject(seurat_integrated.mt1, split.by = "sample")
seurat_integrated.mt1.list <- lapply(X = seurat_integrated.mt1.list, FUN = function(x) {
  x <- NormalizeData(x)
  x <- FindVariableFeatures(x, selection.method = "vst", nfeatures = 2000)
})

# select features that are repeatedly variable across datasets for integration
features <- SelectIntegrationFeatures(object.list = seurat_integrated.mt1.list)
seurat_integrated.mt1.anchors <- FindIntegrationAnchors(object.list = seurat_integrated.mt1.list, anchor.features = features)
seurat_integrated.mt1.combined <- IntegrateData(anchorset = seurat_integrated.mt1.anchors)
DefaultAssay(seurat_integrated.mt1.combined) <- "integrated"

# Run the standard workflow for visualization and clustering
seurat_integrated.mt1.combined <- ScaleData(seurat_integrated.mt1.combined, verbose = FALSE)
seurat_integrated.mt1.combined <- RunPCA(seurat_integrated.mt1.combined, npcs = 30, verbose = FALSE)
seurat_integrated.mt1.combined <- RunUMAP(seurat_integrated.mt1.combined, reduction = "pca", dims = 1:30)
seurat_integrated.mt1.combined <- FindNeighbors(seurat_integrated.mt1.combined, reduction = "pca", dims = 1:30)
seurat_integrated.mt1.combined <- FindClusters(seurat_integrated.mt1.combined, resolution = 0.5)

markers.to.plot <- c(c("CD3D",  "SELL", "GIMAP5",  "GNLY", "NKG7", 
                       "CD8A", "CSF1R",   'SPI1', "AQP4",  "ID4", "VIM", "GFAP", "WSB1",  "SYTL2", "KCNJ6"))

DefaultAssay(seurat_integrated.mt1.combined) <- "RNA"
dot01(seurat_integrated.mt1.combined, gene = markers.to.plot, "DotPlot (2.0)_by radius")+
  theme(axis.text.x = element_text(angle = 45, vjust = 0.5, hjust=0.5))

DimPlot(seurat_integrated.mt1.combined)
table(seurat_integrated.mt1.combined@active.ident, seurat_integrated.mt1.combined@meta.data$addition)


seurat_integrated.mt1.combined.find.all.markers <- FindAllMarkers(seurat_integrated.mt1.combined, 
                                                                 only.pos = TRUE, 
                                                                 min.pct = 0.25,
                                                                 assay = "RNA",
                                                                 slot = "data", 
                                                                 logfc.threshold = 0.25)

seurat_integrated.mt1.combined.find.all.markers.top20 <- seurat_integrated.mt1.combined.find.all.markers %>% 
  group_by(cluster) %>% 
  slice_max(n = 10, order_by = avg_log2FC)
view(seurat_integrated.mt1.combined.find.all.markers.top20)

# done ----

# :: View clusters
# make sure the default assay is integrated
DefaultAssay(seurat_integrated)

view_clusters <- function(res){
  Idents(seurat_integrated) <- res
  DimPlot(seurat_integrated,
          reduction = "umap",
          label = TRUE,
          label.size = 5,
          pt.size = 0.9) +
    ggtitle(paste0("Clustering at ", res)) + 
    theme(plot.title = element_text(hjust=0.5, face="bold"))
}



umap.list <- list()
for(i in 1:length(res.list)){
  umap.list[[i]] <- view_clusters(res.list[[i]])
}



# export
for(i in 1:length(umap.list)){
  svglite(filename = paste0("figures/reduction_clustering/clusters_umap_", res.list[[i]], ".svg"),
          width = 10, height = 7)
  
  print(umap.list[[i]])
  
  dev.off()
}




# take the result at resolution 0.4 to go forward
Idents(seurat_integrated) <- res.list[["res.0.4"]]
# seurat_integrated$seurat_clusters <- seurat_integrated$integrated_snn_res.0.4
identical(seurat_integrated@active.ident, seurat_integrated$integrated_snn_res.0.4)     # TRUE


# Umap by sample
svglite(filename = "figures/reduction_clustering/clusters_umap_sample_0.4.svg",
        width = 20, height = 5)
print(DimPlot(seurat_integrated,
              reduction = "umap",
              split.by = "sample",
              label = TRUE,
              label.size = 5,
              pt.size = 0.9) +
        ggtitle("Clustering by sample at resolution 0.4") + 
        theme(plot.title = element_text(hjust=0.5, face="bold")))
dev.off()


# Umap by stim
seurat_integrated$stim <- factor(seurat_integrated$stim, levels = c("CTRL", "AD"))


svglite(filename = "figures/reduction_clustering/clusters_umap_stim_0.4.svg",
        width = 12, height = 7)
print(DimPlot(seurat_integrated,
              reduction = "umap",
              split.by = "stim",
              label = TRUE,
              label.size = 5,
              pt.size = 0.9) +
        ggtitle("Clustering by stim at resolution 0.4") + 
        theme(plot.title = element_text(hjust=0.5, face="bold")))
dev.off()


# Umap by addition
unique(seurat_integrated$addition)
seurat_integrated$addition <- factor(seurat_integrated$addition, levels = c("None",
                                                                            "Microglia",
                                                                            "T cells",
                                                                            "Microglia & T cells"))




svglite(filename = "figures/reduction_clustering/clusters_umap_addition_0.4.svg",
        width = 15, height = 5)
print(DimPlot(seurat_integrated,
              reduction = "umap",
              split.by = "addition",
              label = TRUE,
              label.size = 5,
              pt.size = 0.9) +
        ggtitle("Clustering by addition at resolution 0.4") + 
        theme(plot.title = element_text(hjust=0.5, face="bold")))
dev.off()





# Cllustering QC ----
n_cells <- FetchData(seurat_integrated, 
                     vars = c("ident", "orig.ident")) %>%
  dplyr::count(ident, orig.ident) %>%
  tidyr::spread(ident, n)


View(t(n_cells))


# ncells by stim and cluster
n_cells <- FetchData(seurat_integrated, 
                     vars = c("ident", "stim")) %>%
  dplyr::count(ident, stim) %>%
  tidyr::spread(ident, n)


View(t(n_cells))


# ncells by sample and cluster
n_cells <- FetchData(seurat_integrated, 
                     vars = c("ident", "sample")) %>%
  dplyr::count(ident, sample) %>%
  tidyr::spread(ident, n)


View(t(n_cells))



# ncells by addition and cluster
n_cells <- FetchData(seurat_integrated, 
                     vars = c("ident", "addition")) %>%
  dplyr::count(ident, addition) %>%
  tidyr::spread(ident, n)


View(t(n_cells))

#_----
#_----
#_----


# 9. Cell Type Identification ----
# :: marker genes (Joseph confirmed 1/30/2022)----

m.mic <- c("HLA-DRB1", "CSF1R")
m.astro <- c("AQP4", "ID4", "APOE")
m.neurons <- c("WSB1", "KCNJ6", "SYTL2")
m.oligo <- c("MOG", "MBP", "PDGFRA", "PLP1")
markers.all <- c(m.mic, m.astro, m.neurons, m.oligo)
markers.list <- list(m.mic, m.astro, m.neurons, m.oligo)
names(markers.list) <- c("Mic", "Astro", "Neurons", "Oligo")





# :: FeaturePlot----
DefaultAssay(seurat_integrated) <- "RNA"
fplot <- function(obj, markers, ncol, min = NA, max = NA){FeaturePlot(obj,
                                                                      reduction = "umap",
                                                                      features = markers,
                                                                      label = TRUE,
                                                                      col = c("grey", "blue"),
                                                                      ncol = ncol,
                                                                      min.cutoff = min,
                                                                      max.cutoff = max,
                                                                      pt.size = 0.3)
}


fplot_all <- fplot(seurat_integrated, markers.all, 3)
svglite(filename = "figures/reduction_clustering/cell_type/feature_plots_all.svg", width = 10, height = 10)
print(fplot_all)
dev.off()



# :: DotPlots ----
# Color setting
my_color <- c('yellow', 'red')

# dotplot scale by size
dot <- function(obj, gene, title){
  DotPlot(obj, features = gene, dot.scale = 10, cols = my_color, scale.by = "size") +
    RotatedAxis() +
    labs(title = title) +
    theme(plot.title = element_text(hjust = 0.5))  +
    theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=0.5))
}


# dotplot scale by radius
dot01 <- function(obj, gene, title){
  DotPlot(obj, features = gene, dot.scale = 10, cols = my_color, scale.by = "radius") +
    RotatedAxis() +
    labs(title = title) +
    theme(plot.title = element_text(hjust = 0.5))  +
    theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=0.5))
}

# by size
svglite(filename = "figures/reduction_clustering/cell_type/dotplot.svg", 
        width = 12, height = 10)
dot(seurat_integrated, gene = markers.all, "DotPlot (0.4)_by size")
dev.off()

# by radius
svglite(filename = "figures/reduction_clustering/cell_type/dotplot_radius.svg", 
        width = 12, height = 10)
dot01(seurat_integrated, gene = markers.all, "DotPlot (0.4)_by radius")
dev.off()







# FindAllMarkers
# To find markers for every cluster compared to all remaining cells, report only the positive ones

# DE with DESeq2 test method
# install DESeq2
if (!requireNamespace("BiocManager", quietly = TRUE))
  install.packages("BiocManager")

BiocManager::install("DESeq2")

# copy the object in case
seurat_integrated_copy <- seurat_integrated


# replace zero values in counts into 1, so add 1 to all the values in counts
# https://github.com/satijalab/seurat/issues/1570
seurat_integrated_copy[["RNA"]]@counts <- as.matrix(seurat_integrated_copy[["RNA"]]@counts)+1



# FindMarkers DESeq2
find.all.markers_deseq2 <- FindAllMarkers(seurat_integrated_copy, 
                                          only.pos = TRUE, 
                                          min.pct = 0.01,
                                          logfc.threshold = 0.01,
                                          test.use = "DESeq2",
                                          assay = "RNA",
                                          slot = "counts")          # when DESeq2 method, slot will be set counts. no need to manually set this later.)


# check the results of FindAllMarkers DESeq2
head(find.all.markers_deseq2)
dim(find.all.markers_deseq2)



# top 20 markers
find.all.markers_deseq_top20 <- find.all.markers_deseq2 %>% 
  group_by(cluster) %>% 
  slice_max(n = 20, order_by = avg_logFC)

View(find.all.markers_deseq_top20)



# FindAllMarkers with wilcox method
find.all.markers_wilcox <- FindAllMarkers(seurat_integrated,
                                          only.pos = TRUE, 
                                          min.pct = 0.01,
                                          logfc.threshold = 0.01,
                                          test.use = "wilcox",
                                          assay = "RNA",
                                          slot = "data")       


find.all.markers_wilcox_top20 <- find.all.markers_wilcox %>% 
  group_by(cluster) %>% 
  slice_max(n = 20, order_by = avg_logFC)

dim(find.all.markers_wilcox)
View(find.all.markers_wilcox_top20)



library(xlsx)
write.xlsx(find.all.markers_deseq2, file = "results/findallmarkers_deseq2.xlsx", col.names = T, row.names = T, append = F)
write.xlsx(find.all.markers_deseq_top20, file = "results/findallmarkers_deseq2_top20.xlsx", col.names = T, row.names = T, append = F)


# comeback 2/4/22 ====

# comeback 2/1/22====
# Invalid from here~~ ====


#_----
## [Branch] ----

# 6. Integration of H10 + H10MG ----


# Using SCTransform object as input
# Select the most variable features to use for integration (default: nfeatures = 2000)
split_seurat_h <- split_seurat[c(1, 6)]
saveRDS(split_seurat_h, file = "branch/H10 line only/objects/split_seurat_h.rds")

integ_features_h <- SelectIntegrationFeatures(object.list = split_seurat_h, nfeatures = 3000)

# Prepare the SCT list object for integration
split_seurat_h <- PrepSCTIntegration(object.list = split_seurat_h,
                                     anchor.features = integ_features_h)


# Find best buddies - can take a while to run
integ_anchors_h <- FindIntegrationAnchors(object.list = split_seurat_h,
                                          normalization.method = "SCT",
                                          anchor.features = integ_features_h)


saveRDS(integ_anchors_h, file = "branch/H10 line only/objects/integ_anchors_h.rds")

# Before integration, manage the memory
rm(cn.list)
rm(ensembl)
# rm(filtered_seurat)
# rm(split_seurat)
# rm(seurat.list)


# Integrate across conditions
seurat_integrated_h <- IntegrateData(anchorset = integ_anchors_h, normalization.method = "SCT")



# save
saveRDS(seurat_integrated_h, file = "branch/H10 line only/objects/seurat_integrated_h.rds")
# seurat_integrated_h <- readRDS(file = "branch/H10 line only/objects/seurat_integrated_h.rds")



# 7. Reduction ----
# Run PCA
seurat_integrated_h <- RunPCA(object = seurat_integrated_h)


# Plot PCA
svglite(filename = "branch/H10 line only/figures/reduction_clustering/pca.svg",width = 10, height = 8)

PCAPlot(seurat_integrated_h, split.by = "sample")  + 
  ggtitle("PCA Plot by Sample") + 
  theme(axis.text.x = element_blank()) +
  # theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=0.5)) +
  theme(plot.title = element_text(hjust=0.5, face="bold"))

dev.off()


# Run UMAP and TSNE
seurat_integrated_h <- RunUMAP(seurat_integrated_h, 
                               dims = 1:40,
                               reduction = "pca")


seurat_integrated_h <- RunTSNE(seurat_integrated_h, 
                               dims = 1:40,
                               reduction = "pca")


# Plot UMAP                             

svglite(filename = "branch/H10 line only/figures/reduction_clustering/umap.svg")
DimPlot(seurat_integrated_h)        
dev.off()

svglite(filename = "branch/H10 line only/figures/reduction_clustering/umap_sample split.svg")
DimPlot(seurat_integrated_h, split.by = "sample")        
dev.off()

svglite(filename = "branch/H10 line only/figures/reduction_clustering/tsne.svg")
TSNEPlot(seurat_integrated_h)
dev.off()


svglite(filename = "branch/H10 line only/figures/reduction_clustering/tsne_sample split.svg")
TSNEPlot(seurat_integrated_h, split.by = "sample")        
dev.off()



# 8. Clustering ----
# Determine the K-nearest neighbor graph
seurat_integrated_h <- FindNeighbors(object = seurat_integrated_h, dims = 1:40)


# Determine the clusters for various resolutions                                
seurat_integrated_h <- FindClusters(object = seurat_integrated_h, 
                                    resolution = c(0.4, 0.6, 0.8, 1.0, 1.4, 2.4, 3.0))

# save 
saveRDS(seurat_integrated_h, file = "branch/H10 line only/objects/seurat_integrated_h_clustered.rds")
# seurat_integrated_h <- readRDS(file = "branch/H10 line only/objects/seurat_integrated_h_clustered.rds")



# :: Clusters by resolution ----
res.list <- list(names(seurat_integrated_h@meta.data[14]),
                 names(seurat_integrated_h@meta.data[15]),
                 names(seurat_integrated_h@meta.data[16]),
                 names(seurat_integrated_h@meta.data[17]),
                 names(seurat_integrated_h@meta.data[18]),
                 names(seurat_integrated_h@meta.data[19]),
                 names(seurat_integrated_h@meta.data[20]))

names(res.list) <- str_sub(names(seurat_integrated_h@meta.data[14:20]), start = 16L, end = -1L)
View(res.list)


# :: View clusters
DefaultAssay(seurat_integrated_h) <- "integrated"

view_clusters <- function(res){
  Idents(seurat_integrated_h) <- res
  DimPlot(seurat_integrated_h,
          reduction = "umap",
          label = TRUE,
          label.size = 5,
          pt.size = 0.8) +
    ggtitle(paste0("Clustering at ", res)) + 
    theme(plot.title = element_text(hjust=0.5, face="bold"))
}

umap.list <- list()
for(i in 1:length(res.list)){
  umap.list[[i]] <- view_clusters(res.list[[i]])
}


# export
for(i in 1:length(umap.list)){
  svglite(filename = paste0("branch/H10 line only/figures/reduction_clustering/clusters_umap_", res.list[[i]], ".svg"),
          width = 12, height = 10)
  
  print(umap.list[[i]])
  
  dev.off()
}


# take the result at resolution 3.0



# 9. Cell Type Identification ----
# :: marker genes ----
m.astro <- c("AQP4", "UBE2C", "NUSAP1", "TOP2A", "PTPRZ1", "HOPX", "FAM107A")
# general astro: "AQP4"
# fetal astro: "UBE2C", "NUSAP1", "TOP2A"
# intermediate astro: "PTPRZ1", "HOPX", "FAM107A"
m.neurons <- c("ABAT", "GAD1", "KCNJ6", "TPH1", "DCX", "GABBR2", "SATB2", "FOXP2")
m.oligo <- c("FTH1", "PLP1", "FGF1", "MBP", "MOBP")
m.opcs <- c("PDGFRA", "VCAN", "CSPG4")
m.mic <- c("HLA-DRB1", "CSF1R", "CX3CR1", "HLA-DQA1")
markers.all <- c(m.mic, m.astro, m.neurons, m.oligo, m.opcs)
markers.list <- list(m.mic, m.astro, m.neurons, m.oligo, m.opcs)
names(markers.list) <- c("Mic", "Ast", "Neu", "Olig", "OPCs")
select.markers <- c("AQP4", "AGT", "GABBR2", "KCNJ6", "MAP2", "FTH1", "FGF1")



# :: FeaturePlot----
DefaultAssay(seurat_integrated_h) <- "RNA"
fplot <- function(obj, markers, ncol, min = NA, max = NA){FeaturePlot(obj,
                                                                      reduction = "umap",
                                                                      features = markers,
                                                                      label = TRUE,
                                                                      col = c("grey", "blue"),
                                                                      ncol = ncol,
                                                                      min.cutoff = min,
                                                                      max.cutoff = max,
                                                                      pt.size = 0.3)
}


# Idents(seurat_integrated_h) <- res.list[[1]]


fplot.list <- list(fplot(seurat_integrated_h, m.mic, 1),
                   fplot(seurat_integrated_h, m.astro, 3),
                   fplot(seurat_integrated_h, m.neurons, 3),
                   fplot(seurat_integrated_h, m.oligo, 3),
                   fplot(seurat_integrated_h, m.opcs, 2),
                   fplot(seurat_integrated_h, select.markers, 3))

names(fplot.list) <- c("m.microglia",
                       "m.astro",
                       "m.neurons",
                       "m.oligo",
                       "m.opcs",
                       "select.markers")


# export
for(i in 1){
  svglite(filename = paste0("branch/H10 line only/figures/reduction_clustering/cell_type/feature_plots_", names(fplot.list[i]), ".svg"), width = 5, height = 10)
  print(fplot.list[[i]])
  dev.off()
}


for(i in c(2, 3, 6)){
  svglite(filename = paste0("branch/H10 line only/figures/reduction_clustering/cell_type/feature_plots_", names(fplot.list[i]), ".svg"), width = 12, height = 10)
  print(fplot.list[[i]])
  dev.off()
}


for(i in 4){
  svglite(filename = paste0("branch/H10 line only/figures/reduction_clustering/cell_type/feature_plots_", names(fplot.list[i]), ".svg"), width = 12, height = 10)
  print(fplot.list[[i]])
  dev.off()
}

for(i in 5){
  svglite(filename = paste0("branch/H10 line only/figures/reduction_clustering/cell_type/feature_plots_", names(fplot.list[i]), ".svg"), width = 12, height = 10)
  print(fplot.list[[i]])
  dev.off()
}

# :: DotPlots ----
# Color setting
my_color <- c('yellow', 'red')

# dotplot scale by size
dot <- function(obj, gene, title){
  DotPlot(obj, features = gene, dot.scale = 6, cols = my_color, scale.by = "size") +
    RotatedAxis() +
    labs(title = title) +
    theme(plot.title = element_text(hjust = 0.5))  +
    theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=0.5))
}


# dotplot scale by radius
dot01 <- function(obj, gene, title){
  DotPlot(obj, features = gene, dot.scale = 6, cols = my_color, scale.by = "radius") +
    RotatedAxis() +
    labs(title = title) +
    theme(plot.title = element_text(hjust = 0.5))  +
    theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=0.5))
}

# by size
svglite(filename = "branch/H10 line only/figures/reduction_clustering/cell_type/dotplot.svg", 
        width = 15, height = 10)
dot(seurat_integrated_h, gene = markers.all, "DotPlot (3.0)_by size")
dev.off()

# DotPlot without FTH1
svglite(filename = "branch/H10 line only/figures/reduction_clustering/cell_type/dotplot_wo_FTH1.svg", 
        width = 21, height = 6)
dot(seurat_integrated_h, gene = unlist(markers.list)[-20], "DotPlot wo FTH1 (3.0)_by size")
dev.off()

# by radius
svglite(filename = "branch/H10 line only/figures/reduction_clustering/cell_type/dotplot_radius.svg", 
        width = 21, height = 6)
dot01(seurat_integrated_h, gene = markers.all, "DotPlot (3.0)_by radius")
dev.off()

# DotPlot without FTH1
svglite(filename = "branch/H10 line only/figures/reduction_clustering/cell_type/dotplot_wo_FTH1_radius.svg", 
        width = 21, height = 6)
dot(seurat_integrated_h, gene = markers.all[-20], "DotPlot wo FTH1 (3.0)_by radius")
dev.off()


#:: Violin Plots ----
violin_cell_type <- function(x, genes, assay, slot, pt){
  VlnPlot(x, features = genes,
          assay = assay,           # "SCT", "RNA"
          slot = slot,             # "scale.data", "data"
          pt.size = pt,
          ncol = 1)
}


# [Additional gene lookup] ----
DefaultAssay(seurat_integrated_h)

test.IL3RA <- WhichCells(seurat_integrated_h, expression = IL3RA > 0)
length(test.IL3RA)   # 62

fplot(seurat_integrated_h, "IL3RA", 1)






# cell_type_vlnplot <- list()
# for(i in 1:length(markers.list)){
#   cell_type_vlnplot[[i]] <- violin_cell_type(seurat_integrated_h, markers.list[[i]], 0)
# }
# 
# names(cell_type_vlnplot) <- names(markers.list)
# 
# for(i in 1:length(cell_type_vlnplot)){
#   # open svglite
#   svglite(filename = paste0("branch/H10 line only/figures/reduction_clustering/cell_type/vln_", names(markers.list[i]), ".svg"),
#        res = 100, quality = 100, width = 1000, height = 2000)
#   
#   # plot
#   print(cell_type_vlnplot[[i]])
#   
#   # device off
#   dev.off()
# }

#.----
#.----
#.----
# ++++ Done so far 9/7 ++++----
#.----
#.----
#.----


cell_type_vlnplot_RNA.data <- list()

for(i in 1:length(markers.list)){
  cell_type_vlnplot_RNA.data[[i]] <- violin_cell_type(seurat_integrated_h, 
                                                      genes = markers.list[[i]],
                                                      assay = "RNA",
                                                      slot = "data",
                                                      pt = 0)
}

names(cell_type_vlnplot_RNA.data) <- names(markers.list)

for(i in 1:length(cell_type_vlnplot_RNA.data)){
  # open svglite
  svglite(filename = paste0("branch/H10 line only/figures/reduction_clustering/cell_type/vln_01_", names(markers.list[i]), ".svg"), width = 10, height = 10)
  
  # plot
  print(cell_type_vlnplot_RNA.data[[i]])
  
  # device off
  dev.off()
}

# 10_1. ConservedMarkers ----

# Reference: https://satijalab.org/seurat/archive/v3.0/sctransform_vignette.html
# The ‘corrected’ UMI counts are stored in pbmc[["SCT"]]@counts. We store log-normalized versions of these corrected counts in pbmc[["SCT"]]@data, which are very helpful for visualization.
# You can use the corrected log-normalized counts for differential expression and integration. However, in principle, it would be most optimal to perform these calculations directly on the residuals (stored in the scale.data slot) themselves. This is not currently supported in Seurat v3, but will be soon.

# Not recommending using SCT assay for integrated different datasets: https://github.com/satijalab/seurat/issues/2906

# Create function to get conserved markers for any given cluster
library(tibble)
get_conserved <- function(cluster){
  FindConservedMarkers(seurat_integrated_h,
                       ident.1 = cluster,
                       assay = "SCT",
                       slot = "data",
                       grouping.var = "sample",
                       only.pos = TRUE,
                       logfc.threshold = 0.01) %>%
    rownames_to_column(var = "gene") %>%
    cbind(cluster_id = cluster, .)
}


library(purrr)
library(dplyr)

# conserved markers for each cluster (0:34)
conserved_markers <- map_dfr(c(0:34), get_conserved)

# Error occurred

# 8. FindMarkers
# To find markers for every cluster compared to all remaining cells, report only the positive ones
find.all.markers <- FindAllMarkers(seurat_integrated_h, 
                                   only.pos = TRUE, 
                                   min.pct = 0.05,
                                   assay = "RNA",
                                   slot = "data", 
                                   logfc.threshold = 0.25)

find.all.markers.top20 <- find.all.markers %>% 
  group_by(cluster) %>% 
  slice_max(n = 20, order_by = avg_logFC)


View(find.all.markers.top20)


# export
write.csv(x=find.all.markers.top20, file = "results/top20_all_markers.csv", 
          row.names = TRUE)




# [MG_subset Ad-hoc] ====
# https://github.com/dearzoo/06.-Triculture_TR2_ACA/issues/2

# 1. DE by condition ----
# 1) WT+K604 vs WT,
# 2) TR2+K604_vs_WT+K604,
# 3) TR2 vs WT,

seurat_mg <- subset(seurat_integrated, idents = "3")

de <- function(x, id1, id2, assay){
  x <- FindMarkers(x,
                   ident.1 = id1,
                   ident.2 = id2,
                   test.use = "wilcox",
                   assay = assay,
                   slot = "data",
                   min.pct = 0.05,
                   logfc.threshold = 0.05)
  return(x)
}

# check
table(seurat_mg$integrated_snn_res.0.4)
table(seurat_mg$line)

# DE by line
Idents(seurat_mg) <- "line"

de.line.list_sct <- list()
de.line.list_sct[["WT.604 vs. WT"]] <- de(seurat_mg, id1 = "A4H1.WT.K604", 
                                          id2 = "A4H1.WT", 
                                          assay = "SCT")
de.line.list_sct[["TR2.604 vs. TR2"]] <- de(seurat_mg, 
                                            id1 = "A4H1.TR2.K604", 
                                            id2 = "A4H1.TR2", 
                                            assay = "SCT")
de.line.list_sct[["TR2 vs. WT"]] <- de(seurat_mg, 
                                       id1 = "A4H1.TR2", 
                                       id2 = "A4H1.WT",
                                       assay = "SCT")


# export as xlsx
library(xlsx)
for(i in 1:length(de.line.list_sct)){
  write.xlsx(de.line.list_sct[[i]], 
             file = "results/microglia_sub/de_line_sct_results.xlsx", 
             sheetName = names(de.line.list_sct[i]),
             append = T)
}


# DE RNA assay
# refernce: https://github.com/satijalab/seurat/issues/2906

de.line.list_rna <- list()
de.line.list_rna[["WT.604 vs. WT"]] <- de(seurat_mg, id1 = "A4H1.WT.K604", 
                                          id2 = "A4H1.WT", 
                                          assay = "RNA")
de.line.list_rna[["TR2.604 vs. TR2"]] <- de(seurat_mg, 
                                            id1 = "A4H1.TR2.K604", 
                                            id2 = "A4H1.TR2", 
                                            assay = "RNA")
de.line.list_rna[["TR2 vs. WT"]] <- de(seurat_mg, 
                                       id1 = "A4H1.TR2", 
                                       id2 = "A4H1.WT",
                                       assay = "RNA")



# export as xlsx
library(xlsx)
for(i in 1:length(de.line.list_rna)){
  write.xlsx(de.line.list_rna[[i]], 
             file = "results/microglia_sub/de_line_rna_results.xlsx", 
             sheetName = names(de.line.list_rna[i]),
             append = T)
}


# 2. MG cluster re-clustering ----
# Re-clustering needed. So, processes from preprocess are needed but because this is a single cluster (3), no integration is needed.
# So, the steps are... 1) pre-process, 2) Reduction, and 3) Clustering

# 1) Pre-process (Active assay = RNA)
seurat_mg <- NormalizeData(seurat_mg, verbose = TRUE)
seurat_mg <- CellCycleScoring(seurat_mg, g2m.features=g2m_genes, s.features=s_genes)
seurat_mg <- SCTransform(seurat_mg, vars.to.regress = c("percent.mt"))


# 2) Reduction
# Run PCA
seurat_mg <- RunPCA(object = seurat_mg)


# Plot PCA
svglite(filename = "figures/subset analysis/microglia_pca.svg", res = 100, quality = 100,
        width = 2000, height = 800)

PCAPlot(seurat_mg, split.by = "line", pt.size = 1)  + 
  ggtitle("Microglia Subset PCA by Sample") + 
  theme(axis.text.x = element_blank()) +
  # theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=0.5)) +
  theme(plot.title = element_text(hjust=0.5, face="bold"))

dev.off()

# Run UMAP and TSNE
seurat_mg <- RunUMAP(seurat_mg, 
                     dims = 1:40,
                     reduction = "pca")


seurat_mg <- RunTSNE(seurat_mg, 
                     dims = 1:40,
                     reduction = "pca")


# Plot UMAP                             

DimPlot(seurat_mg)
TSNEPlot(seurat_mg)


Idents(seurat_mg) <- "line"
Idents(seurat_mg) <- "orig.ident"

svglite(filename = "figures/subset analysis/microglia_subset_umap_by_line.svg",
        width = 2000, height = 800,
        res = 110, quality = 100)
DimPlot(seurat_mg, split.by = "line", pt.size = 1)
dev.off()

# 3) Clustering
# Determine the K-nearest neighbor graph
seurat_mg <- FindNeighbors(object = seurat_mg, dims = 1:40)


# Determine the clusters for various resolutions                                
seurat_mg <- FindClusters(object = seurat_mg, 
                          resolution = c(0.4, 0.6, 0.8, 1.0, 1.4))

# save 
saveRDS(seurat_mg, file = "objects/seurat_mg_clustered.rds")
# seurat_mg <- readRDS(file = "objects/seurat_mg_clustered.rds")


View(names(seurat_mg@meta.data))


# :: Clusters by resolution ----
res.list <- list(names(seurat_mg@meta.data[21]),
                 names(seurat_mg@meta.data[22]),
                 names(seurat_mg@meta.data[23]),
                 names(seurat_mg@meta.data[24]),
                 names(seurat_mg@meta.data[25]))
res.list


table(seurat_mg$SCT_snn_res.0.4)


view_clusters <- function(res){
  Idents(seurat_mg) <- res
  DimPlot(seurat_mg,
          reduction = "umap",
          label = TRUE,
          label.size = 5,
          pt.size = 0.8) +
    ggtitle(paste0("Clustering at ", res)) + 
    theme(plot.title = element_text(hjust=0.5, face="bold"))
}

umap.mg.list <- list()
for(i in 1:length(res.list)){
  umap.mg.list[[i]] <- view_clusters(res.list[[i]])
}


# export
for(i in 1:length(umap.mg.list)){
  svglite(filename = paste0("figures/subset analysis/microglia_subset_clusters_umap_", res.list[[i]], ".svg"),
          width = 1000, height = 1200,
          res = 120, quality = 100)
  
  print(umap.mg.list[[i]])
  
  dev.off()
}


# Clustering result by line
svglite(filename = "figures/subset analysis/microglia_subset_clustering_by_line.svg",
        width = 2000, height = 800,
        res = 110, quality = 100)
Idents(seurat_mg) <- "SCT_snn_res.0.4"
DimPlot(seurat_mg, split.by = "line", pt.size = 1)
dev.off()



# :: Cell type ----
# based on satija lab's advice, DE will be performed on RNA assay (not SCT assay) when the dataset is from integrating multiple datasets. https://github.com/satijalab/seurat/issues/2906
Idents(seurat_mg) <- "SCT_snn_res.0.4"

find.all.markers_mg <- FindAllMarkers(seurat_mg, 
                                      only.pos = TRUE, 
                                      min.pct = 0.05,
                                      assay = "RNA",
                                      slot = "data", 
                                      logfc.threshold = 0.15)


find.all.markers_mg.top20 <- find.all.markers_mg %>% 
  group_by(cluster) %>% 
  slice_max(n = 20, order_by = avg_logFC)


View(find.all.markers_mg.top20)

# export
write.csv(x=find.all.markers_mg.top20, file = "results/microglia_sub/mg_subset_top20_all_markers.csv", 
          row.names = TRUE)


# Remove the cluster 1 from the microglia subset (seems to be Astrocytes)
seurat_mg_final <- subset(x = seurat_mg, idents = "1", invert = TRUE)

# check
DimPlot(seurat_mg_final)
ncol(seurat_mg_final)   # 2417



# 3. Compare Tri MG with Chimeric Male ----
# WT microglia subset만 Chimeric microglia랑 인테그레이션

# :: WT (Non-TR2) micro subset ----
table(seurat_mg_final$sample)

Idents(seurat_mg_final) <- "sample"
table(Idents(seurat_mg_final))

seurat_mg_WT <- subset(x = seurat_mg_final, idents = "09_A4H1.WT")
seurat_mg.sample.list <- SplitObject(seurat_mg_final, split.by = "sample")

View(seurat_mg.sample.list)
unlist(lapply(seurat_mg.sample.list, ncol))



# save
saveRDS(seurat_mg_WT, file = "objects/micro_subset/seurat_mg_WT.rds")
# seurat_mg_WT <- readRDS(file = "objects/micro_subset/seurat_mg_WT.rds")

saveRDS(seurat_mg.sample.list, file = "objects/micro_subset/seurat_mg_sample.list.rds")

# check (should be 909)
ncol(seurat_mg_WT)   # 909


# :: Chimeric data ----
# WT male and FAD male objects
# Call the data
seurat.list_chimeric <- readRDS("../01. Single Cell Project/1st Try G10 H10 IMGL/objects/seurat objects/seurat.list.rds")

View(names(seurat.list_chimeric))
names(seurat.list_chimeric[9])
names(seurat.list_chimeric[12])

chimeric_wt_male <- seurat.list_chimeric[[9]]
chimeric_FAD_male <- seurat.list_chimeric[[11]]
chimeric_FAD_female <- seurat.list_chimeric[[12]]

# delete the big list for memory
rm(seurat.list_chimeric)


# Get the datasets ready for Integration
# Integration of final 100k WT microglia and Chimeric male WT and FAD
# look into data ready for integration
head(seurat_mg_WT@meta.data)
head(chimeric_wt_male@meta.data)
head(chimeric_FAD_male@meta.data)


names(seurat_mg_WT@meta.data)
names(chimeric_wt_male@meta.data)

# add type metadata to Tri
seurat_mg_WT$type <- "TriCulture"


# rename chimeric stim metadata for WT (CTRL to WT)
chimeric_wt_male$stim <- "WT"
table(chimeric_wt_male$stim)


# :: Integration ----
# because Chimeric data was pre-processed in the traditional way, it needs to be processed by SCTransform
# :: SCT Pre-process for Chimeric data
chimeric.list <- list(chimeric_wt_male, chimeric_FAD_male)
names(chimeric.list) <- c("Chimeric_WT_M", "Chimeric_FAD_M")

# Automated Normalize, CellCycleScoring, and SCTransfor (with regressing out mt content)
for (i in 1:length(chimeric.list)) {
  chimeric.list[[i]] <- NormalizeData(chimeric.list[[i]], verbose = TRUE)
  chimeric.list[[i]] <- CellCycleScoring(chimeric.list[[i]], g2m.features=g2m_genes, s.features=s_genes)
  chimeric.list[[i]] <- SCTransform(chimeric.list[[i]], vars.to.regress = c("percent.mt"))
}


# Check which assays are stored in objects (All need to show both RNA and SCT)
for(i in 1:length(chimeric.list)){
  print(names(chimeric.list[[i]]@assays))
}

# save permanent (pre-processed seurat)
saveRDS(chimeric.list, file = "objects/chimeric.list_sct_processed.rds")
# chimeric.list <- readRDS(file = "objects/chimeric.list_sct_processed.rds")



seurat.list_tri.chimeric <- list(seurat_mg_WT, chimeric.list[[1]], chimeric.list[[2]])
View(seurat.list_tri.chimeric)
seurat.list_tri.chimeric[[1]]$stim <- "AD"
names(seurat.list_tri.chimeric) <- c("Tri.100k.Microglia", "Chimeric_WT_M", "Chimeric_FAD_M")


# Using SCTransform object as input
# Select the most variable features to use for integration (default: nfeatures = 2000)
integ_features_01 <- SelectIntegrationFeatures(object.list = seurat.list_tri.chimeric, 
                                               nfeatures = 3000)

# Prepare the SCT list object for integration
seurat.list_tri.chimeric <- PrepSCTIntegration(object.list = seurat.list_tri.chimeric,
                                               anchor.features = integ_features_01)


# Find best buddies - can take a while to run
integ_anchors_01 <- FindIntegrationAnchors(object.list = seurat.list_tri.chimeric,
                                           normalization.method = "SCT",
                                           anchor.features = integ_features_01)


options(future.globals.maxSize = 12000 * 1024^8)

# Integrate across conditions
seurat_integrated_01 <- IntegrateData(anchorset = integ_anchors_01, 
                                      normalization.method = "SCT")

# save
saveRDS(seurat_integrated_01, file = "objects/micro_subset/seurat_integrated_wt_micro_chimeric_male.rds")
# seurat_integrated_01 <- readRDS(file = "objects/micro_subset/seurat_integrated_wt_micro_chimeric_male.rds")




# :: Reduction ----
# Run PCA
seurat_integrated_01 <- RunPCA(object = seurat_integrated_01)

# Plot PCA
Idents(seurat_integrated_01) <- "type"

svglite(filename = "figures/micro_chimeric_comparison/pca.svg", res = 100, quality = 100,
        width = 1000, height = 800)

PCAPlot(seurat_integrated_01, split.by = "type")  + 
  ggtitle("PCA_TriCulture vs. Chimeric MG") + 
  theme(axis.text.x = element_blank()) +
  # theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=0.5)) +
  theme(plot.title = element_text(hjust=0.5, face="bold"))

dev.off()


# Run UMAP and TSNE
seurat_integrated_01 <- RunUMAP(seurat_integrated_01, 
                                dims = 1:40,
                                reduction = "pca")


seurat_integrated_01 <- RunTSNE(seurat_integrated_01, 
                                dims = 1:40,
                                reduction = "pca")


# Plot UMAP                             
Idents(seurat_integrated_01) <- "type"
svglite(filename = "figures/micro_chimeric_comparison/umap_by_type_not_clustered_grouped.svg",
        res = 120, quality = 100,
        width = 1000, height = 1100)

DimPlot(seurat_integrated_01, group.by = "type", cols = c("gray", "red"))
dev.off()

svglite(filename = "figures/micro_chimeric_comparison/umap_by_type_not_clustered.svg",
        res = 120, quality = 100,
        width = 1200, height = 800)
DimPlot(seurat_integrated_01, split.by = "type")
dev.off()

TSNEPlot(seurat_integrated_01, split.by = "type")
DimPlot(seurat_integrated_01, split.by = "line")

dim(seurat_integrated_01)

# :: Clustering ----
# Determine the K-nearest neighbor graph
seurat_integrated_01 <- FindNeighbors(object = seurat_integrated_01, dims = 1:40)


# Determine the clusters for various resolutions                                
seurat_integrated_01 <- FindClusters(object = seurat_integrated_01, resolution = 0.5)


# save 
saveRDS(seurat_integrated_01, file = "objects/micro_subset/seurat_integrated_01_clustered.rds")
# seurat_integrated_01 <- readRDS(file = "objects/micro_subset/seurat_integrated_01_clustered.rds")


DefaultAssay(seurat_integrated_01) <- "integrated"

# :: Clustering result ----
svglite(filename = "figures/micro_chimeric_comparison/umap_whole.svg", 
        res = 120, quality = 100,
        width = 1000, height = 1000)

DimPlot(seurat_integrated_01,
        label = TRUE,
        reduction = "umap",
        label.size = 5,
        pt.size = 0.7) +
  ggtitle(paste0("Tri.A4H.WT and Chimeric Male_Clustering at 0.5")) + 
  theme(plot.title = element_text(hjust=0.5, face="bold"))

dev.off()

Idents(seurat_integrated_01) <- "integrated_snn_res.0.5"

svglite(filename = "figures/micro_chimeric_comparison/umap_split.svg", 
        res = 120, quality = 100,
        width = 1800, height = 1000)

DimPlot(seurat_integrated_01,
        label = TRUE,
        reduction = "umap",
        label.size = 5,
        pt.size = 0.7,
        split.by = "type") +
  ggtitle(paste0("Tri.A4H.WT and Chimeric Male_Clustering at 0.5")) + 
  theme(plot.title = element_text(hjust=0.5, face="bold"))

dev.off()


svglite(filename = "figures/micro_chimeric_comparison/umap_split_stim.svg", 
        res = 120, quality = 100,
        width = 1800, height = 1000)

DimPlot(seurat_integrated_01,
        label = TRUE,
        reduction = "umap",
        label.size = 5,
        pt.size = 0.7,
        split.by = "stim") +
  ggtitle(paste0("Tri.A4H.WT and Chimeric Male_Clustering at 0.5")) + 
  theme(plot.title = element_text(hjust=0.5, face="bold"))

dev.off()



svglite(filename = "figures/micro_chimeric_comparison/umap_split_line.svg", 
        res = 120, quality = 100,
        width = 1800, height = 800)

DimPlot(seurat_integrated_01,
        label = TRUE,
        reduction = "umap",
        label.size = 5,
        pt.size = 0.7,
        split.by = "line") +
  ggtitle(paste0("Tri.A4H.WT and Chimeric Male_Clustering at 0.5")) + 
  theme(plot.title = element_text(hjust=0.5, face="bold"))

dev.off()


# :: FeaturePlots
# All marker genes for MHCII, Interferon, DAMs, Inflammatory Chemokines, Canonical MGs
svglite(filename = "figures/micro_chimeric_comparison/featureplot_all.svg",
        res = 100, quality = 100, width = 2000, height = 2000)
FeaturePlot(seurat_integrated_01, reduction="umap",
            features = c("HLA-DRB1", "HLA-DPB1","HLA-DQA1", "CD74", "IFI6", "IRF7",
                         "ISG15", "STAT1", "IFIT3", "CD9", "TREM2", "LPL", "ITGAX",
                         "CCL2", "CXCL10", "CCL8", "CXCL11","P2RY12", "P2RY13", "CX3CR1", "TMEM119"),
            label = TRUE, min.cutoff = "q5")

dev.off()



#_----
# 4. Compare Tri MG with AD Chimeric ----
# Integrate WT microglia subset with Chimeric AD microglia (Male and Female)

# Call seurat mg wt and check (should be 909)
# seurat_mg_WT <- readRDS(file = "objects/micro_subset/seurat_mg_WT.rds")
ncol(seurat_mg_WT)   # 909


# :: Chimeric data ----
# WT male and FAD male objects
# Call the data
# seurat.list_chimeric <- readRDS("../01. Single Cell Project/1st Try G10 H10 IMGL/objects/seurat objects/seurat.list.rds")

View(names(seurat.list_chimeric))
chimeric_FAD_male <- seurat.list_chimeric[[11]]
chimeric_FAD_female <- seurat.list_chimeric[[12]]

# delete the big list for memory
rm(seurat.list_chimeric)

# Get the datasets ready for Integration
# Integration of final 100k WT microglia and Chimeric male WT and FAD
# look into data ready for integration
head(seurat_mg_WT@meta.data)
head(chimeric_FAD_female@meta.data)
head(chimeric_FAD_male@meta.data)


names(seurat_mg_WT@meta.data)
names(chimeric_FAD_female@meta.data)

# add type metadata to Tri
seurat_mg_WT$type <- "TriCulture"


# :: Integration ----
# because Chimeric data was pre-processed in the traditional way, it needs to be processed by SCTransform
# :: SCT Pre-process for Chimeric data
chimeric.list <- list(chimeric_FAD_female, chimeric_FAD_male)
names(chimeric.list) <- c("Chimeric_FAD_F", "Chimeric_FAD_M")

# Automated Normalize, CellCycleScoring, and SCTransfor (with regressing out mt content)
for (i in 1:length(chimeric.list)) {
  chimeric.list[[i]] <- NormalizeData(chimeric.list[[i]], verbose = TRUE)
  chimeric.list[[i]] <- CellCycleScoring(chimeric.list[[i]], g2m.features=g2m_genes, s.features=s_genes)
  chimeric.list[[i]] <- SCTransform(chimeric.list[[i]], vars.to.regress = c("percent.mt"))
}


# Check which assays are stored in objects (All need to show both RNA and SCT)
for(i in 1:length(chimeric.list)){
  print(names(chimeric.list[[i]]@assays))
}

# save permanent (pre-processed seurat)
saveRDS(chimeric.list, file = "objects/chimeric.AD.list_sct_processed.rds")
# chimeric.list <- readRDS(file = "objects/chimeric.AD.list_sct_processed.rds")

ad.seurat.tri.chimeric <- list(seurat_mg_WT, chimeric.list[[1]], chimeric.list[[2]])
names(ad.seurat.tri.chimeric) <- c("Tri.100k.Microglia", names(chimeric.list))
View(ad.seurat.tri.chimeric)


# TriCulture MG stim to AD (WT meant Non-TR2, but the disease condition is AD because the cell line is A4H)
ad.seurat.tri.chimeric[[1]]$stim <- "AD"

# Using SCTransform object as input
# Select the most variable features to use for integration (default: nfeatures = 2000)
integ_features_02 <- SelectIntegrationFeatures(object.list = ad.seurat.tri.chimeric, 
                                               nfeatures = 3000)

# Prepare the SCT list object for integration
ad.seurat.tri.chimeric <- PrepSCTIntegration(object.list = ad.seurat.tri.chimeric,
                                             anchor.features = integ_features_02)


# Find best buddies - can take a while to run
integ_anchors_02 <- FindIntegrationAnchors(object.list = ad.seurat.tri.chimeric,
                                           normalization.method = "SCT",
                                           anchor.features = integ_features_02)


options(future.globals.maxSize = 12000 * 1024^8)

# Integrate across conditions
seurat_integrated_02 <- IntegrateData(anchorset = integ_anchors_02, 
                                      normalization.method = "SCT")

# save
saveRDS(seurat_integrated_02, file = "objects/micro_subset_02/seurat_integrated_ad.micro_ad.chimeric.rds")
# seurat_integrated_02 <- readRDS(file = "objects/micro_subset_02/seurat_integrated_ad.micro_ad.chimeric.rds")




# :: Reductions ----
# Run PCA
seurat_integrated_02 <- RunPCA(object = seurat_integrated_02)

# Plot PCA
Idents(seurat_integrated_02) <- "type"

svglite(filename = "figures/micro_chimeric_comparison_AD only/pca.svg", res = 100, quality = 100,
        width = 1000, height = 800)

PCAPlot(seurat_integrated_02, split.by = "type")  + 
  ggtitle("PCA_TriCulture vs. Chimeric MG") + 
  theme(axis.text.x = element_blank()) +
  # theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=0.5)) +
  theme(plot.title = element_text(hjust=0.5, face="bold"))

dev.off()


# Run UMAP and TSNE
seurat_integrated_02 <- RunUMAP(seurat_integrated_02, 
                                dims = 1:40,
                                reduction = "pca")


seurat_integrated_02 <- RunTSNE(seurat_integrated_02, 
                                dims = 1:40,
                                reduction = "pca")


# Plot UMAP                             
Idents(seurat_integrated_02) <- "type"

# define the umap plot and set the alpha (if type is TriCulture, set alpha = 1, otherwise 0.2)
## this will help the TriCulture dots look through the Chimeric dots
uplot <- DimPlot(seurat_integrated_02, group.by = "type", cols = c("light gray", "red")) 
uplot[[1]]$layers[[1]]$aes_params$alpha = ifelse (seurat_integrated_02@meta.data$type == "TriCulture", 1, .2 )

svglite(filename = "figures/micro_chimeric_comparison_AD only/umap_by_type_not_clustered_grouped.svg",
        res = 120, quality = 100,
        width = 1000, height = 1100)
uplot
dev.off()

svglite(filename = "figures/micro_chimeric_comparison_AD only/umap_by_type_not_clustered.svg",
        res = 120, quality = 100,
        width = 1200, height = 800)
DimPlot(seurat_integrated_02, split.by = "type")
dev.off()

TSNEPlot(seurat_integrated_02, split.by = "type")
DimPlot(seurat_integrated_02, split.by = "line")

dim(seurat_integrated_02)

# :: Clustering ----
# Determine the K-nearest neighbor graph
seurat_integrated_02 <- FindNeighbors(object = seurat_integrated_02, dims = 1:40)


# Determine the clusters for various resolutions                                
seurat_integrated_02 <- FindClusters(object = seurat_integrated_02, resolution = 0.5)


# save 
saveRDS(seurat_integrated_02, file = "objects/micro_subset/seurat_integrated_ad.micro_ad.chimeric_clustered.rds")
# seurat_integrated_02 <- readRDS(file = "objects/micro_subset/seurat_integrated_02_clustered.rds")


DefaultAssay(seurat_integrated_02) <- "integrated"

# :: Clustering result ----
svglite(filename = "figures/micro_chimeric_comparison_AD only/umap_whole.svg", 
        res = 120, quality = 100,
        width = 1000, height = 1000)

DimPlot(seurat_integrated_02,
        label = TRUE,
        reduction = "umap",
        label.size = 5,
        pt.size = 0.7) +
  ggtitle(paste0("Tri.A4H.Non-TR2 and Chimeric AD_Clustering at 0.5")) + 
  theme(plot.title = element_text(hjust=0.5, face="bold"))

dev.off()

Idents(seurat_integrated_02) <- "integrated_snn_res.0.5"

svglite(filename = "figures/micro_chimeric_comparison_AD only/umap_split.svg", 
        res = 120, quality = 100,
        width = 1800, height = 1000)

DimPlot(seurat_integrated_02,
        label = TRUE,
        reduction = "umap",
        label.size = 5,
        pt.size = 0.7,
        split.by = "type") +
  ggtitle(paste0("Tri.A4H.Non-TR2 and Chimeric AD_Clustering at 0.5")) + 
  theme(plot.title = element_text(hjust=0.5, face="bold"))

dev.off()



svglite(filename = "figures/micro_chimeric_comparison_AD only/umap_split_line.svg", 
        res = 120, quality = 100,
        width = 1800, height = 800)

DimPlot(seurat_integrated_02,
        label = TRUE,
        reduction = "umap",
        label.size = 5,
        pt.size = 0.7,
        split.by = "line") +
  ggtitle(paste0("Tri.A4H.WT and Chimeric Male_Clustering at 0.5")) + 
  theme(plot.title = element_text(hjust=0.5, face="bold"))

dev.off()


# :: Cell type ----
# __ FeaturePlots ----
# All marker genes for MHCII, Interferon, DAMs, Inflammatory Chemokines, Canonical MGs
DefaultAssay(seurat_integrated_02) <- "RNA"

svglite(filename = "figures/micro_chimeric_comparison_AD only/featureplot_all.svg",
        res = 100, quality = 100, width = 2000, height = 2000)
FeaturePlot(seurat_integrated_02, reduction="umap",
            features = c("HLA-DRB1", "HLA-DPB1","HLA-DQA1", "CD74", "IFI6", "IRF7",
                         "ISG15", "STAT1", "IFIT3", "CD9", "TREM2", "LPL", "ITGAX",
                         "CCL2", "CXCL10", "CCL8", "CXCL11","P2RY12", "P2RY13", "CX3CR1", "TMEM119"),
            label = TRUE, min.cutoff = "q5")

dev.off()


# __ Violin plot----
violin_cell_type <- function(x, genes, pt){
  VlnPlot(x, features = genes, 
          assay = "RNA", 
          slot = "data",
          pt.size = pt, 
          ncol = 1)
}



svglite(filename = "figures/micro_chimeric_comparison_AD only/violin_gene_all.svg",
        res = 100, quality = 100, width = 1000, height = 5000)

violin_cell_type(seurat_integrated_02, c("HLA-DRB1", "HLA-DPB1","HLA-DQA1", "CD74", "IFI6", "IRF7",
                                         "ISG15", "STAT1", "IFIT3", "CD9", "TREM2", "LPL", "ITGAX",
                                         "CCL2", "CXCL10", "CCL8", "CXCL11","P2RY12", "P2RY13", "CX3CR1", "TMEM119"), 0)
dev.off()


# VlnPlots based on SCT residual
violin_cell_type_sct <- function(x, genes, pt){
  VlnPlot(x, features = genes, 
          assay = "SCT", 
          slot = "scale.data",
          pt.size = pt, 
          ncol = 1)
}

svglite(filename = "figures/micro_chimeric_comparison_AD only/violin_gene_all_sct.svg",
        res = 100, quality = 100, width = 1000, height = 5000)

violin_cell_type_sct(seurat_integrated_02, c("HLA-DRB1", "HLA-DPB1","HLA-DQA1", "CD74", "IFI6", "IRF7",
                                             "ISG15", "STAT1", "IFIT3", "CD9", "TREM2", "LPL", "ITGAX",
                                             "CCL2", "CXCL10", "CCL8", "CXCL11","P2RY12", "P2RY13", "CX3CR1", "TMEM119"), 0.1)
dev.off()


# __ Avg expression----
# Average expression level by genes 
?AverageExpression
avg.exp <- AverageExpression(object = seurat_integrated_02,
                             assays = "RNA",
                             features = c("HLA-DRB1", "HLA-DPB1","HLA-DQA1", "CD74", "IFI6", "IRF7",
                                          "ISG15", "STAT1", "IFIT3", "CD9", "TREM2", "LPL", "ITGAX",
                                          "CCL2", "CXCL10", "CCL8", "CXCL11","P2RY12", "P2RY13", "CX3CR1", "TMEM119"),
                             slot = "data")


# export to xlsx
library(xlsx)
write.xlsx(avg.exp[["RNA"]], 
           file = "results/microglia_sub/vs. AD Chimeric/average expression of marker genes.xlsx",
           col.names = TRUE,
           row.names = TRUE)

# convert to a data frame
library(tibble)
avg.exp_df <- as.data.frame(avg.exp[["RNA"]]) %>% rownames_to_column(var = "gene")
View(avg.exp_df)



# __ DotPlot----
# dotplot scale by size
my_color <- c('yellow', 'red')
dot <- function(obj, gene, title){
  DotPlot(obj, features = gene, dot.scale = 6, cols = my_color, scale.by = "size") +
    RotatedAxis() +
    labs(title = title) +
    theme(plot.title = element_text(hjust = 0.5))  +
    theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=0.5))
}


svglite(filename = "figures/micro_chimeric_comparison_AD only/dot_all_genes.svg",
        res = 100, quality = 100, width = 1500, height = 1400)

dot(obj = seurat_integrated_02, 
    gene = c("HLA-DRB1", "HLA-DPB1","HLA-DQA1", "CD74", "IFI6", "IRF7",
             "ISG15", "STAT1", "IFIT3", "CD9", "TREM2", "LPL", "ITGAX",
             "CCL2", "CXCL10", "CCL8", "CXCL11","P2RY12", "P2RY13", "CX3CR1", "TMEM119"),
    title = "DotPlot by Size")

dev.off()



#_----
# 5. Compare Tri MG with AD Chimeric (ver2)----
# :: This way looks correct ====
# SCT pre-process on the subsetted microglia as well

# :: Integration ----
# because Chimeric data was pre-processed in the traditional way, it needs to be processed by SCTransform
# :: SCT Pre-process for Chimeric data
tri.chimeric.list <- list(seurat_mg_WT, chimeric_FAD_female, chimeric_FAD_male)
names(tri.chimeric.list) <- c("A4H MG", "Chimeric_FAD_F", "Chimeric_FAD_M")

# Automated Normalize, CellCycleScoring, and SCTransfor (with regressing out mt content)
for (i in 1:length(tri.chimeric.list)) {
  tri.chimeric.list[[i]] <- NormalizeData(tri.chimeric.list[[i]], verbose = TRUE)
  tri.chimeric.list[[i]] <- CellCycleScoring(tri.chimeric.list[[i]], g2m.features=g2m_genes, s.features=s_genes)
  tri.chimeric.list[[i]] <- SCTransform(tri.chimeric.list[[i]], vars.to.regress = c("percent.mt"))
}


# Check which assays are stored in objects (All need to show both RNA and SCT)
for(i in 1:length(tri.chimeric.list)){
  print(names(tri.chimeric.list[[i]]@assays))
}

# save permanent (pre-processed seurat)
saveRDS(tri.chimeric.list, file = "objects/tri.chimeric.AD.list_all.sct_processed.rds")
# tri.chimeric.list <- readRDS(file = "objects/tri.chimeric.AD.list_all.sct_processed.rds")



# TriCulture MG stim to AD (WT meant Non-TR2, but the disease condition is AD because the cell line is A4H)
names(tri.chimeric.list[1])
tri.chimeric.list[[1]]$stim <- "AD"

# Using SCTransform object as input
# Select the most variable features to use for integration (default: nfeatures = 2000)
integ_features_03 <- SelectIntegrationFeatures(object.list = tri.chimeric.list, 
                                               nfeatures = 3000)

# Prepare the SCT list object for integration
tri.chimeric.list_03 <- PrepSCTIntegration(object.list = tri.chimeric.list,
                                           anchor.features = integ_features_03)


# Find best buddies - can take a while to run
integ_anchors_03 <- FindIntegrationAnchors(object.list = tri.chimeric.list_03,
                                           normalization.method = "SCT",
                                           anchor.features = integ_features_03)


options(future.globals.maxSize = 12000 * 1024^8)

# Integrate across conditions
seurat_integrated_03 <- IntegrateData(anchorset = integ_anchors_03, 
                                      normalization.method = "SCT")

# save
saveRDS(seurat_integrated_03, file = "objects/micro_subset_03/seurat_integrated_03.rds")
# seurat_integrated_03 <- readRDS(file = "objects/micro_subset_03/seurat_integrated_03.rds")




# :: Reductions ----
# Run PCA
seurat_integrated_03 <- RunPCA(object = seurat_integrated_03)

# Plot PCA
Idents(seurat_integrated_03) <- "type"

svglite(filename = "figures/micro_chimeric_comparison_AD_all_sct/pca.svg", res = 100, quality = 100,
        width = 1000, height = 800)

PCAPlot(seurat_integrated_03, split.by = "type")  + 
  ggtitle("PCA_TriCulture vs. Chimeric MG") + 
  theme(axis.text.x = element_blank()) +
  # theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=0.5)) +
  theme(plot.title = element_text(hjust=0.5, face="bold"))

dev.off()


# Run UMAP and TSNE
seurat_integrated_03 <- RunUMAP(seurat_integrated_03, 
                                dims = 1:40,
                                reduction = "pca")


seurat_integrated_03 <- RunTSNE(seurat_integrated_03, 
                                dims = 1:40,
                                reduction = "pca")


# Plot UMAP                             
Idents(seurat_integrated_03) <- "type"

# define the umap plot and set the alpha (if type is TriCulture, set alpha = 1, otherwise 0.2)
## this will help the TriCulture dots look through the Chimeric dots
uplot_03 <- DimPlot(seurat_integrated_03, group.by = "type", cols = c("light gray", "red")) 
uplot_03[[1]]$layers[[1]]$aes_params$alpha = ifelse (seurat_integrated_03@meta.data$type == "TriCulture", 1, .2 )

svglite(filename = "figures/micro_chimeric_comparison_AD_all_sct/umap_by_type_not_clustered_grouped.svg",
        res = 120, quality = 100,
        width = 1000, height = 1100)
uplot_03
dev.off()

svglite(filename = "figures/micro_chimeric_comparison_AD_all_sct/umap_by_type_not_clustered.svg",
        res = 120, quality = 100,
        width = 1200, height = 800)
DimPlot(seurat_integrated_03, split.by = "type")
dev.off()

TSNEPlot(seurat_integrated_03, split.by = "type")
DimPlot(seurat_integrated_03, split.by = "line")

dim(seurat_integrated_03)

# :: Clustering ----
# Determine the K-nearest neighbor graph
seurat_integrated_03 <- FindNeighbors(object = seurat_integrated_03, dims = 1:40)


# Determine the clusters for various resolutions                                
seurat_integrated_03 <- FindClusters(object = seurat_integrated_03, resolution = 0.5)


# save 
saveRDS(seurat_integrated_03, file = "objects/micro_subset_03/seurat_integrated_03_clustered.rds")
# seurat_integrated_03 <- readRDS(file = "objects/micro_subset_03/seurat_integrated_03_clustered.rds")


DefaultAssay(seurat_integrated_03) <- "integrated"

# :: Clustering result ----
svglite(filename = "figures/micro_chimeric_comparison_AD_all_sct/umap_whole.svg", 
        res = 120, quality = 100,
        width = 1000, height = 1000)

DimPlot(seurat_integrated_03,
        label = TRUE,
        reduction = "umap",
        label.size = 5,
        pt.size = 0.7) +
  ggtitle(paste0("Tri.A4H.Non-TR2 and Chimeric AD_Clustering at 0.5")) + 
  theme(plot.title = element_text(hjust=0.5, face="bold"))

dev.off()

Idents(seurat_integrated_03) <- "integrated_snn_res.0.5"

svglite(filename = "figures/micro_chimeric_comparison_AD_all_sct/umap_split.svg", 
        res = 120, quality = 100,
        width = 1800, height = 1000)

DimPlot(seurat_integrated_03,
        label = TRUE,
        reduction = "umap",
        label.size = 5,
        pt.size = 0.7,
        split.by = "type") +
  ggtitle(paste0("Tri.A4H.Non-TR2 and Chimeric AD_Clustering at 0.5")) + 
  theme(plot.title = element_text(hjust=0.5, face="bold"))

dev.off()



svglite(filename = "figures/micro_chimeric_comparison_AD_all_sct/umap_split_line.svg", 
        res = 120, quality = 100,
        width = 1800, height = 800)

DimPlot(seurat_integrated_03,
        label = TRUE,
        reduction = "umap",
        label.size = 5,
        pt.size = 0.7,
        split.by = "line") +
  ggtitle(paste0("Tri.A4H.WT and Chimeric Male_Clustering at 0.5")) + 
  theme(plot.title = element_text(hjust=0.5, face="bold"))

dev.off()


# :: Cell type ----
# __ FeaturePlots ----
# All marker genes for MHCII, Interferon, DAMs, Inflammatory Chemokines, Canonical MGs
DefaultAssay(seurat_integrated_03) <- "RNA"

svglite(filename = "figures/micro_chimeric_comparison_AD_all_sct/featureplot_all.svg",
        res = 100, quality = 100, width = 2000, height = 2000)
FeaturePlot(seurat_integrated_03, reduction="umap",
            features = c("HLA-DRB1", "HLA-DPB1","HLA-DQA1", "CD74", "IFI6", "IRF7",
                         "ISG15", "STAT1", "IFIT3", "CD9", "TREM2", "LPL", "ITGAX",
                         "CCL2", "CXCL10", "CCL8", "CXCL11","P2RY12", "P2RY13", "CX3CR1", "TMEM119"),
            label = TRUE, min.cutoff = "q5")

dev.off()


# __ Violin plot----
violin_cell_type <- function(x, genes, pt){
  VlnPlot(x, features = genes, 
          assay = "RNA", 
          slot = "data",
          pt.size = pt, 
          ncol = 1)
}



svglite(filename = "figures/micro_chimeric_comparison_AD_all_sct/violin_gene_all.svg",
        res = 100, quality = 100, width = 1000, height = 5000)

violin_cell_type(seurat_integrated_03, c("HLA-DRB1", "HLA-DPB1","HLA-DQA1", "CD74", "IFI6", "IRF7",
                                         "ISG15", "STAT1", "IFIT3", "CD9", "TREM2", "LPL", "ITGAX",
                                         "CCL2", "CXCL10", "CCL8", "CXCL11","P2RY12", "P2RY13", "CX3CR1", "TMEM119"), 0)
dev.off()


# VlnPlots based on SCT residual
violin_cell_type_sct <- function(x, genes, pt){
  VlnPlot(x, features = genes, 
          assay = "SCT", 
          slot = "scale.data",
          pt.size = pt, 
          ncol = 1)
}

svglite(filename = "figures/micro_chimeric_comparison_AD_all_sct/violin_gene_all_sct.svg",
        res = 100, quality = 100, width = 1000, height = 5000)

violin_cell_type_sct(seurat_integrated_03, c("HLA-DRB1", "HLA-DPB1","HLA-DQA1", "CD74", "IFI6", "IRF7",
                                             "ISG15", "STAT1", "IFIT3", "CD9", "TREM2", "LPL", "ITGAX",
                                             "CCL2", "CXCL10", "CCL8", "CXCL11","P2RY12", "P2RY13", "CX3CR1", "TMEM119"), 0.1)
dev.off()


# __ DotPlot----
# dotplot scale by size
my_color <- c('yellow', 'red')
dot <- function(obj, gene, title){
  DotPlot(obj, features = gene, dot.scale = 6, cols = my_color, scale.by = "size") +
    RotatedAxis() +
    labs(title = title) +
    theme(plot.title = element_text(hjust = 0.5))  +
    theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=0.5))
}


svglite(filename = "figures/micro_chimeric_comparison_AD_all_sct/dot_all_genes.svg",
        res = 100, quality = 100, width = 1500, height = 1400)

dot(obj = seurat_integrated_03, 
    gene = c("HLA-DRB1", "HLA-DPB1","HLA-DQA1", "CD74", "IFI6", "IRF7",
             "ISG15", "STAT1", "IFIT3", "CD9", "TREM2", "LPL", "ITGAX",
             "CCL2", "CXCL10", "CCL8", "CXCL11","P2RY12", "P2RY13", "CX3CR1", "TMEM119"),
    title = "DotPlot by Size")

dev.off()


# rename the clusters
seurat_integrated_03 <- RenameIdents(object = seurat_integrated_03, 
                                     "0" = "Canonical MGs",
                                     "1" = "DAMs",
                                     "2" = "Canonical MGs",
                                     "3" = "Canonical MGs",
                                     "4" = "MHCII",
                                     "5" = "MHCII",
                                     "6" = "Interferon",
                                     "7" = "Canonical MGs",
                                     "8" = "Chemokines",
                                     "9" = "Canonical MGs",
                                     "10" = "Canonical MGs",
                                     "11" = "MHCII")



DefaultAssay(seurat_integrated_03) <- "integrated"

# Plotting again
# whole

svglite(filename = "figures/micro_chimeric_comparison_AD_all_sct/umap_annotated.svg", res = 100, quality = 100,
        width = 1000, height = 800)
DimPlot(seurat_integrated_03, label = TRUE, pt.size = 0.7)
dev.off()

# split by type
svglite(filename = "figures/micro_chimeric_comparison_AD_all_sct/umap_split_annotated.svg", res = 100, quality = 100,
        width = 2000, height = 1000)

DimPlot(seurat_integrated_03, label = TRUE, pt.size = 0.7, split.by = "type")
dev.off()



# Composition by cell type
table(Idents(seurat_integrated_03))
n_cells <- FetchData(seurat_integrated_03, 
                     vars = c("ident", "type")) %>%
  dplyr::count(ident, type) %>%
  tidyr::spread(ident, n)

View(n_cells)


# composition
comp.chimeric <- t(n_cells[1, 2:6])
colnames(comp.chimeric) <- "Chimeric"
comp.a4h <- t(n_cells[2, 2:6])
colnames(comp.a4h) <- "Tri_A4H"


composition.list <- list(comp.chimeric, comp.a4h)
names(composition.list) <- c("Chimeric", "Tri A4H")

percent.list <- lapply(composition.list, FUN = function(x) {
  x <- round(proportions(x, margin = 2)*100, 1)
})

comp.df_a4h <- cbind(percent.list[[1]], percent.list[[2]])

# Re-arrange
comp.df_a4h <- comp.df_a4h[c(1, 3, 2, 4, 5), ]
View(t(comp.df_a4h))

# _ ----
# [Non-MG_subset Ad-hoc] ====
# https://github.com/dearzoo/06.-Triculture_TR2_ACA/issues/4
# Re-cluster the clusters that are not Microglia
## Cluster 4, 5, 9, 20, 26, 28 and 34


# Subset_All others. Reclustering

# :: subsetting ----
table(Idents(seurat_integrated))
seurat_others <- subset(seurat_integrated, idents = c("4", "5", "9", "20", "26", "28", "34"))
table(Idents(seurat_others))

seurat_others.list <- SplitObject(seurat_others, split.by = "ident")


# :: Pre-process (Active assay = RNA) ----
for(i in 1:length(seurat_others.list)){
  # log normalize in case it's needed
  seurat_others.list[[i]] <- NormalizeData(seurat_others.list[[i]], verbose = TRUE)
  
  # cell cycle effect in case it's needed for quality check
  seurat_others.list[[i]] <- CellCycleScoring(seurat_others.list[[i]], g2m.features=g2m_genes, s.features=s_genes)
  
  # SCTransform
  seurat_others.list[[i]] <- SCTransform(seurat_others.list[[i]], vars.to.regress = c("percent.mt"))
}


# save permanent (pre-processed seurat)
saveRDS(seurat_others.list, file = "objects/non_micro_subset/seurat_others.list.rds")
# seurat_others.list <- readRDS(file = "objects/non_micro_subset/seurat_others.list.rds")


# :: Integration ----

# Using SCTransform object as input
# Select the most variable features to use for integration (default: nfeatures = 2000)
integ_features_others <- SelectIntegrationFeatures(object.list = seurat_others.list, nfeatures = 3000)

# Prepare the SCT list object for integration
seurat_others.list <- PrepSCTIntegration(object.list = seurat_others.list,
                                         anchor.features = integ_features_others)


# Find best buddies - can take a while to run
integ_anchors_others <- FindIntegrationAnchors(object.list = seurat_others.list,
                                               normalization.method = "SCT",
                                               anchor.features = integ_features_others)


# Error occurred due to low cell counts (than the minimum required k.filter)

# manually edit the k.filter parameter and re-do the FindIntegrationAnchors
k.filter <- min(sapply(seurat_others.list, ncol))

integ_anchors_others <- FindIntegrationAnchors(object.list = seurat_others.list,
                                               normalization.method = "SCT",
                                               anchor.features = integ_features_others, 
                                               k.filter = k.filter)



saveRDS(integ_anchors_others, file = "objects/non_micro_subset/integ_anchors_others.rds")

# Before integration, manage the memory
options(future.globals.maxSize = 12000 * 1024^8)

# Integrate across conditions
seurat_integrated_others <- IntegrateData(anchorset = integ_anchors_others, normalization.method = "SCT")


# :: Reduction ----
# Run PCA
seurat_integrated_others <- RunPCA(object = seurat_integrated_others)


# Plot PCA
# svglite(filename = "figures/subset analysis/non_micro/others_pca.svg", res = 100, quality = 100,
#      width = 2000, height = 800)

Idents(seurat_integrated_others) <- "line"
PCAPlot(seurat_integrated_others, split.by = "line", pt.size = 1)  + 
  ggtitle("Non_microglia Subset PCA by Sample") + 
  theme(axis.text.x = element_blank()) +
  # theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=0.5)) +
  theme(plot.title = element_text(hjust=0.5, face="bold"))

# dev.off()

# Run UMAP and TSNE
seurat_integrated_others <- RunUMAP(seurat_integrated_others, 
                                    dims = 1:40,
                                    reduction = "pca")


seurat_integrated_others <- RunTSNE(seurat_integrated_others, 
                                    dims = 1:40,
                                    reduction = "pca")


# Plot UMAP                             
Idents(seurat_integrated_others) <- "integrated_snn_res.0.4"
DimPlot(seurat_integrated_others)
TSNEPlot(seurat_integrated_others)


# Idents(seurat_integrated_others) <- "line"
# Idents(seurat_integrated_others) <- "orig.ident"
# 
# svglite(filename = "figures/subset analysis/microglia_subset_umap_by_line.svg",
#      width = 2000, height = 800,
#      res = 110, quality = 100)
# DimPlot(seurat_integrated_others, split.by = "line", pt.size = 1)
# dev.off()


# :: Clustering ----
# Determine the K-nearest neighbor graph
seurat_integrated_others <- FindNeighbors(object = seurat_integrated_others, dims = 1:40)


# Determine the clusters for various resolutions                                
seurat_integrated_others <- FindClusters(object = seurat_integrated_others, 
                                         resolution = c(0.4, 0.6, 0.8, 1.0, 1.4))

# save 
saveRDS(seurat_integrated_others, file = "objects/non_micro_subset/seurat_integrated_others_clustered.rds")
# seurat_integrated_others <- readRDS(file = "objects/non_micro_subset/seurat_integrated_others_clustered.rds")


View(names(seurat_integrated_others@meta.data))

res.others.list <- list(names(seurat_integrated_others@meta.data[15]),
                        names(seurat_integrated_others@meta.data[16]),
                        names(seurat_integrated_others@meta.data[17]),
                        names(seurat_integrated_others@meta.data[18]),
                        names(seurat_integrated_others@meta.data[19]))



table(seurat_integrated_others$integrated_snn_res.0.4)    # 7
table(seurat_integrated_others$integrated_snn_res.0.6)    # 8
table(seurat_integrated_others$integrated_snn_res.0.8)    # 10
table(seurat_integrated_others$integrated_snn_res.1)      # 14
table(seurat_integrated_others$integrated_snn_res.1.4)    # 13

Idents(seurat_integrated_others) <- "integrated_snn_res.0.4"
DimPlot(seurat_integrated_others, label = TRUE, pt.size = 0.8)


# :: Cell type identification ----
DefaultAssay(seurat_integrated_others) <- "RNA"

# :: FeaturePlot----
DefaultAssay(seurat_integrated_others_others) <- "RNA"


fplot_others.list <- list(fplot(seurat_integrated_others, m.astro, 3),
                          fplot(seurat_integrated_others, m.neurons, 3),
                          fplot(seurat_integrated_others, m.oligo, 3),
                          fplot(seurat_integrated_others, m.opcs, 2))

names(fplot_others.list) <- c("m.astro",
                              "m.neurons",
                              "m.oligo",
                              "m.opcs")


# Print FeaturePlots
print(fplot_others.list[[1]])
print(fplot_others.list[[2]])
print(fplot_others.list[[3]])
print(fplot_others.list[[4]])


# _ ----
# Neuron DEG ----
seurat_nr <- subset(seurat_integrated, idents = "34")

table(seurat_nr$stim)
View(seurat_nr)

Idents(seurat_nr) <- "stim"
de.neuron_stim.rna <- de(seurat_nr, id1 = "TR2", id2 = "WT", assay = "RNA")
View(de.neuron_stim.rna)

library(xlsx)
write.xlsx(de.neuron_stim.rna,
           file = "results/neuron_sub/de_stim_in_neuron.xlsx", 
           col.names = TRUE, row.names = TRUE)



# _ ----